// SPDX-FileCopyrightText: 2017-2022 Carl Zeiss Microscopy GmbH
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#include "include_gtest.h"
#include "MockMetadataSegment.h"

using namespace libCZI;
using namespace std;

// editorconfig-checker-disable
static const char* testMetadata[] =
{
R"(<?xml version="1.0"?>)",
R"(<ImageDocument>)",
 R"(<Metadata>)",
  R"(<HardwareSetting>)",
   R"(<Microscope Id="Microscope" Name="Axio Scan.Z1" UniqueName="Pollux.Stand" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<StandSpecification>FixedStage, Pollux</StandSpecification>)",
    R"(<Devices>)",
     R"(<DeviceRef Id="MTBMicroscopeManager" />)",
     R"(<DeviceRef Id="MTBObject" />)",
     R"(<DeviceRef Id="MTBPulseGenerator" />)",
     R"(<DeviceRef Id="MTBSlideScanner" />)",
     R"(<DeviceRef Id="MTBFocus" />)",
     R"(<DeviceRef Id="MTBStage" />)",
     R"(<DeviceRef Id="MTBStageAxisX" />)",
     R"(<DeviceRef Id="MTBStageAxisY" />)",
     R"(<DeviceRef Id="MTBTrigger" />)",
     R"(<DeviceRef Id="MTBRLPrescanLamp" />)",
     R"(<DeviceRef Id="MTBTLPrescanLamp" />)",
     R"(<DeviceRef Id="MTBTLHalogenLamp" />)",
     R"(<DeviceRef Id="MTBObjectiveChanger" />)",
     R"(<DeviceRef Id="MTBReflectorChanger" />)",
     R"(<DeviceRef Id="MTBFLLEDController" />)",
     R"(<DeviceRef Id="MTBLED4" />)",
     R"(<DeviceRef Id="MTBHXP120Shutter" />)",
     R"(<DeviceRef Id="MTBHXPLamp" />)",
     R"(<DeviceRef Id="MTBCondenserContrastChanger" />)",
     R"(<DeviceRef Id="MTBTLShutter" />)",
     R"(<DeviceRef Id="MTBCameraAdapter_MTBCameraPortChanger_Front" />)",
     R"(<DeviceRef Id="MTBCameraAdapter_MTBCameraPortChanger_Left" />)",
     R"(<DeviceRef Id="MTBCameraPortChanger" />)",
     R"(<DeviceRef Id="MTBLoaderDoor" />)",
     R"(<DeviceRef Id="MTBTrayChanger" />)",
     R"(<DeviceRef Id="MTBObject" />)",
     R"(<DeviceRef Id="Condenser" />)",
     R"(<DeviceRef Id="CameraUnmounted0.AxioCamICAVTGuppy" />)",
     R"(<DeviceRef Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" />)",
     R"(<DeviceRef Id="MTBCamera_MTBCameraPortChanger_Front.MILCamHitachi_FD202SCL" />)",
     R"(<DeviceRef Id="Autofocus" />)",
     R"(<DeviceRef Id="IOCard" />)",
    R"(</Devices>)",
    R"(<ObservationPath>)",
     R"(<LightPathNode Id="MTBObject">)",
      R"(<Successors>)",
       R"(<LightPathNode Id="MTBObjectiveChanger">)",
        R"(<Successors>)",
         R"(<LightPathNode Id="MTBReflectorChanger">)",
          R"(<Successors>)",
           R"(<LightPathNode Id="MTBCameraPortChanger">)",
            R"(<Successors>)",
             R"(<LightPathNode Id="MTBCameraAdapter_MTBCameraPortChanger_Front">)",
              R"(<Successors>)",
               R"(<LightPathNode Id="MTBCamera_MTBCameraPortChanger_Front.MILCamHitachi_FD202SCL">)",
                R"(<Successors />)",
               R"(</LightPathNode>)",
              R"(</Successors>)",
             R"(</LightPathNode>)",
             R"(<LightPathNode Id="MTBCameraAdapter_MTBCameraPortChanger_Left">)",
              R"(<Successors>)",
               R"(<LightPathNode Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C">)",
                R"(<Successors />)",
               R"(</LightPathNode>)",
              R"(</Successors>)",
             R"(</LightPathNode>)",
            R"(</Successors>)",
           R"(</LightPathNode>)",
          R"(</Successors>)",
         R"(</LightPathNode>)",
        R"(</Successors>)",
       R"(</LightPathNode>)",
      R"(</Successors>)",
     R"(</LightPathNode>)",
    R"(</ObservationPath>)",
    R"(<ReflectedLightPath>)",
     R"(<LightPathNode Id="MTBObject">)",
      R"(<Successors>)",
       R"(<LightPathNode Id="MTBObjectiveChanger">)",
        R"(<Successors>)",
         R"(<LightPathNode Id="MTBReflectorChanger">)",
          R"(<Successors>)",
           R"(<LightPathNode Id="MTBFLLEDController">)",
            R"(<Successors>)",
             R"(<LightPathNode Id="MTBLED4">)",
              R"(<Successors />)",
             R"(</LightPathNode>)",
             R"(<LightPathNode Id="MTBHXP120Shutter">)",
              R"(<Successors>)",
               R"(<LightPathNode Id="MTBHXPLamp">)",
                R"(<Successors />)",
               R"(</LightPathNode>)",
              R"(</Successors>)",
             R"(</LightPathNode>)",
            R"(</Successors>)",
           R"(</LightPathNode>)",
          R"(</Successors>)",
         R"(</LightPathNode>)",
        R"(</Successors>)",
       R"(</LightPathNode>)",
      R"(</Successors>)",
     R"(</LightPathNode>)",
    R"(</ReflectedLightPath>)",
    R"(<TransmittedLightPath>)",
     R"(<LightPathNode Id="MTBObject">)",
      R"(<Successors>)",
       R"(<LightPathNode Id="MTBTLShutter">)",
        R"(<Successors>)",
         R"(<LightPathNode Id="Condenser">)",
          R"(<Successors>)",
           R"(<LightPathNode Id="MTBCondenserContrastChanger">)",
            R"(<Successors>)",
             R"(<LightPathNode Id="MTBTLHalogenLamp">)",
              R"(<Successors />)",
             R"(</LightPathNode>)",
            R"(</Successors>)",
           R"(</LightPathNode>)",
          R"(</Successors>)",
         R"(</LightPathNode>)",
        R"(</Successors>)",
       R"(</LightPathNode>)",
      R"(</Successors>)",
     R"(</LightPathNode>)",
    R"(</TransmittedLightPath>)",
    R"(<ObliqueLightPath>)",
     R"(<LightPathNode Id="MTBObject">)",
      R"(<Successors />)",
     R"(</LightPathNode>)",
    R"(</ObliqueLightPath>)",
   R"(</Microscope>)",
   R"(<MicroscopeManager Id="MTBMicroscopeManager" Name="Microscope Manager" UniqueName="Pollux.MicroscopeManager" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
    R"(<ContrastManagerContrastMethods>Unknown</ContrastManagerContrastMethods>)",
    R"(<ContrastManagerModes>Off</ContrastManagerModes>)",
    R"(<LightManagerModes>Off</LightManagerModes>)",
    R"(<IsCondenserFocusingAvailable>false</IsCondenserFocusingAvailable>)",
    R"(<IsCurrentFeedAvailable>false</IsCurrentFeedAvailable>)",
    R"(<IsDazzleProtectionAvailable>false</IsDazzleProtectionAvailable>)",
    R"(<IsLightManagerTemporaryAvailable>false</IsLightManagerTemporaryAvailable>)",
    R"(<IsParfocalCorrectionAvailable>true</IsParfocalCorrectionAvailable>)",
    R"(<IsParfocalCorrectionSilentModeOnOffAvailable>true</IsParfocalCorrectionSilentModeOnOffAvailable>)",
    R"(<IsParcentralCorrectionAvailable>true</IsParcentralCorrectionAvailable>)",
    R"(<IsParcentralCorrectionSilentModeOnOffAvailable>true</IsParcentralCorrectionSilentModeOnOffAvailable>)",
    R"(<IsLoadWorkCouplingOnOffAvailable>false</IsLoadWorkCouplingOnOffAvailable>)",
    R"(<IsVignettingFreeModeTemporaryAvailable>false</IsVignettingFreeModeTemporaryAvailable>)",
    R"(<IsLaserSafetyAvailable>false</IsLaserSafetyAvailable>)",
    R"(<SoftKeyCount>2</SoftKeyCount>)",
    R"(<HasSoftKeyLabeling>true</HasSoftKeyLabeling>)",
    R"(<ContrastManagerContrastMethod Status="Valid">Unknown</ContrastManagerContrastMethod>)",
    R"(<ContrastManagerMode Status="Valid">Off</ContrastManagerMode>)",
    R"(<LightManagerMode Status="Valid">Off</LightManagerMode>)",
    R"(<IsCondenserFocusing Status="Valid">false</IsCondenserFocusing>)",
    R"(<IsCurrentFeed Status="Valid">false</IsCurrentFeed>)",
    R"(<IsDazzleProtection Status="Valid">false</IsDazzleProtection>)",
    R"(<IsLightManagerTemporary Status="Valid">false</IsLightManagerTemporary>)",
    R"(<IsLoadWorkCoupling Status="Valid">false</IsLoadWorkCoupling>)",
    R"(<IsVignettingFreeModeTemporary Status="Valid">false</IsVignettingFreeModeTemporary>)",
    R"(<IsLaserSafety Status="Valid">false</IsLaserSafety>)",
    R"(<IsParfocalCorrection Status="Valid">true</IsParfocalCorrection>)",
    R"(<IsParfocalCorrectionSilentMode Status="Valid">true</IsParfocalCorrectionSilentMode>)",
    R"(<IsParcentralCorrection Status="Valid">true</IsParcentralCorrection>)",
    R"(<IsParcentralCorrectionSilentMode Status="Valid">true</IsParcentralCorrectionSilentMode>)",
    R"(<ParfocalParcentralValues Status="Valid">0|0|0|-180.615|-12.419|-2.724|-11.635|5.502|-1.603|-5.324|5.919|8.951|0|0|0|0|0|0|0|0|0|0|0|0|</ParfocalParcentralValues>)",
   R"(</MicroscopeManager>)",
   R"(<Specimen Id="MTBObject" Name="Object" UniqueName="Object" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false" />)",
   R"(<PulseGenerator Id="MTBPulseGenerator" Name="Pulse Generator" UniqueName="Pollux.PulseGenerator" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Manual" IsLightSource="false" IsLightSink="false" />)",
   R"(<SlideScanner Id="MTBSlideScanner" Name="Slide Scanner" UniqueName="Pollux.SlideScanner" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<SlideScannerPosition Status="Valid">Scan1</SlideScannerPosition>)",
    R"(<SlideScannerWorkingState Status="Valid">Processing</SlideScannerWorkingState>)",
    R"(<SlotLEDState1 Status="Valid">SlotClosed_TrayLoaded</SlotLEDState1>)",
    R"(<SlotLEDState2 Status="Valid">SlotClosed_NoTray</SlotLEDState2>)",
    R"(<SlotLEDState3 Status="Valid">SlotClosed_NoTray</SlotLEDState3>)",
    R"(<SlotLEDState4 Status="Valid">SlotClosed_NoTray</SlotLEDState4>)",
    R"(<SlotLEDState5 Status="Valid">SlotClosed_NoTray</SlotLEDState5>)",
    R"(<SlotLEDState6 Status="Valid">SlotClosed_NoTray</SlotLEDState6>)",
    R"(<SlotLEDState7 Status="Valid">SlotClosed_NoTray</SlotLEDState7>)",
    R"(<SlotLEDState8 Status="Valid">SlotClosed_NoTray</SlotLEDState8>)",
    R"(<SlotLEDState9 Status="Valid">SlotClosed_NoTray</SlotLEDState9>)",
    R"(<SlotLEDState10 Status="Valid">SlotClosed_NoTray</SlotLEDState10>)",
    R"(<SlotLEDState11 Status="Valid">SlotClosed_NoTray</SlotLEDState11>)",
    R"(<SlotLEDState12 Status="Valid">SlotClosed_NoTray</SlotLEDState12>)",
    R"(<SlotLEDState13 Status="Valid">SlotClosed_NoTray</SlotLEDState13>)",
    R"(<SlotLEDState14 Status="Valid">SlotClosed_NoTray</SlotLEDState14>)",
    R"(<SlotLEDState15 Status="Valid">SlotClosed_NoTray</SlotLEDState15>)",
    R"(<SlotLEDState16 Status="Valid">SlotClosed_NoTray</SlotLEDState16>)",
    R"(<SlotLEDState17 Status="Valid">SlotClosed_NoTray</SlotLEDState17>)",
    R"(<SlotLEDState18 Status="Valid">SlotClosed_NoTray</SlotLEDState18>)",
    R"(<SlotLEDState19 Status="Valid">SlotClosed_NoTray</SlotLEDState19>)",
    R"(<SlotLEDState20 Status="Valid">SlotClosed_NoTray</SlotLEDState20>)",
    R"(<SlotLEDState21 Status="Valid">SlotClosed_NoTray</SlotLEDState21>)",
    R"(<SlotLEDState22 Status="Valid">SlotClosed_NoTray</SlotLEDState22>)",
    R"(<SlotLEDState23 Status="Valid">SlotClosed_NoTray</SlotLEDState23>)",
    R"(<SlotLEDState24 Status="Valid">SlotClosed_NoTray</SlotLEDState24>)",
    R"(<SlotLEDState25 Status="Valid">SlotClosed_NoTray</SlotLEDState25>)",
    R"(<SlotNumberOfLoadedTray Status="Valid">1</SlotNumberOfLoadedTray>)",
   R"(</SlideScanner>)",
   R"(<Focus Id="MTBFocus" Name="Motorized Focus" UniqueName="Pollux.Focus_mot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>-5000</MinPosition>)",
    R"(<MaxPosition>5000</MaxPosition>)",
    R"(<MaxDeviation>0.005</MaxDeviation>)",
    R"(<TypicalDeviation>0.005</TypicalDeviation>)",
    R"(<StepWidth>0.01</StepWidth>)",
    R"(<PositionUnit>&#181;m</PositionUnit>)",
    R"(<MinMoveSpeed>0.062</MinMoveSpeed>)",
    R"(<MaxMoveSpeed>6000</MaxMoveSpeed>)",
    R"(<SpeedUnit>&#181;m/s</SpeedUnit>)",
    R"(<HasContinualAcceleration>true</HasContinualAcceleration>)",
    R"(<HasContinualStartSpeed>false</HasContinualStartSpeed>)",
    R"(<MinContinualSpeed>0.062</MinContinualSpeed>)",
    R"(<MaxContinualSpeed>6000</MaxContinualSpeed>)",
    R"(<ContinualSpeedUnit>&#181;m/s</ContinualSpeedUnit>)",
    R"(<MinContinualAcceleration>0</MinContinualAcceleration>)",
    R"(<MaxContinualAcceleration>10000</MaxContinualAcceleration>)",
    R"(<ContinualAccelerationUnit>&#181;m/s&#178;</ContinualAccelerationUnit>)",
    R"(<CoordinateDirection>1</CoordinateDirection>)",
    R"(<HasLoadWork>false</HasLoadWork>)",
    R"(<IsPositiveDirectionAgainstGravity>false</IsPositiveDirectionAgainstGravity>)",
    R"(<IsPositiveDirectionTowardSpecimen>true</IsPositiveDirectionTowardSpecimen>)",
    R"(<Position Status="Valid">2983.376</Position>)",
    R"(<MeasurementPosition Status="Valid">2983.376</MeasurementPosition>)",
    R"(<IsPrecise Status="Valid">false</IsPrecise>)",
    R"(<AxisSetPositionMode Status="Valid">Default</AxisSetPositionMode>)",
    R"(<Speed Status="Valid">0</Speed>)",
    R"(<ContinualSpeed Status="Valid">6000</ContinualSpeed>)",
    R"(<ContinualAcceleration Status="Valid">10000</ContinualAcceleration>)",
   R"(</Focus>)",
   R"(<Stage Id="MTBStage" Name="Motorized Stage" UniqueName="Pollux.Stage" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<StageAxes>)",
     R"(<StageAxis Name="x-Axis" Id="MTBStageAxisX" UniqueName="XAxis" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
      R"(<MinPosition>-350000</MinPosition>)",
      R"(<MaxPosition>350000</MaxPosition>)",
      R"(<MaxDeviation>0.125</MaxDeviation>)",
      R"(<TypicalDeviation>0.125</TypicalDeviation>)",
      R"(<StepWidth>0.25</StepWidth>)",
      R"(<PositionUnit>&#181;m</PositionUnit>)",
      R"(<MinMoveSpeed>0.001</MinMoveSpeed>)",
      R"(<MaxMoveSpeed>25000</MaxMoveSpeed>)",
      R"(<SpeedUnit>&#181;m/s</SpeedUnit>)",
      R"(<HasContinualAcceleration>true</HasContinualAcceleration>)",
      R"(<HasContinualStartSpeed>false</HasContinualStartSpeed>)",
      R"(<MinContinualSpeed>0</MinContinualSpeed>)",
      R"(<MaxContinualSpeed>25000</MaxContinualSpeed>)",
      R"(<ContinualSpeedUnit>&#181;m/s</ContinualSpeedUnit>)",
      R"(<MinContinualAcceleration>0</MinContinualAcceleration>)",
      R"(<MaxContinualAcceleration>500000</MaxContinualAcceleration>)",
      R"(<ContinualAccelerationUnit>&#181;m/s2</ContinualAccelerationUnit>)",
      R"(<CoordinateDirection>1</CoordinateDirection>)",
      R"(<HasLoadWork>false</HasLoadWork>)",
     R"(</StageAxis>)",
     R"(<StageAxis Name="y-Axis" Id="MTBStageAxisY" UniqueName="YAxis" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
      R"(<MinPosition>-350000</MinPosition>)",
      R"(<MaxPosition>350000</MaxPosition>)",
      R"(<MaxDeviation>0.125</MaxDeviation>)",
      R"(<TypicalDeviation>0.125</TypicalDeviation>)",
      R"(<StepWidth>0.25</StepWidth>)",
      R"(<PositionUnit>&#181;m</PositionUnit>)",
      R"(<MinMoveSpeed>0.001</MinMoveSpeed>)",
      R"(<MaxMoveSpeed>25000</MaxMoveSpeed>)",
      R"(<SpeedUnit>&#181;m/s</SpeedUnit>)",
      R"(<HasContinualAcceleration>true</HasContinualAcceleration>)",
      R"(<HasContinualStartSpeed>false</HasContinualStartSpeed>)",
      R"(<MinContinualSpeed>0</MinContinualSpeed>)",
      R"(<MaxContinualSpeed>25000</MaxContinualSpeed>)",
      R"(<ContinualSpeedUnit>&#181;m/s</ContinualSpeedUnit>)",
      R"(<MinContinualAcceleration>0</MinContinualAcceleration>)",
      R"(<MaxContinualAcceleration>500000</MaxContinualAcceleration>)",
      R"(<ContinualAccelerationUnit>&#181;m/s2</ContinualAccelerationUnit>)",
      R"(<CoordinateDirection>1</CoordinateDirection>)",
      R"(<HasLoadWork>false</HasLoadWork>)",
     R"(</StageAxis>)",
    R"(</StageAxes>)",
   R"(</Stage>)",
   R"(<StageAxisX Id="MTBStageAxisX" Name="x-Axis" UniqueName="XAxis" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>-350000</MinPosition>)",
    R"(<MaxPosition>350000</MaxPosition>)",
    R"(<MaxDeviation>0.125</MaxDeviation>)",
    R"(<TypicalDeviation>0.125</TypicalDeviation>)",
    R"(<StepWidth>0.25</StepWidth>)",
    R"(<PositionUnit>&#181;m</PositionUnit>)",
    R"(<MinMoveSpeed>0.001</MinMoveSpeed>)",
    R"(<MaxMoveSpeed>25000</MaxMoveSpeed>)",
    R"(<SpeedUnit>&#181;m/s</SpeedUnit>)",
    R"(<HasContinualAcceleration>true</HasContinualAcceleration>)",
    R"(<HasContinualStartSpeed>false</HasContinualStartSpeed>)",
    R"(<MinContinualSpeed>0</MinContinualSpeed>)",
    R"(<MaxContinualSpeed>25000</MaxContinualSpeed>)",
    R"(<ContinualSpeedUnit>&#181;m/s</ContinualSpeedUnit>)",
    R"(<MinContinualAcceleration>0</MinContinualAcceleration>)",
    R"(<MaxContinualAcceleration>500000</MaxContinualAcceleration>)",
    R"(<ContinualAccelerationUnit>&#181;m/s2</ContinualAccelerationUnit>)",
    R"(<CoordinateDirection>1</CoordinateDirection>)",
    R"(<HasLoadWork>false</HasLoadWork>)",
    R"(<Position Status="Valid">0</Position>)",
    R"(<MeasurementPosition Status="Valid">-885.001</MeasurementPosition>)",
    R"(<IsPrecise Status="Valid">false</IsPrecise>)",
    R"(<AxisSetPositionMode Status="Valid">Default</AxisSetPositionMode>)",
    R"(<Speed Status="Valid">0</Speed>)",
    R"(<ContinualSpeed Status="Valid">100000</ContinualSpeed>)",
    R"(<ContinualAcceleration Status="Valid">200000</ContinualAcceleration>)",
   R"(</StageAxisX>)",
   R"(<StageAxisY Id="MTBStageAxisY" Name="y-Axis" UniqueName="YAxis" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>-350000</MinPosition>)",
    R"(<MaxPosition>350000</MaxPosition>)",
    R"(<MaxDeviation>0.125</MaxDeviation>)",
    R"(<TypicalDeviation>0.125</TypicalDeviation>)",
    R"(<StepWidth>0.25</StepWidth>)",
    R"(<PositionUnit>&#181;m</PositionUnit>)",
    R"(<MinMoveSpeed>0.001</MinMoveSpeed>)",
    R"(<MaxMoveSpeed>25000</MaxMoveSpeed>)",
    R"(<SpeedUnit>&#181;m/s</SpeedUnit>)",
    R"(<HasContinualAcceleration>true</HasContinualAcceleration>)",
    R"(<HasContinualStartSpeed>false</HasContinualStartSpeed>)",
    R"(<MinContinualSpeed>0</MinContinualSpeed>)",
    R"(<MaxContinualSpeed>25000</MaxContinualSpeed>)",
    R"(<ContinualSpeedUnit>&#181;m/s</ContinualSpeedUnit>)",
    R"(<MinContinualAcceleration>0</MinContinualAcceleration>)",
    R"(<MaxContinualAcceleration>500000</MaxContinualAcceleration>)",
    R"(<ContinualAccelerationUnit>&#181;m/s2</ContinualAccelerationUnit>)",
    R"(<CoordinateDirection>1</CoordinateDirection>)",
    R"(<HasLoadWork>false</HasLoadWork>)",
    R"(<Position Status="Valid">-0.048</Position>)",
    R"(<MeasurementPosition Status="Valid">163999.922</MeasurementPosition>)",
    R"(<IsPrecise Status="Valid">false</IsPrecise>)",
    R"(<AxisSetPositionMode Status="Valid">Default</AxisSetPositionMode>)",
    R"(<Speed Status="Valid">0</Speed>)",
    R"(<ContinualSpeed Status="Valid">100000</ContinualSpeed>)",
    R"(<ContinualAcceleration Status="Valid">50000</ContinualAcceleration>)",
   R"(</StageAxisY>)",
   R"(<TriggerCard Id="MTBTrigger" Name="Trigger" UniqueName="Pollux.Trigger" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false" />)",
   R"(<RLPrescanLamp Id="MTBRLPrescanLamp" Name="RL Prescan Lamp" UniqueName="Pollux.RL_PrescanLamp" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
    R"(<MinPosition>0</MinPosition>)",
    R"(<MaxPosition>100</MaxPosition>)",
    R"(<MaxDeviation>0.24437927663734116</MaxDeviation>)",
    R"(<TypicalDeviation>0.24437927663734116</TypicalDeviation>)",
    R"(<StepWidth>0.097751710654936458</StepWidth>)",
    R"(<PositionUnit>%</PositionUnit>)",
    R"(<Has3200K>false</Has3200K>)",
    R"(<HasOnOff>false</HasOnOff>)",
    R"(<LampType>LED</LampType>)",
    R"(<Intensity Status="Valid">0</Intensity>)",
    R"(<LampMode Status="Valid">Manual</LampMode>)",
    R"(<LampActive Status="Valid">true</LampActive>)",
   R"(</RLPrescanLamp>)",
   R"(<TLPrescanLamp Id="MTBTLPrescanLamp" Name="RL Prescan Lamp" UniqueName="Pollux.TL_PrescanLamp" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
    R"(<MinPosition>0</MinPosition>)",
    R"(<MaxPosition>100</MaxPosition>)",
    R"(<MaxDeviation>0.24437927663734116</MaxDeviation>)",
    R"(<TypicalDeviation>0.24437927663734116</TypicalDeviation>)",
    R"(<StepWidth>0.097751710654936458</StepWidth>)",
    R"(<PositionUnit>%</PositionUnit>)",
    R"(<Has3200K>false</Has3200K>)",
    R"(<HasOnOff>false</HasOnOff>)",
    R"(<LampType>LED</LampType>)",
    R"(<Intensity Status="Valid">0</Intensity>)",
    R"(<LampMode Status="Valid">Manual</LampMode>)",
    R"(<LampActive Status="Valid">true</LampActive>)",
   R"(</TLPrescanLamp>)",
   R"(<TransmittedLightHalogenLamp Id="MTBTLHalogenLamp" Name="TL LED Lamp" UniqueName="Pollux.TL_LEDLamp" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
    R"(<MinPosition>0</MinPosition>)",
    R"(<MaxPosition>100</MaxPosition>)",
    R"(<MaxDeviation>0.83612040133779264</MaxDeviation>)",
    R"(<TypicalDeviation>0.83612040133779264</TypicalDeviation>)",
    R"(<StepWidth>0.33444816053511706</StepWidth>)",
    R"(<PositionUnit>%</PositionUnit>)",
    R"(<Has3200K>false</Has3200K>)",
    R"(<HasOnOff>false</HasOnOff>)",
    R"(<LampType>LED</LampType>)",
    R"(<Intensity Status="Valid">0</Intensity>)",
    R"(<LampMode Status="Valid">Manual</LampMode>)",
    R"(<LampActive Status="Valid">true</LampActive>)",
   R"(</TransmittedLightHalogenLamp>)",
   R"(<ObjectiveChanger Id="MTBObjectiveChanger" Name="7x Motorized Nosepiece" UniqueName="Pollux.Nosepiece_7xmot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>7</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<Objective Name="Fluar 2.5x/0.12 M27" Id="" UniqueName="Objective.420120-9900-000" Model="" IsMounted="true" Position="1" Type="Objective">)",
      R"(<ContrastMethods>BrightField, DarkField, RLBrightField, RLPol, TLPol</ContrastMethods>)",
      R"(<Features>Pol</Features>)",
      R"(<Immersions>Air</Immersions>)",
      R"(<Magnification>2.5</Magnification>)",
      R"(<NumericalAperture>0.12</NumericalAperture>)",
      R"(<WorkingDistance>6300</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="Plan-Apochromat 10x/0.45 M27" Id="" UniqueName="Objective.420640-9900-000" Model="" IsMounted="true" Position="2" Type="Objective">)",
      R"(<ContrastMethods>DIC2, BrightField, DarkField, RLBrightField, RLPol, TLPol</ContrastMethods>)",
      R"(<Features>Pol</Features>)",
      R"(<Immersions>Air</Immersions>)",
      R"(<Magnification>10</Magnification>)",
      R"(<NumericalAperture>0.45</NumericalAperture>)",
      R"(<WorkingDistance>2100</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="Plan-Apochromat 20x/0.8 M27" Id="" UniqueName="Objective.420650-9901-000" Model="" IsMounted="true" Position="3" Type="Objective">)",
      R"(<ContrastMethods>DIC2, BrightField, DarkField, RLBrightField, RLPol, TLPol</ContrastMethods>)",
      R"(<Features>Pol</Features>)",
      R"(<Immersions>Air</Immersions>)",
      R"(<Magnification>20</Magnification>)",
      R"(<NumericalAperture>0.8</NumericalAperture>)",
      R"(<WorkingDistance>610</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="Plan-Apochromat 40x/0.95 Korr M27" Id="" UniqueName="Objective.420660-9970-000" Model="" IsMounted="true" Position="4" Type="Objective">)",
      R"(<ContrastMethods>DIC3, BrightField, DarkField, RLBrightField, RLPol, TLPol</ContrastMethods>)",
      R"(<Features>Korr, Pol</Features>)",
      R"(<Immersions>Air</Immersions>)",
      R"(<Magnification>40</Magnification>)",
      R"(<NumericalAperture>0.95</NumericalAperture>)",
      R"(<WorkingDistance>160</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="none" Id="" UniqueName="Objective.none" Model="" IsMounted="false" Position="5" Type="Objective">)",
      R"(<ContrastMethods>Unknown</ContrastMethods>)",
      R"(<Features>Nothing</Features>)",
      R"(<Immersions>Unknown</Immersions>)",
      R"(<Magnification>0</Magnification>)",
      R"(<NumericalAperture>0</NumericalAperture>)",
      R"(<WorkingDistance>0</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="none" Id="" UniqueName="Objective.none" Model="" IsMounted="false" Position="6" Type="Objective">)",
      R"(<ContrastMethods>Unknown</ContrastMethods>)",
      R"(<Features>Nothing</Features>)",
      R"(<Immersions>Unknown</Immersions>)",
      R"(<Magnification>0</Magnification>)",
      R"(<NumericalAperture>0</NumericalAperture>)",
      R"(<WorkingDistance>0</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
     R"(<Objective Name="none" Id="" UniqueName="Objective.none" Model="" IsMounted="false" Position="7" Type="Objective">)",
      R"(<ContrastMethods>Unknown</ContrastMethods>)",
      R"(<Features>Nothing</Features>)",
      R"(<Immersions>Unknown</Immersions>)",
      R"(<Magnification>0</Magnification>)",
      R"(<NumericalAperture>0</NumericalAperture>)",
      R"(<WorkingDistance>0</WorkingDistance>)",
      R"(<TIRFAngleCorrection>0</TIRFAngleCorrection>)",
     R"(</Objective>)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">4</Position>)",
    R"(<PositionName Status="Valid">Objective.420660-9970-000</PositionName>)",
   R"(</ObjectiveChanger>)",
   R"(<ReflectorChanger Id="MTBReflectorChanger" Name="10x Motorized Reflector Changer" UniqueName="Pollux.ReflectorChanger_10xmot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>10</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<ChangerElement Name="Transm." Id="" UniqueName="Reflector.424931-0000-000" Model="" IsMounted="true" Position="1" Type="None" />)",
     R"(<Reflector Name="49 DAPI" Id="" UniqueName="Reflector.488049-9901-000" Model="" IsMounted="true" Position="2" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="395" EmissionLow="420" EmissionHigh="470" ExcitationLow="335" ExcitationHigh="383" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>None</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="Chroma 655" Id="" UniqueName="User.Chroma 655" Model="" IsMounted="true" Position="3" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="475" EmissionLow="635" EmissionHigh="675" ExcitationLow="350" ExcitationHigh="450" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>None</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="Chroma 605" Id="" UniqueName="User.Chroma 605" Model="" IsMounted="true" Position="4" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="475" EmissionLow="585" EmissionHigh="625" ExcitationLow="350" ExcitationHigh="450" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>None</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="Chroma 705" Id="" UniqueName="User.Chroma 705" Model="" IsMounted="true" Position="5" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="475" EmissionLow="685" EmissionHigh="725" ExcitationLow="350" ExcitationHigh="450" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>None</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="Chroma 565" Id="" UniqueName="User.Chroma 565" Model="" IsMounted="true" Position="6" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="475" EmissionLow="545" EmissionHigh="585" ExcitationLow="350" ExcitationHigh="450" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>None</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="43 HE DsRed" Id="" UniqueName="Reflector.489043-9901-000" Model="" IsMounted="true" Position="7" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="570" EmissionLow="570" EmissionHigh="640" ExcitationLow="538" ExcitationHigh="562" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>HE</Features>)",
     R"(</Reflector>)",
     R"(<Reflector Name="38 HE Green Fluorescent Prot" Id="" UniqueName="Reflector.489038-9901-000" Model="" IsMounted="true" Position="8" Type="Reflector">)",
      R"(<WavelengthAreas>)",
       R"(<WavelengthArea BeamSplitter="495" EmissionLow="500" EmissionHigh="550" ExcitationLow="450" ExcitationHigh="490" />)",
      R"(</WavelengthAreas>)",
      R"(<Features>HE, TIRF</Features>)",
     R"(</Reflector>)",
     R"(<ChangerElement Name="none" Id="" UniqueName="Reflector.none" Model="" IsMounted="true" Position="9" Type="None" />)",
     R"(<ChangerElement Name="none" Id="" UniqueName="Reflector.none" Model="" IsMounted="true" Position="10" Type="None" />)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">2</Position>)",
    R"(<PositionName Status="Valid">Reflector.488049-9901-000</PositionName>)",
   R"(</ReflectorChanger>)",
   R"(<Colibri Id="MTBFLLEDController" Name="Colibri/Colibri.2" UniqueName="Aquarius.FLLEDController" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<LED1 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<MinPosition>0</MinPosition>)",
     R"(<MaxPosition>0</MaxPosition>)",
     R"(<MaxDeviation>0</MaxDeviation>)",
     R"(<TypicalDeviation>0</TypicalDeviation>)",
     R"(<StepWidth>0</StepWidth>)",
     R"(<PositionUnit />)",
     R"(<Wavelength>0</Wavelength>)",
     R"(<FilterName />)",
     R"(<FilterLow>0</FilterLow>)",
     R"(<FilterHigh>0</FilterHigh>)",
    R"(</LED1>)",
    R"(<LED2 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<MinPosition>0</MinPosition>)",
     R"(<MaxPosition>0</MaxPosition>)",
     R"(<MaxDeviation>0</MaxDeviation>)",
     R"(<TypicalDeviation>0</TypicalDeviation>)",
     R"(<StepWidth>0</StepWidth>)",
     R"(<PositionUnit />)",
     R"(<Wavelength>0</Wavelength>)",
     R"(<FilterName />)",
     R"(<FilterLow>0</FilterLow>)",
     R"(<FilterHigh>0</FilterHigh>)",
    R"(</LED2>)",
    R"(<LED3 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<MinPosition>0</MinPosition>)",
     R"(<MaxPosition>0</MaxPosition>)",
     R"(<MaxDeviation>0</MaxDeviation>)",
     R"(<TypicalDeviation>0</TypicalDeviation>)",
     R"(<StepWidth>0</StepWidth>)",
     R"(<PositionUnit />)",
     R"(<Wavelength>0</Wavelength>)",
     R"(<FilterName />)",
     R"(<FilterLow>0</FilterLow>)",
     R"(<FilterHigh>0</FilterHigh>)",
    R"(</LED3>)",
    R"(<LED4 Name="LED-Module 400nm    " Id="MTBLED4" UniqueName="AquariusLED.423052-9030-000" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
     R"(<MinPosition>0</MinPosition>)",
     R"(<MaxPosition>100</MaxPosition>)",
     R"(<MaxDeviation>0.048875855327468229</MaxDeviation>)",
     R"(<TypicalDeviation>0.048875855327468229</TypicalDeviation>)",
     R"(<StepWidth>0.097751710654936458</StepWidth>)",
     R"(<PositionUnit>%</PositionUnit>)",
     R"(<Wavelength>400</Wavelength>)",
     R"(<FilterName>No Excitation Filter</FilterName>)",
     R"(<FilterLow>0</FilterLow>)",
     R"(<FilterHigh>0</FilterHigh>)",
    R"(</LED4>)",
    R"(<BeamCombiner12 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<Wavelength>0</Wavelength>)",
    R"(</BeamCombiner12>)",
    R"(<BeamCombiner34 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<Wavelength>0</Wavelength>)",
    R"(</BeamCombiner34>)",
    R"(<BeamCombiner1234 Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<Wavelength>0</Wavelength>)",
    R"(</BeamCombiner1234>)",
    R"(<FLLEDShutter Name="" Id="" UniqueName="" Model="" IsAvailable="false" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
     R"(<MinPosition>-1</MinPosition>)",
     R"(<MaxPosition>-1</MaxPosition>)",
     R"(<ChangerElements />)",
    R"(</FLLEDShutter>)",
    R"(<LampMode Status="Valid">External</LampMode>)",
   R"(</Colibri>)",
   R"(<LED4 Id="MTBLED4" Name="LED-Module 400nm    " UniqueName="AquariusLED.423052-9030-000" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
    R"(<MinPosition>0</MinPosition>)",
    R"(<MaxPosition>100</MaxPosition>)",
    R"(<MaxDeviation>0.048875855327468229</MaxDeviation>)",
    R"(<TypicalDeviation>0.048875855327468229</TypicalDeviation>)",
    R"(<StepWidth>0.097751710654936458</StepWidth>)",
    R"(<PositionUnit>%</PositionUnit>)",
    R"(<Wavelength>400</Wavelength>)",
    R"(<FilterName>No Excitation Filter</FilterName>)",
    R"(<FilterLow>0</FilterLow>)",
    R"(<FilterHigh>0</FilterHigh>)",
    R"(<Intensity Status="Valid">0</Intensity>)",
    R"(<IsEnabled Status="Valid">false</IsEnabled>)",
   R"(</LED4>)",
   R"(<HXP120Shutter Id="MTBHXP120Shutter" Name="HXP 120 V Shutter" UniqueName="HXP120V.Shutter" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>2</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<ShutterState Name="Closed" Id="" UniqueName="Shutter.Closed" Model="" IsMounted="true" Position="1" Type="ShutterState">)",
      R"(<IsOpen>false</IsOpen>)",
     R"(</ShutterState>)",
     R"(<ShutterState Name="Open" Id="" UniqueName="Shutter.Open" Model="" IsMounted="true" Position="2" Type="ShutterState">)",
      R"(<IsOpen>true</IsOpen>)",
     R"(</ShutterState>)",
    R"(</ChangerElements>)",
    R"(<PositionName Status="Valid">Shutter.Closed</PositionName>)",
    R"(<IsClosed Status="Valid">true</IsClosed>)",
   R"(</HXP120Shutter>)",
   R"(<HXP120V Id="MTBHXPLamp" Name="HXP 120 V" UniqueName="Lamp.HXP120V" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="true" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>100</MaxPosition>)",
    R"(<MaxDeviation>2.6052631578947367</MaxDeviation>)",
    R"(<TypicalDeviation>2.6052631578947367</TypicalDeviation>)",
    R"(<StepWidth>5.2105263157894735</StepWidth>)",
    R"(<PositionUnit>%</PositionUnit>)",
    R"(<Has3200K>false</Has3200K>)",
    R"(<HasOnOff>false</HasOnOff>)",
    R"(<LampType>ShortArc</LampType>)",
    R"(<Intensity Status="Valid">27.052631578947</Intensity>)",
    R"(<LampMode Status="Valid">Manual</LampMode>)",
    R"(<LampActive Status="Valid">true</LampActive>)",
   R"(</HXP120V>)",
   R"(<CondenserContrastChanger Id="MTBCondenserContrastChanger" Name="Motorized Condenser" UniqueName="Pollux.Condenser_mot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>9</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<Contrast Name="Pos1" Id="" UniqueName="Pos1" Model="" IsMounted="true" Position="1" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos2" Id="" UniqueName="Pos2" Model="" IsMounted="true" Position="2" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos3" Id="" UniqueName="Pos3" Model="" IsMounted="true" Position="3" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos4" Id="" UniqueName="Pos4" Model="" IsMounted="true" Position="4" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos5" Id="" UniqueName="Pos5" Model="" IsMounted="true" Position="5" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos6" Id="" UniqueName="Pos6" Model="" IsMounted="true" Position="6" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos7" Id="" UniqueName="Pos7" Model="" IsMounted="true" Position="7" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos8" Id="" UniqueName="Pos8" Model="" IsMounted="true" Position="8" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
     R"(<Contrast Name="Pos9" Id="" UniqueName="Pos9" Model="" IsMounted="true" Position="9" Type="Contrast">)",
      R"(<ContrastMethods>BrightField</ContrastMethods>)",
     R"(</Contrast>)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">1</Position>)",
    R"(<PositionName Status="Valid">Pos1</PositionName>)",
   R"(</CondenserContrastChanger>)",
   R"(<TransmittedLightShutter Id="MTBTLShutter" Name="TL Motorized Shutter" UniqueName="Pollux.TL_Shutter_mot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>2</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<ShutterState Name="Open" Id="" UniqueName="Shutter.Open" Model="" IsMounted="true" Position="1" Type="ShutterState">)",
      R"(<IsOpen>true</IsOpen>)",
     R"(</ShutterState>)",
     R"(<ShutterState Name="Closed" Id="" UniqueName="Shutter.Closed" Model="" IsMounted="true" Position="2" Type="ShutterState">)",
      R"(<IsOpen>false</IsOpen>)",
     R"(</ShutterState>)",
    R"(</ChangerElements>)",
    R"(<PositionName Status="Valid">Shutter.Closed</PositionName>)",
    R"(<IsClosed Status="Valid">true</IsClosed>)",
   R"(</TransmittedLightShutter>)",
   R"(<CameraAdapter Id="MTBCameraAdapter_MTBCameraPortChanger_Front" Name="1x Camera Adapter" UniqueName="CameraAdapter.1x" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
    R"(<Magnification>1</Magnification>)",
    R"(<CameraId />)",
    R"(<TheoreticalTotalMagnification Status="Valid">40</TheoreticalTotalMagnification>)",
    R"(<TotalMagnification Status="Valid">39.55571</TotalMagnification>)",
   R"(</CameraAdapter>)",
   R"(<CameraAdapter Id="MTBCameraAdapter_MTBCameraPortChanger_Left" Name="1x Camera Adapter" UniqueName="CameraAdapter.1x" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="None" IsLightSource="false" IsLightSink="false">)",
    R"(<Magnification>1</Magnification>)",
    R"(<CameraId />)",
    R"(<TheoreticalTotalMagnification Status="Valid">40</TheoreticalTotalMagnification>)",
    R"(<TotalMagnification Status="Valid">39.55571</TotalMagnification>)",
   R"(</CameraAdapter>)",
   R"(<CameraPortChanger Id="MTBCameraPortChanger" Name="3x Motorized Camera Changer" UniqueName="Pollux.CameraChanger_3xmot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>3</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<Mirror Name="Camera Changer Front" Id="" UniqueName="Pollux.CameraChangerElement_Front" Model="" IsMounted="true" Position="1" Type="Mirror">)",
      R"(<SuccessorId>MTBCameraAdapter_MTBCameraPortChanger_Front</SuccessorId>)",
     R"(</Mirror>)",
     R"(<Mirror Name="Camera Changer Left" Id="" UniqueName="Pollux.CameraChangerElement_Left" Model="" IsMounted="true" Position="2" Type="Mirror">)",
      R"(<SuccessorId>MTBCameraAdapter_MTBCameraPortChanger_Left</SuccessorId>)",
     R"(</Mirror>)",
     R"(<Mirror Name="Camera Changer Back" Id="" UniqueName="Pollux.CameraChangerElement_Back" Model="" IsMounted="true" Position="3" Type="Mirror">)",
      R"(<SuccessorId>MTBCameraPortBack</SuccessorId>)",
     R"(</Mirror>)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">2</Position>)",
    R"(<PositionName Status="Valid">Pollux.CameraChangerElement_Left</PositionName>)",
   R"(</CameraPortChanger>)",
   R"(<LoaderDoor Id="MTBLoaderDoor" Name="Tray Door" UniqueName="Pollux.TrayDoor_2xcod" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Coded" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>2</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<ShutterState Name="Open" Id="" UniqueName="Shutter.Open" Model="" IsMounted="true" Position="1" Type="ShutterState">)",
      R"(<IsOpen>true</IsOpen>)",
     R"(</ShutterState>)",
     R"(<ShutterState Name="Closed" Id="" UniqueName="Shutter.Closed" Model="" IsMounted="true" Position="2" Type="ShutterState">)",
      R"(<IsOpen>false</IsOpen>)",
     R"(</ShutterState>)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">2</Position>)",
    R"(<PositionName Status="Valid">Shutter.Closed</PositionName>)",
   R"(</LoaderDoor>)",
   R"(<TrayChanger Id="MTBTrayChanger" Name="25x Motorized Loader" UniqueName="Pollux.Loader_25xmot" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Coded" IsLightSource="false" IsLightSink="false">)",
    R"(<MinPosition>1</MinPosition>)",
    R"(<MaxPosition>26</MaxPosition>)",
    R"(<ChangerElements>)",
     R"(<SlideScannerTray Name="Tray1" Id="" UniqueName="Tray1" Model="" IsMounted="true" Position="1" Type="SlideScannerTray">)",
      R"(<IsLoaded>true</IsLoaded>)",
      R"(<MaxNumOfSlides>4</MaxNumOfSlides>)",
      R"(<NumOfSlides>2</NumOfSlides>)",
      R"(<TrayType>Scan1x3</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>4</NumSlidesAvailable>)",
      R"(<SlidesAvailable0>true</SlidesAvailable0>)",
      R"(<SlidesAvailable1>true</SlidesAvailable1>)",
      R"(<SlidesAvailable2>false</SlidesAvailable2>)",
      R"(<SlidesAvailable3>false</SlidesAvailable3>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray2" Id="" UniqueName="Tray2" Model="" IsMounted="true" Position="2" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>Prescanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray3" Id="" UniqueName="Tray3" Model="" IsMounted="true" Position="3" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray4" Id="" UniqueName="Tray4" Model="" IsMounted="true" Position="4" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray5" Id="" UniqueName="Tray5" Model="" IsMounted="true" Position="5" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray6" Id="" UniqueName="Tray6" Model="" IsMounted="true" Position="6" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray7" Id="" UniqueName="Tray7" Model="" IsMounted="true" Position="7" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray8" Id="" UniqueName="Tray8" Model="" IsMounted="true" Position="8" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray9" Id="" UniqueName="Tray9" Model="" IsMounted="true" Position="9" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray10" Id="" UniqueName="Tray10" Model="" IsMounted="true" Position="10" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray11" Id="" UniqueName="Tray11" Model="" IsMounted="true" Position="11" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray12" Id="" UniqueName="Tray12" Model="" IsMounted="true" Position="12" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray13" Id="" UniqueName="Tray13" Model="" IsMounted="true" Position="13" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray14" Id="" UniqueName="Tray14" Model="" IsMounted="true" Position="14" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray15" Id="" UniqueName="Tray15" Model="" IsMounted="true" Position="15" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray16" Id="" UniqueName="Tray16" Model="" IsMounted="true" Position="16" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray17" Id="" UniqueName="Tray17" Model="" IsMounted="true" Position="17" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray18" Id="" UniqueName="Tray18" Model="" IsMounted="true" Position="18" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray19" Id="" UniqueName="Tray19" Model="" IsMounted="true" Position="19" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray20" Id="" UniqueName="Tray20" Model="" IsMounted="true" Position="20" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray21" Id="" UniqueName="Tray21" Model="" IsMounted="true" Position="21" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray22" Id="" UniqueName="Tray22" Model="" IsMounted="true" Position="22" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray23" Id="" UniqueName="Tray23" Model="" IsMounted="true" Position="23" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray24" Id="" UniqueName="Tray24" Model="" IsMounted="true" Position="24" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="Tray25" Id="" UniqueName="Tray25" Model="" IsMounted="true" Position="25" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>None</TrayType>)",
      R"(<TrayWorkingState>NotScanned</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
     R"(<SlideScannerTray Name="ParkingPosition" Id="" UniqueName="ParkingPosition" Model="" IsMounted="true" Position="26" Type="SlideScannerTray">)",
      R"(<IsLoaded>false</IsLoaded>)",
      R"(<MaxNumOfSlides>0</MaxNumOfSlides>)",
      R"(<NumOfSlides>0</NumOfSlides>)",
      R"(<TrayType>Parking</TrayType>)",
      R"(<TrayWorkingState>NotAvailable</TrayWorkingState>)",
      R"(<NumSlidesAvailable>0</NumSlidesAvailable>)",
     R"(</SlideScannerTray>)",
    R"(</ChangerElements>)",
    R"(<Position Status="Valid">26</Position>)",
    R"(<PositionName Status="Valid">ParkingPosition</PositionName>)",
   R"(</TrayChanger>)",
   R"(<Camera Id="CameraUnmounted0.AxioCamICAVTGuppy" Name="AxioCamIC" UniqueName="AVTGuppy_S252343071" Model="AxioCamICAVTGuppy" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="true">)",
    R"(<ApplyCameraProfile Status="Valid">false</ApplyCameraProfile>)",
    R"(<ApplyCameraProfile.Type IsReadOnly="true" Status="Valid">BoolType</ApplyCameraProfile.Type>)",
    R"(<ApplyCameraProfile.Default IsReadOnly="true" Status="Valid">false</ApplyCameraProfile.Default>)",
    R"(<ApplyImageOrientation Status="Valid">true</ApplyImageOrientation>)",
    R"(<ApplyImageOrientation.Type IsReadOnly="true" Status="Valid">BoolType</ApplyImageOrientation.Type>)",
    R"(<ApplyImageOrientation.Default IsReadOnly="true" Status="Valid">true</ApplyImageOrientation.Default>)",
    R"(<CameraBias IsReadOnly="true" Status="Valid">0</CameraBias>)",
    R"(<CameraBias.Type IsReadOnly="true" Status="Valid">IntegerType</CameraBias.Type>)",
    R"(<CameraOrientation IsReadOnly="true" Status="Valid">Original</CameraOrientation>)",
    R"(<CameraOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</CameraOrientation.Type>)",
    R"(<CameraOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,4|Rotate|Rotate 90 CW,7|FlipReverseRotate|Rotate 90 CCW,3|FlipReverse|Rotate 180,6|ReverseRotate|Mirror at +45 Diagonal,5|FlipRotate|Mirror at -45 Diagonal</CameraOrientation.ListItems>)",
    R"(<CameraOrientation.Default IsReadOnly="true" Status="Valid">0</CameraOrientation.Default>)",
    R"(<CameraPixelAccuracy IsReadOnly="true" Status="Valid">0.00390625</CameraPixelAccuracy>)",
    R"(<CameraPixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</CameraPixelAccuracy.Type>)",
    R"(<CameraPixelDistances IsReadOnly="true" Status="Valid">2.2,2.2</CameraPixelDistances>)",
    R"(<CameraPixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</CameraPixelDistances.Type>)",
    R"(<CameraPixelMaximum IsReadOnly="true" Status="Valid">255</CameraPixelMaximum>)",
    R"(<CameraPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMaximum.Type>)",
    R"(<CameraPixelMinimum IsReadOnly="true" Status="Valid">0</CameraPixelMinimum>)",
    R"(<CameraPixelMinimum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMinimum.Type>)",
    R"(<CameraPixelType IsReadOnly="true" Status="Valid">Bgr48</CameraPixelType>)",
    R"(<CameraPixelType.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelType.Type>)",
    R"(<ExposureTime Status="Valid">11</ExposureTime>)",
    R"(<ExposureTime.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureTime.Type>)",
    R"(<ExposureTime.Default IsReadOnly="true" Status="Valid">20</ExposureTime.Default>)",
    R"(<ExposureTime.Min IsReadOnly="true" Status="Valid">1</ExposureTime.Min>)",
    R"(<ExposureTime.Max IsReadOnly="true" Status="Valid">4095</ExposureTime.Max>)",
    R"(<ExposureTime.Increment IsReadOnly="true" Status="Valid">1</ExposureTime.Increment>)",
    R"(<ExposureTime.GuiHint IsReadOnly="true" Status="Valid">GuiLogSlider</ExposureTime.GuiHint>)",
    R"(<Frame Status="Valid">878,555,1500,723</Frame>)",
    R"(<Frame.Type IsReadOnly="true" Status="Valid">RectType</Frame.Type>)",
    R"(<Frame.Default IsReadOnly="true" Status="Valid">Empty</Frame.Default>)",
    R"(<Frame.Max IsReadOnly="true" Status="Valid">0,0,2592,1944</Frame.Max>)",
    R"(<Frame.GuiHint IsReadOnly="true" Status="Valid">GuiRect</Frame.GuiHint>)",
    R"(<ImageByteSize IsReadOnly="true" Status="Valid">6507000</ImageByteSize>)",
    R"(<ImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</ImageByteSize.Type>)",
    R"(<ImageFrame IsReadOnly="true" Status="Valid">878,555,1500,723</ImageFrame>)",
    R"(<ImageFrame.Type IsReadOnly="true" Status="Valid">RectType</ImageFrame.Type>)",
    R"(<ImageOrientation Status="Valid">1</ImageOrientation>)",
    R"(<ImageOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</ImageOrientation.Type>)",
    R"(<ImageOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,4|Rotate|Rotate 90 CW,7|FlipReverseRotate|Rotate 90 CCW,3|FlipReverse|Rotate 180,6|ReverseRotate|Mirror at +45 Diagonal,5|FlipRotate|Mirror at -45 Diagonal</ImageOrientation.ListItems>)",
    R"(<ImageOrientation.Default IsReadOnly="true" Status="Valid">0</ImageOrientation.Default>)",
    R"(<ImageOrientation.Label IsReadOnly="true" Status="Valid">Orientation</ImageOrientation.Label>)",
    R"(<ImageOrientation.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</ImageOrientation.GuiHint>)",
    R"(<ImagePixelAccuracy IsReadOnly="true" Status="Valid">0.000244140625</ImagePixelAccuracy>)",
    R"(<ImagePixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</ImagePixelAccuracy.Type>)",
    R"(<ImagePixelDistances IsReadOnly="true" Status="Valid">2.2,2.2</ImagePixelDistances>)",
    R"(<ImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</ImagePixelDistances.Type>)",
    R"(<ImagePixelType IsReadOnly="true" Status="Valid">Bgr48</ImagePixelType>)",
    R"(<ImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</ImagePixelType.Type>)",
    R"(<LiveFrame IsReadOnly="true" Status="Valid">878,555,1500,723</LiveFrame>)",
    R"(<LiveFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveFrame.Type>)",
    R"(<LiveImageByteSize IsReadOnly="true" Status="Valid">6507000</LiveImageByteSize>)",
    R"(<LiveImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImageByteSize.Type>)",
    R"(<LiveImageFrame IsReadOnly="true" Status="Valid">878,555,1500,723</LiveImageFrame>)",
    R"(<LiveImageFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveImageFrame.Type>)",
    R"(<LiveImagePixelDistances IsReadOnly="true" Status="Valid">2.2,2.2</LiveImagePixelDistances>)",
    R"(<LiveImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</LiveImagePixelDistances.Type>)",
    R"(<LiveImagePixelType IsReadOnly="true" Status="Valid">Bgr48</LiveImagePixelType>)",
    R"(<LiveImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImagePixelType.Type>)",
    R"(<LiveValidBits IsReadOnly="true" Status="Valid">12</LiveValidBits>)",
    R"(<LiveValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</LiveValidBits.Type>)",
    R"(<ValidBits IsReadOnly="true" Status="Valid">12</ValidBits>)",
    R"(<ValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</ValidBits.Type>)",
    R"(<ValidPixelMaximum IsReadOnly="true" Status="Valid">255</ValidPixelMaximum>)",
    R"(<ValidPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</ValidPixelMaximum.Type>)",
    R"(<ValidPixelMaximum.Default IsReadOnly="true" Status="Valid">255</ValidPixelMaximum.Default>)",
    R"(<Adjust Status="Valid">1</Adjust>)",
    R"(<Adjust.Type IsReadOnly="true" Status="Valid">DoubleType</Adjust.Type>)",
    R"(<Adjust.Default IsReadOnly="true" Status="Valid">1</Adjust.Default>)",
    R"(<Adjust.Min IsReadOnly="true" Status="Valid">0.05</Adjust.Min>)",
    R"(<Adjust.Max IsReadOnly="true" Status="Valid">2</Adjust.Max>)",
    R"(<Adjust.Increment IsReadOnly="true" Status="Valid">0.05</Adjust.Increment>)",
    R"(<Adjust.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</Adjust.GuiHint>)",
    R"(<Adjust.Enabled IsReadOnly="true" Status="Valid">true</Adjust.Enabled>)",
    R"(<AdjustFrameRate Status="Valid">false</AdjustFrameRate>)",
    R"(<AdjustFrameRate.Type IsReadOnly="true" Status="Valid">BoolType</AdjustFrameRate.Type>)",
    R"(<AdjustFrameRate.Default IsReadOnly="true" Status="Valid">false</AdjustFrameRate.Default>)",
    R"(<AdjustFrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Adjust Live Frame Rate</AdjustFrameRate.Label>)",
    R"(<AdjustFrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AdjustFrameRate.GuiHint>)",
    R"(<AdjustFrameRate.Mode IsReadOnly="true" Status="Valid">Expert</AdjustFrameRate.Mode>)",
    R"(<AdjustFrameRate.Enabled IsReadOnly="true" Status="Valid">true</AdjustFrameRate.Enabled>)",
    R"(<AdjustFrameRate.DisplayOrder IsReadOnly="true" Status="Valid">200</AdjustFrameRate.DisplayOrder>)",
    R"(<AutoExposure Status="Valid">false</AutoExposure>)",
    R"(<AutoExposure.Type IsReadOnly="true" Status="Valid">BoolType</AutoExposure.Type>)",
    R"(<AutoExposure.Default IsReadOnly="true" Status="Valid">false</AutoExposure.Default>)",
    R"(<AutoExposure.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AutoExposure.GuiHint>)",
    R"(<AutoExposure.Enabled IsReadOnly="true" Status="Valid">true</AutoExposure.Enabled>)",
    R"(<AutoExposureFrame Status="Valid">879.666666666667,555,1498.33333333333,721.333333333333</AutoExposureFrame>)",
    R"(<AutoExposureFrame.Type IsReadOnly="true" Status="Valid">RectType</AutoExposureFrame.Type>)",
    R"(<AutoExposureFrame.Default IsReadOnly="true" Status="Valid">Empty</AutoExposureFrame.Default>)",
    R"(<AutoExposureFrame.GuiHint IsReadOnly="true" Status="Valid">GuiRect</AutoExposureFrame.GuiHint>)",
    R"(<AutoExposureFrame.Enabled IsReadOnly="true" Status="Valid">true</AutoExposureFrame.Enabled>)",
    R"(<Binning Status="Valid">1,1</Binning>)",
    R"(<Binning.Type IsReadOnly="true" Status="Valid">SizeType</Binning.Type>)",
    R"(<Binning.Default IsReadOnly="true" Status="Valid">1,1</Binning.Default>)",
    R"(<Binning.GuiHint IsReadOnly="true" Status="Valid">GuiNone</Binning.GuiHint>)",
    R"(<BinningList Status="Valid">0</BinningList>)",
    R"(<BinningList.Type IsReadOnly="true" Status="Valid">IntegerType</BinningList.Type>)",
    R"(<BinningList.ListItems IsReadOnly="true" Status="Valid">0|1x1|1x1</BinningList.ListItems>)",
    R"(<BinningList.Default IsReadOnly="true" Status="Valid">0</BinningList.Default>)",
    R"(<BinningList.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</BinningList.GuiHint>)",
    R"(<BlackReference Status="Valid">false</BlackReference>)",
    R"(<BlackReference.Type IsReadOnly="true" Status="Valid">BoolType</BlackReference.Type>)",
    R"(<BlackReference.Default IsReadOnly="true" Status="Valid">false</BlackReference.Default>)",
    R"(<BlackReference.Enabled IsReadOnly="true" Status="Valid">false</BlackReference.Enabled>)",
    R"(<DualCameraCalibration Status="Valid">false</DualCameraCalibration>)",
    R"(<DualCameraCalibration.Type IsReadOnly="true" Status="Valid">BoolType</DualCameraCalibration.Type>)",
    R"(<DualCameraCalibration.Default IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Default>)",
    R"(<DualCameraCalibration.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</DualCameraCalibration.GuiHint>)",
    R"(<DualCameraCalibrationId Status="Valid">None</DualCameraCalibrationId>)",
    R"(<DualCameraCalibrationId.Type IsReadOnly="true" Status="Valid">StringType</DualCameraCalibrationId.Type>)",
    R"(<DualCameraCalibrationId.Default IsReadOnly="true" Status="Valid" />)",
    R"(<DualCameraCalibrationId.Mode IsReadOnly="true" Status="Valid">Normal</DualCameraCalibrationId.Mode>)",
    R"(<DualCameraCalibration.Enabled IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Enabled>)",
    R"(<ExposureDelay Status="Valid">0</ExposureDelay>)",
    R"(<ExposureDelay.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureDelay.Type>)",
    R"(<ExposureDelay.Default IsReadOnly="true" Status="Valid">0</ExposureDelay.Default>)",
    R"(<ExposureDelay.Label IsReadOnly="true" Status="Valid">Acquire|Acquisition Delay</ExposureDelay.Label>)",
    R"(<ExposureDelay.GuiHint IsReadOnly="true" Status="Valid">GuiNone</ExposureDelay.GuiHint>)",
    R"(<ExposureDelay.Min IsReadOnly="true" Status="Valid">0</ExposureDelay.Min>)",
    R"(<ExposureDelay.Max IsReadOnly="true" Status="Valid">1000</ExposureDelay.Max>)",
    R"(<ExposureDelay.Increment IsReadOnly="true" Status="Valid">1</ExposureDelay.Increment>)",
    R"(<FrameRate IsReadOnly="true" Status="Valid">3.3</FrameRate>)",
    R"(<FrameRate.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRate.Type>)",
    R"(<FrameRate.Default IsReadOnly="true" Status="Valid">0</FrameRate.Default>)",
    R"(<FrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate</FrameRate.Label>)",
    R"(<FrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</FrameRate.GuiHint>)",
    R"(<FrameRate.Mode IsReadOnly="true" Status="Valid">Expert</FrameRate.Mode>)",
    R"(<FrameRate.DisplayOrder IsReadOnly="true" Status="Valid">202</FrameRate.DisplayOrder>)",
    R"(<FrameRateTarget Status="Valid">30</FrameRateTarget>)",
    R"(<FrameRateTarget.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRateTarget.Type>)",
    R"(<FrameRateTarget.Default IsReadOnly="true" Status="Valid">30</FrameRateTarget.Default>)",
    R"(<FrameRateTarget.Min IsReadOnly="true" Status="Valid">1</FrameRateTarget.Min>)",
    R"(<FrameRateTarget.Max IsReadOnly="true" Status="Valid">100</FrameRateTarget.Max>)",
    R"(<FrameRateTarget.Increment IsReadOnly="true" Status="Valid">1</FrameRateTarget.Increment>)",
    R"(<FrameRateTarget.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate Max</FrameRateTarget.Label>)",
    R"(<FrameRateTarget.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</FrameRateTarget.GuiHint>)",
    R"(<FrameRateTarget.Mode IsReadOnly="true" Status="Valid">Expert</FrameRateTarget.Mode>)",
    R"(<FrameRateTarget.Enabled IsReadOnly="true" Status="Valid">false</FrameRateTarget.Enabled>)",
    R"(<FrameRateTarget.DisplayOrder IsReadOnly="true" Status="Valid">201</FrameRateTarget.DisplayOrder>)",
    R"(<ImageOrientation.Enabled IsReadOnly="true" Status="Valid">true</ImageOrientation.Enabled>)",
    R"(<NoiseFilter Status="Valid">false</NoiseFilter>)",
    R"(<NoiseFilter.Type IsReadOnly="true" Status="Valid">BoolType</NoiseFilter.Type>)",
    R"(<NoiseFilter.Default IsReadOnly="true" Status="Valid">false</NoiseFilter.Default>)",
    R"(<NoiseFilter.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</NoiseFilter.GuiHint>)",
    R"(<NoiseFilterThreshold Status="Valid">1.5</NoiseFilterThreshold>)",
    R"(<NoiseFilterThreshold.Type IsReadOnly="true" Status="Valid">DoubleType</NoiseFilterThreshold.Type>)",
    R"(<NoiseFilterThreshold.Default IsReadOnly="true" Status="Valid">1.5</NoiseFilterThreshold.Default>)",
    R"(<NoiseFilterThreshold.Min IsReadOnly="true" Status="Valid">0</NoiseFilterThreshold.Min>)",
    R"(<NoiseFilterThreshold.Max IsReadOnly="true" Status="Valid">10</NoiseFilterThreshold.Max>)",
    R"(<NoiseFilterThreshold.Increment IsReadOnly="true" Status="Valid">0.1</NoiseFilterThreshold.Increment>)",
    R"(<NoiseFilterThreshold.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</NoiseFilterThreshold.GuiHint>)",
    R"(<NoiseFilter.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilter.Enabled>)",
    R"(<NoiseFilterThreshold.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilterThreshold.Enabled>)",
    R"(<ReferenceImageCount Status="Valid">3</ReferenceImageCount>)",
    R"(<ReferenceImageCount.Type IsReadOnly="true" Status="Valid">IntegerType</ReferenceImageCount.Type>)",
    R"(<ReferenceImageCount.Default IsReadOnly="true" Status="Valid">3</ReferenceImageCount.Default>)",
    R"(<ReferenceImageCount.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ReferenceImageCount.GuiHint>)",
    R"(<ReferenceImageCount.Label IsReadOnly="true" Status="Valid">Post Processing|Reference Image Count</ReferenceImageCount.Label>)",
    R"(<ReferenceImageCount.Enabled IsReadOnly="true" Status="Valid">true</ReferenceImageCount.Enabled>)",
    R"(<ReferenceImageCount.Min IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Min>)",
    R"(<ReferenceImageCount.Max IsReadOnly="true" Status="Valid">5</ReferenceImageCount.Max>)",
    R"(<ReferenceImageCount.Increment IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Increment>)",
    R"(<ReferenceImageCount.Mode IsReadOnly="true" Status="Valid">Expert</ReferenceImageCount.Mode>)",
    R"(<ShadingReference Status="Valid">false</ShadingReference>)",
    R"(<ShadingReference.Type IsReadOnly="true" Status="Valid">BoolType</ShadingReference.Type>)",
    R"(<ShadingReference.Default IsReadOnly="true" Status="Valid">false</ShadingReference.Default>)",
    R"(<ShadingReference.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingReference.GuiHint>)",
    R"(<ShadingReference.Enabled IsReadOnly="true" Status="Valid">false</ShadingReference.Enabled>)",
    R"(<ShadingStrength Status="Valid">1</ShadingStrength>)",
    R"(<ShadingStrength.Type IsReadOnly="true" Status="Valid">DoubleType</ShadingStrength.Type>)",
    R"(<ShadingStrength.Default IsReadOnly="true" Status="Valid">1</ShadingStrength.Default>)",
    R"(<ShadingStrength.Min IsReadOnly="true" Status="Valid">0.1</ShadingStrength.Min>)",
    R"(<ShadingStrength.Max IsReadOnly="true" Status="Valid">2</ShadingStrength.Max>)",
    R"(<ShadingStrength.Increment IsReadOnly="true" Status="Valid">0.01</ShadingStrength.Increment>)",
    R"(<ShadingStrength.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ShadingStrength.GuiHint>)",
    R"(<ShadingStrength.Mode IsReadOnly="true" Status="Valid">Expert</ShadingStrength.Mode>)",
    R"(<ShadingStrength.Enabled IsReadOnly="true" Status="Valid">true</ShadingStrength.Enabled>)",
    R"(<ShadingStrength.Label IsReadOnly="true" Status="Valid">Shading Strength</ShadingStrength.Label>)",
    R"(<ShadingUseBias Status="Valid">false</ShadingUseBias>)",
    R"(<ShadingUseBias.Type IsReadOnly="true" Status="Valid">BoolType</ShadingUseBias.Type>)",
    R"(<ShadingUseBias.Default IsReadOnly="true" Status="Valid">false</ShadingUseBias.Default>)",
    R"(<ShadingUseBias.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingUseBias.GuiHint>)",
    R"(<ShadingUseBias.Mode IsReadOnly="true" Status="Valid">Expert</ShadingUseBias.Mode>)",
    R"(<ShadingUseBias.Enabled IsReadOnly="true" Status="Valid">true</ShadingUseBias.Enabled>)",
    R"(<ShadingUseBias.Label IsReadOnly="true" Status="Valid">Shading Use Bias</ShadingUseBias.Label>)",
    R"(<ColorMode Status="Valid">1</ColorMode>)",
    R"(<Resolution.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</Resolution.GuiHint>)",
    R"(<Resolution Status="Valid">0</Resolution>)",
    R"(<Resolution.Type IsReadOnly="true" Status="Valid">IntegerType</Resolution.Type>)",
    R"(<Resolution.Default IsReadOnly="true" Status="Valid">0</Resolution.Default>)",
    R"(<Resolution.ListItems IsReadOnly="true" Status="Valid">0|2592x1944    12bit|2592x1944    12bit</Resolution.ListItems>)",
    R"(<Resolution.Enabled IsReadOnly="true" Status="Valid">false</Resolution.Enabled>)",
    R"(<UnsharpMask Status="Valid">false</UnsharpMask>)",
    R"(<UnsharpMask.Type IsReadOnly="true" Status="Valid">BoolType</UnsharpMask.Type>)",
    R"(<UnsharpMask.Default IsReadOnly="true" Status="Valid">false</UnsharpMask.Default>)",
    R"(<UnsharpMask.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</UnsharpMask.GuiHint>)",
    R"(<UnsharpMaskState Status="Valid" />)",
    R"(<UnsharpMaskState.Type IsReadOnly="true" Status="Valid">StringType</UnsharpMaskState.Type>)",
    R"(<UnsharpMaskState.Default IsReadOnly="true" Status="Valid" />)",
    R"(<UnsharpMask.Enabled IsReadOnly="true" Status="Valid">true</UnsharpMask.Enabled>)",
    R"(<ColorMode.Type IsReadOnly="true" Status="Valid">IntegerType</ColorMode.Type>)",
    R"(<ColorMode.ListItems IsReadOnly="true" Status="Valid">0|0|Monochrome,1|1|Color</ColorMode.ListItems>)",
    R"(<ColorMode.Default IsReadOnly="true" Status="Valid">1</ColorMode.Default>)",
    R"(<ColorMode.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</ColorMode.GuiHint>)",
    R"(<ColorMode.Enabled IsReadOnly="true" Status="Valid">true</ColorMode.Enabled>)",
    R"(<ExposureTime.Enabled IsReadOnly="true" Status="Valid">true</ExposureTime.Enabled>)",
    R"(<Frame.Enabled IsReadOnly="true" Status="Valid">true</Frame.Enabled>)",
    R"(<WhiteBalance Status="Valid">0.55700765137786,1,0.77327999776287</WhiteBalance>)",
    R"(<WhiteBalance.Type IsReadOnly="true" Status="Valid">StringType</WhiteBalance.Type>)",
    R"(<WhiteBalance.Default IsReadOnly="true" Status="Valid">1,1,1</WhiteBalance.Default>)",
    R"(<WhiteBalance.GuiHint IsReadOnly="true" Status="Valid">GuiWhiteBalanceValues</WhiteBalance.GuiHint>)",
    R"(<WhiteBalance.Enabled IsReadOnly="true" Status="Valid">true</WhiteBalance.Enabled>)",
    R"(<WhiteBalance.ListItems IsReadOnly="true" Status="Valid">0|1.01&amp;comma;0.8&amp;comma;0.5|3200K,1|1&amp;comma;1&amp;comma;1|5500K</WhiteBalance.ListItems>)",
    R"(<WhiteBalanceOffset Status="Valid">0</WhiteBalanceOffset>)",
    R"(<WhiteBalanceOffset.Type IsReadOnly="true" Status="Valid">DoubleType</WhiteBalanceOffset.Type>)",
    R"(<WhiteBalanceOffset.Default IsReadOnly="true" Status="Valid">0</WhiteBalanceOffset.Default>)",
    R"(<WhiteBalanceOffset.Min IsReadOnly="true" Status="Valid">-3</WhiteBalanceOffset.Min>)",
    R"(<WhiteBalanceOffset.Max IsReadOnly="true" Status="Valid">3</WhiteBalanceOffset.Max>)",
    R"(<WhiteBalanceOffset.Increment IsReadOnly="true" Status="Valid">0.1</WhiteBalanceOffset.Increment>)",
    R"(<WhiteBalanceOffset.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</WhiteBalanceOffset.GuiHint>)",
    R"(<WhiteBalanceOffset.Enabled IsReadOnly="true" Status="Valid">true</WhiteBalanceOffset.Enabled>)",
    R"(<ColorSaturation Status="Valid">0</ColorSaturation>)",
    R"(<ColorSaturation.Type IsReadOnly="true" Status="Valid">DoubleType</ColorSaturation.Type>)",
    R"(<ColorSaturation.Default IsReadOnly="true" Status="Valid">0</ColorSaturation.Default>)",
    R"(<ColorSaturation.Min IsReadOnly="true" Status="Valid">-1</ColorSaturation.Min>)",
    R"(<ColorSaturation.Max IsReadOnly="true" Status="Valid">1</ColorSaturation.Max>)",
    R"(<ColorSaturation.Increment IsReadOnly="true" Status="Valid">0.1</ColorSaturation.Increment>)",
    R"(<ColorSaturation.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ColorSaturation.GuiHint>)",
    R"(<ColorSaturation.Enabled IsReadOnly="true" Status="Valid">true</ColorSaturation.Enabled>)",
    R"(<Trigger Status="Valid">false</Trigger>)",
    R"(<Trigger.Type IsReadOnly="true" Status="Valid">BoolType</Trigger.Type>)",
    R"(<Trigger.Default IsReadOnly="true" Status="Valid">false</Trigger.Default>)",
    R"(<Trigger.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</Trigger.GuiHint>)",
    R"(<Trigger.Enabled IsReadOnly="true" Status="Valid">true</Trigger.Enabled>)",
    R"(<TriggerMode Status="Valid">0</TriggerMode>)",
    R"(<TriggerMode.Type IsReadOnly="true" Status="Valid">IntegerType</TriggerMode.Type>)",
    R"(<TriggerMode.ListItems IsReadOnly="true" Status="Valid">0|Active High|Active High,1|Active Low|Active Low</TriggerMode.ListItems>)",
    R"(<TriggerMode.Default IsReadOnly="true" Status="Valid">0</TriggerMode.Default>)",
    R"(<TriggerMode.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</TriggerMode.GuiHint>)",
    R"(<TriggerMode.Label IsReadOnly="true" Status="Valid">Acquire|Trigger Mode</TriggerMode.Label>)",
    R"(<TriggerMode.Mode IsReadOnly="true" Status="Valid">Expert</TriggerMode.Mode>)",
    R"(<TriggerMode.Enabled IsReadOnly="true" Status="Valid">true</TriggerMode.Enabled>)",
    R"(<ImageFrame.Max IsReadOnly="true" Status="Valid">0,0,2592,1944</ImageFrame.Max>)",
    R"(<LiveImageFrame.Max IsReadOnly="true" Status="Valid">0,0,2592,1944</LiveImageFrame.Max>)",
    R"(<CameraIdentifier IsReadOnly="true" Status="Valid">AVTGuppy_S252343071</CameraIdentifier>)",
    R"(<CameraIdentifier.Type IsReadOnly="true" Status="Valid">StringType</CameraIdentifier.Type>)",
    R"(<CameraIdentifier.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</CameraIdentifier.GuiHint>)",
    R"(<CameraIdentifier.DisplayOrder IsReadOnly="true" Status="Valid">-999</CameraIdentifier.DisplayOrder>)",
    R"(<CameraIdentifier.Label IsReadOnly="true" Status="Valid">Camera Identifier</CameraIdentifier.Label>)",
    R"(<AutomaticTriggerControl Status="Valid">true</AutomaticTriggerControl>)",
    R"(<AutomaticTriggerControl.Type IsReadOnly="true" Status="Valid">BoolType</AutomaticTriggerControl.Type>)",
    R"(<AutomaticTriggerControl.Default IsReadOnly="true" Status="Valid">true</AutomaticTriggerControl.Default>)",
    R"(<AutomaticTriggerControl.Enabled IsReadOnly="true" Status="Valid">true</AutomaticTriggerControl.Enabled>)",
    R"(<AutomaticTriggerControl.Label IsReadOnly="true" Status="Valid">Acquire|Auto Trigger Control</AutomaticTriggerControl.Label>)",
    R"(<AutomaticTriggerControl.Mode IsReadOnly="true" Status="Valid">Expert</AutomaticTriggerControl.Mode>)",
    R"(<AutomaticTriggerControl.GuiHint IsReadOnly="true" Status="Valid">GuiNone</AutomaticTriggerControl.GuiHint>)",
    R"(<CameraGeometry Status="Valid" />)",
    R"(<HDRBits Status="Valid">0</HDRBits>)",
    R"(<ImageCount Status="Valid">0</ImageCount>)",
    R"(<SnapMode Status="Valid">0</SnapMode>)",
    R"(<ThreadCount Status="Valid" />)",
    R"(<AdjustLiveFrameRate Status="Valid">false</AdjustLiveFrameRate>)",
    R"(<HDRBits.Enabled IsReadOnly="true" Status="Valid">false</HDRBits.Enabled>)",
    R"(<ImageCount.Enabled IsReadOnly="true" Status="Valid">false</ImageCount.Enabled>)",
    R"(<EMGain Status="Valid">0</EMGain>)",
   R"(</Camera>)",
   R"(<Camera Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" Name="Orca Flash 4.0" UniqueName="HDCamC11440-22C_SSN 8Y0633" Model="HDCamC11440-22C" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="true">)",
    R"(<ApplyCameraProfile Status="Valid">false</ApplyCameraProfile>)",
    R"(<ApplyCameraProfile.Type IsReadOnly="true" Status="Valid">BoolType</ApplyCameraProfile.Type>)",
    R"(<ApplyCameraProfile.Default IsReadOnly="true" Status="Valid">false</ApplyCameraProfile.Default>)",
    R"(<ApplyImageOrientation Status="Valid">true</ApplyImageOrientation>)",
    R"(<ApplyImageOrientation.Type IsReadOnly="true" Status="Valid">BoolType</ApplyImageOrientation.Type>)",
    R"(<ApplyImageOrientation.Default IsReadOnly="true" Status="Valid">true</ApplyImageOrientation.Default>)",
    R"(<CameraBias IsReadOnly="true" Status="Valid">200</CameraBias>)",
    R"(<CameraBias.Type IsReadOnly="true" Status="Valid">IntegerType</CameraBias.Type>)",
    R"(<CameraOrientation IsReadOnly="true" Status="Valid">0</CameraOrientation>)",
    R"(<CameraOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</CameraOrientation.Type>)",
    R"(<CameraOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,4|Rotate|Rotate 90 CW,7|FlipReverseRotate|Rotate 90 CCW,3|FlipReverse|Rotate 180,6|ReverseRotate|Mirror at +45 Diagonal,5|FlipRotate|Mirror at -45 Diagonal</CameraOrientation.ListItems>)",
    R"(<CameraOrientation.Default IsReadOnly="true" Status="Valid">0</CameraOrientation.Default>)",
    R"(<CameraPixelAccuracy IsReadOnly="true" Status="Valid">0.00390625</CameraPixelAccuracy>)",
    R"(<CameraPixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</CameraPixelAccuracy.Type>)",
    R"(<CameraPixelDistances IsReadOnly="true" Status="Valid">6.5,6.5</CameraPixelDistances>)",
    R"(<CameraPixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</CameraPixelDistances.Type>)",
    R"(<CameraPixelMaximum IsReadOnly="true" Status="Valid">255</CameraPixelMaximum>)",
    R"(<CameraPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMaximum.Type>)",
    R"(<CameraPixelMinimum IsReadOnly="true" Status="Valid">0</CameraPixelMinimum>)",
    R"(<CameraPixelMinimum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMinimum.Type>)",
    R"(<CameraPixelType IsReadOnly="true" Status="Valid">Gray16</CameraPixelType>)",
    R"(<CameraPixelType.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelType.Type>)",
    R"(<ExposureTime Status="Valid">15</ExposureTime>)",
    R"(<ExposureTime.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureTime.Type>)",
    R"(<ExposureTime.Default IsReadOnly="true" Status="Valid">20</ExposureTime.Default>)",
    R"(<ExposureTime.Min IsReadOnly="true" Status="Valid">1.0036692256108</ExposureTime.Min>)",
    R"(<ExposureTime.Max IsReadOnly="true" Status="Valid">10000</ExposureTime.Max>)",
    R"(<ExposureTime.Increment IsReadOnly="true" Status="Valid">0.01</ExposureTime.Increment>)",
    R"(<ExposureTime.GuiHint IsReadOnly="true" Status="Valid">GuiLogSlider</ExposureTime.GuiHint>)",
    R"(<Frame Status="Valid">4,4,2040,2040</Frame>)",
    R"(<Frame.Type IsReadOnly="true" Status="Valid">RectType</Frame.Type>)",
    R"(<Frame.Default IsReadOnly="true" Status="Valid">Empty</Frame.Default>)",
    R"(<Frame.Max IsReadOnly="true" Status="Valid">0,0,2048,2048</Frame.Max>)",
    R"(<Frame.GuiHint IsReadOnly="true" Status="Valid">GuiRect</Frame.GuiHint>)",
    R"(<ImageByteSize IsReadOnly="true" Status="Valid">8323200</ImageByteSize>)",
    R"(<ImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</ImageByteSize.Type>)",
    R"(<ImageFrame IsReadOnly="true" Status="Valid">4,4,2040,2040</ImageFrame>)",
    R"(<ImageFrame.Type IsReadOnly="true" Status="Valid">RectType</ImageFrame.Type>)",
    R"(<ImageOrientation Status="Valid">2</ImageOrientation>)",
    R"(<ImageOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</ImageOrientation.Type>)",
    R"(<ImageOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,4|Rotate|Rotate 90 CW,7|FlipReverseRotate|Rotate 90 CCW,3|FlipReverse|Rotate 180,6|ReverseRotate|Mirror at +45 Diagonal,5|FlipRotate|Mirror at -45 Diagonal</ImageOrientation.ListItems>)",
    R"(<ImageOrientation.Default IsReadOnly="true" Status="Valid">0</ImageOrientation.Default>)",
    R"(<ImageOrientation.Label IsReadOnly="true" Status="Valid">Orientation</ImageOrientation.Label>)",
    R"(<ImageOrientation.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</ImageOrientation.GuiHint>)",
    R"(<ImagePixelAccuracy IsReadOnly="true" Status="Valid">1.52587890625E-05</ImagePixelAccuracy>)",
    R"(<ImagePixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</ImagePixelAccuracy.Type>)",
    R"(<ImagePixelDistances IsReadOnly="true" Status="Valid">6.5,6.5</ImagePixelDistances>)",
    R"(<ImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</ImagePixelDistances.Type>)",
    R"(<ImagePixelType IsReadOnly="true" Status="Valid">Gray16</ImagePixelType>)",
    R"(<ImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</ImagePixelType.Type>)",
    R"(<LiveFrame IsReadOnly="true" Status="Valid">4,4,2040,2040</LiveFrame>)",
    R"(<LiveFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveFrame.Type>)",
    R"(<LiveImageByteSize IsReadOnly="true" Status="Valid">8323200</LiveImageByteSize>)",
    R"(<LiveImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImageByteSize.Type>)",
    R"(<LiveImageFrame IsReadOnly="true" Status="Valid">4,4,2040,2040</LiveImageFrame>)",
    R"(<LiveImageFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveImageFrame.Type>)",
    R"(<LiveImagePixelDistances IsReadOnly="true" Status="Valid">6.5,6.5</LiveImagePixelDistances>)",
    R"(<LiveImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</LiveImagePixelDistances.Type>)",
    R"(<LiveImagePixelType IsReadOnly="true" Status="Valid">Gray16</LiveImagePixelType>)",
    R"(<LiveImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImagePixelType.Type>)",
    R"(<LiveValidBits IsReadOnly="true" Status="Valid">16</LiveValidBits>)",
    R"(<LiveValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</LiveValidBits.Type>)",
    R"(<ValidBits IsReadOnly="true" Status="Valid">16</ValidBits>)",
    R"(<ValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</ValidBits.Type>)",
    R"(<ValidPixelMaximum IsReadOnly="true" Status="Valid">255</ValidPixelMaximum>)",
    R"(<ValidPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</ValidPixelMaximum.Type>)",
    R"(<ValidPixelMaximum.Default IsReadOnly="true" Status="Valid">255</ValidPixelMaximum.Default>)",
    R"(<Adjust Status="Valid">0.5</Adjust>)",
    R"(<Adjust.Type IsReadOnly="true" Status="Valid">DoubleType</Adjust.Type>)",
    R"(<Adjust.Default IsReadOnly="true" Status="Valid">1</Adjust.Default>)",
    R"(<Adjust.Min IsReadOnly="true" Status="Valid">0.05</Adjust.Min>)",
    R"(<Adjust.Max IsReadOnly="true" Status="Valid">2</Adjust.Max>)",
    R"(<Adjust.Increment IsReadOnly="true" Status="Valid">0.05</Adjust.Increment>)",
    R"(<Adjust.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</Adjust.GuiHint>)",
    R"(<Adjust.Enabled IsReadOnly="true" Status="Valid">true</Adjust.Enabled>)",
    R"(<AdjustFrameRate Status="Valid">false</AdjustFrameRate>)",
    R"(<AdjustFrameRate.Type IsReadOnly="true" Status="Valid">BoolType</AdjustFrameRate.Type>)",
    R"(<AdjustFrameRate.Default IsReadOnly="true" Status="Valid">false</AdjustFrameRate.Default>)",
    R"(<AdjustFrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Adjust Live Frame Rate</AdjustFrameRate.Label>)",
    R"(<AdjustFrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AdjustFrameRate.GuiHint>)",
    R"(<AdjustFrameRate.Mode IsReadOnly="true" Status="Valid">Expert</AdjustFrameRate.Mode>)",
    R"(<AdjustFrameRate.Enabled IsReadOnly="true" Status="Valid">true</AdjustFrameRate.Enabled>)",
    R"(<AdjustFrameRate.DisplayOrder IsReadOnly="true" Status="Valid">200</AdjustFrameRate.DisplayOrder>)",
    R"(<AutoExposure Status="Valid">false</AutoExposure>)",
    R"(<AutoExposure.Type IsReadOnly="true" Status="Valid">BoolType</AutoExposure.Type>)",
    R"(<AutoExposure.Default IsReadOnly="true" Status="Valid">false</AutoExposure.Default>)",
    R"(<AutoExposure.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AutoExposure.GuiHint>)",
    R"(<AutoExposure.Enabled IsReadOnly="true" Status="Valid">true</AutoExposure.Enabled>)",
    R"(<AutoExposureFrame Status="Valid">Empty</AutoExposureFrame>)",
    R"(<AutoExposureFrame.Type IsReadOnly="true" Status="Valid">RectType</AutoExposureFrame.Type>)",
    R"(<AutoExposureFrame.Default IsReadOnly="true" Status="Valid">Empty</AutoExposureFrame.Default>)",
    R"(<AutoExposureFrame.GuiHint IsReadOnly="true" Status="Valid">GuiRect</AutoExposureFrame.GuiHint>)",
    R"(<AutoExposureFrame.Enabled IsReadOnly="true" Status="Valid">true</AutoExposureFrame.Enabled>)",
    R"(<Binning Status="Valid">1,1</Binning>)",
    R"(<Binning.Type IsReadOnly="true" Status="Valid">SizeType</Binning.Type>)",
    R"(<Binning.Default IsReadOnly="true" Status="Valid">1,1</Binning.Default>)",
    R"(<Binning.GuiHint IsReadOnly="true" Status="Valid">GuiNone</Binning.GuiHint>)",
    R"(<BinningList Status="Valid">0</BinningList>)",
    R"(<BinningList.Type IsReadOnly="true" Status="Valid">IntegerType</BinningList.Type>)",
    R"(<BinningList.ListItems IsReadOnly="true" Status="Valid">0|1x1|1x1,1|2x2|2x2,2|4x4|4x4</BinningList.ListItems>)",
    R"(<BinningList.Default IsReadOnly="true" Status="Valid">0</BinningList.Default>)",
    R"(<BinningList.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</BinningList.GuiHint>)",
    R"(<BlackReference Status="Valid">false</BlackReference>)",
    R"(<BlackReference.Type IsReadOnly="true" Status="Valid">BoolType</BlackReference.Type>)",
    R"(<BlackReference.Default IsReadOnly="true" Status="Valid">false</BlackReference.Default>)",
    R"(<BlackReference.Enabled IsReadOnly="true" Status="Valid">false</BlackReference.Enabled>)",
    R"(<DualCameraCalibration Status="Valid">false</DualCameraCalibration>)",
    R"(<DualCameraCalibration.Type IsReadOnly="true" Status="Valid">BoolType</DualCameraCalibration.Type>)",
    R"(<DualCameraCalibration.Default IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Default>)",
    R"(<DualCameraCalibration.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</DualCameraCalibration.GuiHint>)",
    R"(<DualCameraCalibrationId Status="Valid">None</DualCameraCalibrationId>)",
    R"(<DualCameraCalibrationId.Type IsReadOnly="true" Status="Valid">StringType</DualCameraCalibrationId.Type>)",
    R"(<DualCameraCalibrationId.Default IsReadOnly="true" Status="Valid" />)",
    R"(<DualCameraCalibrationId.Mode IsReadOnly="true" Status="Valid">Normal</DualCameraCalibrationId.Mode>)",
    R"(<DualCameraCalibration.Enabled IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Enabled>)",
    R"(<ExposureDelay Status="Valid">0</ExposureDelay>)",
    R"(<ExposureDelay.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureDelay.Type>)",
    R"(<ExposureDelay.Default IsReadOnly="true" Status="Valid">0</ExposureDelay.Default>)",
    R"(<ExposureDelay.Label IsReadOnly="true" Status="Valid">Acquire|Acquisition Delay</ExposureDelay.Label>)",
    R"(<ExposureDelay.GuiHint IsReadOnly="true" Status="Valid">GuiNone</ExposureDelay.GuiHint>)",
    R"(<ExposureDelay.Min IsReadOnly="true" Status="Valid">0</ExposureDelay.Min>)",
    R"(<ExposureDelay.Max IsReadOnly="true" Status="Valid">1000</ExposureDelay.Max>)",
    R"(<ExposureDelay.Increment IsReadOnly="true" Status="Valid">1</ExposureDelay.Increment>)",
    R"(<FrameRate IsReadOnly="true" Status="Valid">33.3</FrameRate>)",
    R"(<FrameRate.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRate.Type>)",
    R"(<FrameRate.Default IsReadOnly="true" Status="Valid">0</FrameRate.Default>)",
    R"(<FrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate</FrameRate.Label>)",
    R"(<FrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</FrameRate.GuiHint>)",
    R"(<FrameRate.Mode IsReadOnly="true" Status="Valid">Expert</FrameRate.Mode>)",
    R"(<FrameRate.DisplayOrder IsReadOnly="true" Status="Valid">202</FrameRate.DisplayOrder>)",
    R"(<FrameRateTarget Status="Valid">30</FrameRateTarget>)",
    R"(<FrameRateTarget.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRateTarget.Type>)",
    R"(<FrameRateTarget.Default IsReadOnly="true" Status="Valid">30</FrameRateTarget.Default>)",
    R"(<FrameRateTarget.Min IsReadOnly="true" Status="Valid">1</FrameRateTarget.Min>)",
    R"(<FrameRateTarget.Max IsReadOnly="true" Status="Valid">100</FrameRateTarget.Max>)",
    R"(<FrameRateTarget.Increment IsReadOnly="true" Status="Valid">1</FrameRateTarget.Increment>)",
    R"(<FrameRateTarget.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate Max</FrameRateTarget.Label>)",
    R"(<FrameRateTarget.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</FrameRateTarget.GuiHint>)",
    R"(<FrameRateTarget.Mode IsReadOnly="true" Status="Valid">Expert</FrameRateTarget.Mode>)",
    R"(<FrameRateTarget.Enabled IsReadOnly="true" Status="Valid">false</FrameRateTarget.Enabled>)",
    R"(<FrameRateTarget.DisplayOrder IsReadOnly="true" Status="Valid">201</FrameRateTarget.DisplayOrder>)",
    R"(<ImageOrientation.Enabled IsReadOnly="true" Status="Valid">true</ImageOrientation.Enabled>)",
    R"(<NoiseFilter Status="Valid">false</NoiseFilter>)",
    R"(<NoiseFilter.Type IsReadOnly="true" Status="Valid">BoolType</NoiseFilter.Type>)",
    R"(<NoiseFilter.Default IsReadOnly="true" Status="Valid">false</NoiseFilter.Default>)",
    R"(<NoiseFilter.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</NoiseFilter.GuiHint>)",
    R"(<NoiseFilterThreshold Status="Valid">1.5</NoiseFilterThreshold>)",
    R"(<NoiseFilterThreshold.Type IsReadOnly="true" Status="Valid">DoubleType</NoiseFilterThreshold.Type>)",
    R"(<NoiseFilterThreshold.Default IsReadOnly="true" Status="Valid">1.5</NoiseFilterThreshold.Default>)",
    R"(<NoiseFilterThreshold.Min IsReadOnly="true" Status="Valid">0</NoiseFilterThreshold.Min>)",
    R"(<NoiseFilterThreshold.Max IsReadOnly="true" Status="Valid">10</NoiseFilterThreshold.Max>)",
    R"(<NoiseFilterThreshold.Increment IsReadOnly="true" Status="Valid">0.1</NoiseFilterThreshold.Increment>)",
    R"(<NoiseFilterThreshold.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</NoiseFilterThreshold.GuiHint>)",
    R"(<NoiseFilter.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilter.Enabled>)",
    R"(<NoiseFilterThreshold.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilterThreshold.Enabled>)",
    R"(<ReferenceImageCount Status="Valid">3</ReferenceImageCount>)",
    R"(<ReferenceImageCount.Type IsReadOnly="true" Status="Valid">IntegerType</ReferenceImageCount.Type>)",
    R"(<ReferenceImageCount.Default IsReadOnly="true" Status="Valid">3</ReferenceImageCount.Default>)",
    R"(<ReferenceImageCount.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ReferenceImageCount.GuiHint>)",
    R"(<ReferenceImageCount.Label IsReadOnly="true" Status="Valid">Post Processing|Reference Image Count</ReferenceImageCount.Label>)",
    R"(<ReferenceImageCount.Enabled IsReadOnly="true" Status="Valid">true</ReferenceImageCount.Enabled>)",
    R"(<ReferenceImageCount.Min IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Min>)",
    R"(<ReferenceImageCount.Max IsReadOnly="true" Status="Valid">5</ReferenceImageCount.Max>)",
    R"(<ReferenceImageCount.Increment IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Increment>)",
    R"(<ReferenceImageCount.Mode IsReadOnly="true" Status="Valid">Expert</ReferenceImageCount.Mode>)",
    R"(<ShadingReference Status="Valid">false</ShadingReference>)",
    R"(<ShadingReference.Type IsReadOnly="true" Status="Valid">BoolType</ShadingReference.Type>)",
    R"(<ShadingReference.Default IsReadOnly="true" Status="Valid">false</ShadingReference.Default>)",
    R"(<ShadingReference.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingReference.GuiHint>)",
    R"(<ShadingReference.Enabled IsReadOnly="true" Status="Valid">false</ShadingReference.Enabled>)",
    R"(<ShadingStrength Status="Valid">1</ShadingStrength>)",
    R"(<ShadingStrength.Type IsReadOnly="true" Status="Valid">DoubleType</ShadingStrength.Type>)",
    R"(<ShadingStrength.Default IsReadOnly="true" Status="Valid">1</ShadingStrength.Default>)",
    R"(<ShadingStrength.Min IsReadOnly="true" Status="Valid">0.1</ShadingStrength.Min>)",
    R"(<ShadingStrength.Max IsReadOnly="true" Status="Valid">2</ShadingStrength.Max>)",
    R"(<ShadingStrength.Increment IsReadOnly="true" Status="Valid">0.01</ShadingStrength.Increment>)",
    R"(<ShadingStrength.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ShadingStrength.GuiHint>)",
    R"(<ShadingStrength.Mode IsReadOnly="true" Status="Valid">Expert</ShadingStrength.Mode>)",
    R"(<ShadingStrength.Enabled IsReadOnly="true" Status="Valid">true</ShadingStrength.Enabled>)",
    R"(<ShadingStrength.Label IsReadOnly="true" Status="Valid">Shading Strength</ShadingStrength.Label>)",
    R"(<ShadingUseBias Status="Valid">false</ShadingUseBias>)",
    R"(<ShadingUseBias.Type IsReadOnly="true" Status="Valid">BoolType</ShadingUseBias.Type>)",
    R"(<ShadingUseBias.Default IsReadOnly="true" Status="Valid">false</ShadingUseBias.Default>)",
    R"(<ShadingUseBias.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingUseBias.GuiHint>)",
    R"(<ShadingUseBias.Mode IsReadOnly="true" Status="Valid">Expert</ShadingUseBias.Mode>)",
    R"(<ShadingUseBias.Enabled IsReadOnly="true" Status="Valid">true</ShadingUseBias.Enabled>)",
    R"(<ShadingUseBias.Label IsReadOnly="true" Status="Valid">Shading Use Bias</ShadingUseBias.Label>)",
    R"(<UnsharpMask Status="Valid">false</UnsharpMask>)",
    R"(<UnsharpMask.Type IsReadOnly="true" Status="Valid">BoolType</UnsharpMask.Type>)",
    R"(<UnsharpMask.Default IsReadOnly="true" Status="Valid">false</UnsharpMask.Default>)",
    R"(<UnsharpMask.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</UnsharpMask.GuiHint>)",
    R"(<UnsharpMaskState Status="Valid" />)",
    R"(<UnsharpMaskState.Type IsReadOnly="true" Status="Valid">StringType</UnsharpMaskState.Type>)",
    R"(<UnsharpMaskState.Default IsReadOnly="true" Status="Valid" />)",
    R"(<UnsharpMask.Enabled IsReadOnly="true" Status="Valid">true</UnsharpMask.Enabled>)",
    R"(<CameraIdentifier IsReadOnly="true" Status="Valid">HDCamC11440-22C_SSN 8Y0633</CameraIdentifier>)",
    R"(<CameraIdentifier.Type IsReadOnly="true" Status="Valid">StringType</CameraIdentifier.Type>)",
    R"(<CameraIdentifier.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</CameraIdentifier.GuiHint>)",
    R"(<CameraIdentifier.DisplayOrder IsReadOnly="true" Status="Valid">-999</CameraIdentifier.DisplayOrder>)",
    R"(<CameraIdentifier.Label IsReadOnly="true" Status="Valid">Camera Identifier</CameraIdentifier.Label>)",
    R"(<AutoExposureTimeIncrement IsReadOnly="true" Status="Valid">0.1</AutoExposureTimeIncrement>)",
    R"(<AutoExposureTimeIncrement.Type IsReadOnly="true" Status="Valid">IntegerType</AutoExposureTimeIncrement.Type>)",
    R"(<AutoExposureTimeIncrement.Default IsReadOnly="true" Status="Valid">1</AutoExposureTimeIncrement.Default>)",
    R"(<ExposureTime.Enabled IsReadOnly="true" Status="Valid">true</ExposureTime.Enabled>)",
    R"(<InternalTriggerMinExposure IsReadOnly="true" Status="Valid">1.00 ms</InternalTriggerMinExposure>)",
    R"(<InternalTriggerMinExposure.Type IsReadOnly="true" Status="Valid">IntegerType</InternalTriggerMinExposure.Type>)",
    R"(<InternalTriggerMinExposure.Default IsReadOnly="true" Status="Valid">0</InternalTriggerMinExposure.Default>)",
    R"(<InternalTriggerMinExposure.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</InternalTriggerMinExposure.GuiHint>)",
    R"(<InternalTriggerMinExposure.Label IsReadOnly="true" Status="Valid">Acquire|Min. Exposure Int. Trigger</InternalTriggerMinExposure.Label>)",
    R"(<InternalTriggerMinExposure.Enabled IsReadOnly="true" Status="Valid">false</InternalTriggerMinExposure.Enabled>)",
    R"(<DefectPixelCorrection Status="Valid">true</DefectPixelCorrection>)",
    R"(<DefectPixelCorrection.Type IsReadOnly="true" Status="Valid">BoolType</DefectPixelCorrection.Type>)",
    R"(<DefectPixelCorrection.Default IsReadOnly="true" Status="Valid">true</DefectPixelCorrection.Default>)",
    R"(<DefectPixelCorrection.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</DefectPixelCorrection.GuiHint>)",
    R"(<DefectPixelCorrection.Label IsReadOnly="true" Status="Valid">Defect Pixel Correction</DefectPixelCorrection.Label>)",
    R"(<DefectPixelCorrection.Enabled IsReadOnly="true" Status="Valid">true</DefectPixelCorrection.Enabled>)",
    R"(<BinningList.Enabled IsReadOnly="true" Status="Valid">true</BinningList.Enabled>)",
    R"(<Frame.Enabled IsReadOnly="true" Status="Valid">true</Frame.Enabled>)",
    R"(<TriggerMode Status="Valid">3</TriggerMode>)",
    R"(<TriggerMode.Type IsReadOnly="true" Status="Valid">IntegerType</TriggerMode.Type>)",
    R"(<TriggerMode.ListItems IsReadOnly="true" Status="Valid">0|Level Low|Level Low,1|Level High|Level High,2|Falling Edge|Falling Edge,3|Rising Edge|Rising Edge</TriggerMode.ListItems>)",
    R"(<TriggerMode.Default IsReadOnly="true" Status="Valid">0</TriggerMode.Default>)",
    R"(<TriggerMode.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</TriggerMode.GuiHint>)",
    R"(<TriggerMode.Label IsReadOnly="true" Status="Valid">Acquire|Trigger Mode</TriggerMode.Label>)",
    R"(<TriggerMode.Mode IsReadOnly="true" Status="Valid">Expert</TriggerMode.Mode>)",
    R"(<TriggerMode.Enabled IsReadOnly="true" Status="Valid">true</TriggerMode.Enabled>)",
    R"(<Trigger Status="Valid">false</Trigger>)",
    R"(<Trigger.Type IsReadOnly="true" Status="Valid">BoolType</Trigger.Type>)",
    R"(<Trigger.Default IsReadOnly="true" Status="Valid">false</Trigger.Default>)",
    R"(<Trigger.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</Trigger.GuiHint>)",
    R"(<Trigger.Enabled IsReadOnly="true" Status="Valid">true</Trigger.Enabled>)",
    R"(<ImageFrame.Max IsReadOnly="true" Status="Valid">0,0,2048,2048</ImageFrame.Max>)",
    R"(<LiveImageFrame.Max IsReadOnly="true" Status="Valid">0,0,2048,2048</LiveImageFrame.Max>)",
    R"(<TheoreticalTotalMagnification IsActivated="true" IsReadOnly="true" Status="Valid">40</TheoreticalTotalMagnification>)",
    R"(<TotalMagnification IsActivated="true" IsReadOnly="true" Status="Valid">39.55571</TotalMagnification>)",
    R"(<AutomaticTriggerControl Status="Valid">true</AutomaticTriggerControl>)",
   R"(</Camera>)",
   R"(<Camera Id="MTBCamera_MTBCameraPortChanger_Front.MILCamHitachi_FD202SCL" Name="Hitachi HV-F202SCL" UniqueName="MILCamHitachi_FD202SCL_S0" Model="MILCamHitachi_FD202SCL" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="true">)",
    R"(<ApplyCameraProfile Status="Valid">false</ApplyCameraProfile>)",
    R"(<ApplyCameraProfile.Type IsReadOnly="true" Status="Valid">BoolType</ApplyCameraProfile.Type>)",
    R"(<ApplyCameraProfile.Default IsReadOnly="true" Status="Valid">false</ApplyCameraProfile.Default>)",
    R"(<ApplyImageOrientation Status="Valid">true</ApplyImageOrientation>)",
    R"(<ApplyImageOrientation.Type IsReadOnly="true" Status="Valid">BoolType</ApplyImageOrientation.Type>)",
    R"(<ApplyImageOrientation.Default IsReadOnly="true" Status="Valid">true</ApplyImageOrientation.Default>)",
    R"(<CameraBias IsReadOnly="true" Status="Valid">0</CameraBias>)",
    R"(<CameraBias.Type IsReadOnly="true" Status="Valid">IntegerType</CameraBias.Type>)",
    R"(<CameraOrientation IsReadOnly="true" Status="Valid">Flip</CameraOrientation>)",
    R"(<CameraOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</CameraOrientation.Type>)",
    R"(<CameraOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,4|Rotate|Rotate 90 CW,7|FlipReverseRotate|Rotate 90 CCW,3|FlipReverse|Rotate 180,6|ReverseRotate|Mirror at +45 Diagonal,5|FlipRotate|Mirror at -45 Diagonal</CameraOrientation.ListItems>)",
    R"(<CameraOrientation.Default IsReadOnly="true" Status="Valid">0</CameraOrientation.Default>)",
    R"(<CameraPixelAccuracy IsReadOnly="true" Status="Valid">0.00390625</CameraPixelAccuracy>)",
    R"(<CameraPixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</CameraPixelAccuracy.Type>)",
    R"(<CameraPixelDistances IsReadOnly="true" Status="Valid">4.4,4.4</CameraPixelDistances>)",
    R"(<CameraPixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</CameraPixelDistances.Type>)",
    R"(<CameraPixelMaximum IsReadOnly="true" Status="Valid">255</CameraPixelMaximum>)",
    R"(<CameraPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMaximum.Type>)",
    R"(<CameraPixelMinimum IsReadOnly="true" Status="Valid">0</CameraPixelMinimum>)",
    R"(<CameraPixelMinimum.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelMinimum.Type>)",
    R"(<CameraPixelType IsReadOnly="true" Status="Valid">Bgr24</CameraPixelType>)",
    R"(<CameraPixelType.Type IsReadOnly="true" Status="Valid">IntegerType</CameraPixelType.Type>)",
    R"(<ExposureTime Status="Valid">0.2</ExposureTime>)",
    R"(<ExposureTime.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureTime.Type>)",
    R"(<ExposureTime.Default IsReadOnly="true" Status="Valid">20</ExposureTime.Default>)",
    R"(<ExposureTime.Min IsReadOnly="true" Status="Valid">0.01</ExposureTime.Min>)",
    R"(<ExposureTime.Max IsReadOnly="true" Status="Valid">10000</ExposureTime.Max>)",
    R"(<ExposureTime.Increment IsReadOnly="true" Status="Valid">0.01035</ExposureTime.Increment>)",
    R"(<ExposureTime.GuiHint IsReadOnly="true" Status="Valid">GuiLogSlider</ExposureTime.GuiHint>)",
    R"(<Frame Status="Valid">0,0,1600,1200</Frame>)",
    R"(<Frame.Type IsReadOnly="true" Status="Valid">RectType</Frame.Type>)",
    R"(<Frame.Default IsReadOnly="true" Status="Valid">Empty</Frame.Default>)",
    R"(<Frame.Max IsReadOnly="true" Status="Valid">0,0,1600,1200</Frame.Max>)",
    R"(<Frame.GuiHint IsReadOnly="true" Status="Valid">GuiRect</Frame.GuiHint>)",
    R"(<ImageByteSize IsReadOnly="true" Status="Valid">5760000</ImageByteSize>)",
    R"(<ImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</ImageByteSize.Type>)",
    R"(<ImageFrame IsReadOnly="true" Status="Valid">0,0,1600,1200</ImageFrame>)",
    R"(<ImageFrame.Type IsReadOnly="true" Status="Valid">RectType</ImageFrame.Type>)",
    R"(<ImageOrientation.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</ImageOrientation.GuiHint>)",
    R"(<ImageOrientation.Label IsReadOnly="true" Status="Valid">Orientation</ImageOrientation.Label>)",
    R"(<ImageOrientation Status="Valid">1</ImageOrientation>)",
    R"(<ImageOrientation.Type IsReadOnly="true" Status="Valid">IntegerType</ImageOrientation.Type>)",
    R"(<ImageOrientation.ListItems IsReadOnly="true" Status="Valid">0|Original|Original,1|Flip|Flip Horizontally,2|Reverse|Flip Vertically,3|FlipReverse|Rotate 180</ImageOrientation.ListItems>)",
    R"(<ImageOrientation.Default IsReadOnly="true" Status="Valid">0</ImageOrientation.Default>)",
    R"(<ImagePixelAccuracy IsReadOnly="true" Status="Valid">0.00390625</ImagePixelAccuracy>)",
    R"(<ImagePixelAccuracy.Type IsReadOnly="true" Status="Valid">DoubleType</ImagePixelAccuracy.Type>)",
    R"(<ImagePixelDistances IsReadOnly="true" Status="Valid">4.4,4.4</ImagePixelDistances>)",
    R"(<ImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</ImagePixelDistances.Type>)",
    R"(<ImagePixelType IsReadOnly="true" Status="Valid">Bgr24</ImagePixelType>)",
    R"(<ImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</ImagePixelType.Type>)",
    R"(<LiveFrame IsReadOnly="true" Status="Valid">0,0,1600,1200</LiveFrame>)",
    R"(<LiveFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveFrame.Type>)",
    R"(<LiveImageByteSize IsReadOnly="true" Status="Valid">0</LiveImageByteSize>)",
    R"(<LiveImageByteSize.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImageByteSize.Type>)",
    R"(<LiveImageFrame IsReadOnly="true" Status="Valid">0,0,1600,1200</LiveImageFrame>)",
    R"(<LiveImageFrame.Type IsReadOnly="true" Status="Valid">RectType</LiveImageFrame.Type>)",
    R"(<LiveImagePixelDistances IsReadOnly="true" Status="Valid">4.4,4.4</LiveImagePixelDistances>)",
    R"(<LiveImagePixelDistances.Type IsReadOnly="true" Status="Valid">SizeType</LiveImagePixelDistances.Type>)",
    R"(<LiveImagePixelType IsReadOnly="true" Status="Valid">Bgr24</LiveImagePixelType>)",
    R"(<LiveImagePixelType.Type IsReadOnly="true" Status="Valid">IntegerType</LiveImagePixelType.Type>)",
    R"(<LiveValidBits IsReadOnly="true" Status="Valid">8</LiveValidBits>)",
    R"(<LiveValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</LiveValidBits.Type>)",
    R"(<ValidBits IsReadOnly="true" Status="Valid">8</ValidBits>)",
    R"(<ValidBits.Type IsReadOnly="true" Status="Valid">IntegerType</ValidBits.Type>)",
    R"(<ValidPixelMaximum IsReadOnly="true" Status="Valid">255</ValidPixelMaximum>)",
    R"(<ValidPixelMaximum.Type IsReadOnly="true" Status="Valid">IntegerType</ValidPixelMaximum.Type>)",
    R"(<ValidPixelMaximum.Default IsReadOnly="true" Status="Valid">255</ValidPixelMaximum.Default>)",
    R"(<MaxTriggerFrameRate Status="Valid">29.95</MaxTriggerFrameRate>)",
    R"(<MaxTriggerFrameRate.Type IsReadOnly="true" Status="Valid">DoubleType</MaxTriggerFrameRate.Type>)",
    R"(<MaxTriggerFrameRate.Default IsReadOnly="true" Status="Valid">29.95</MaxTriggerFrameRate.Default>)",
    R"(<MaxTriggerFrameRate.Enabled IsReadOnly="true" Status="Valid">false</MaxTriggerFrameRate.Enabled>)",
    R"(<MaxTriggerFrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</MaxTriggerFrameRate.GuiHint>)",
    R"(<MaxTriggerFrameRate.Label IsReadOnly="true" Status="Valid">Maximum frame rate in triggered mode.</MaxTriggerFrameRate.Label>)",
    R"(<TriggerTimeout Status="Valid">10000</TriggerTimeout>)",
    R"(<TriggerTimeout.Type IsReadOnly="true" Status="Valid">IntegerType</TriggerTimeout.Type>)",
    R"(<TriggerTimeout.Default IsReadOnly="true" Status="Valid">10000</TriggerTimeout.Default>)",
    R"(<TriggerTimeout.Enabled IsReadOnly="true" Status="Valid">true</TriggerTimeout.Enabled>)",
    R"(<TriggerTimeout.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</TriggerTimeout.GuiHint>)",
    R"(<TriggerTimeout.Mode IsReadOnly="true" Status="Valid">Debug</TriggerTimeout.Mode>)",
    R"(<TriggerTimeout.Label IsReadOnly="true" Status="Valid">Timeout [ms] for a camera image in trigger mode.</TriggerTimeout.Label>)",
    R"(<LineScanImageCount Status="Valid">0</LineScanImageCount>)",
    R"(<LineScanImageCount.Type IsReadOnly="true" Status="Valid">IntegerType</LineScanImageCount.Type>)",
    R"(<LineScanImageCount.Default IsReadOnly="true" Status="Valid">0</LineScanImageCount.Default>)",
    R"(<Adjust Status="Valid">1</Adjust>)",
    R"(<Adjust.Type IsReadOnly="true" Status="Valid">DoubleType</Adjust.Type>)",
    R"(<Adjust.Default IsReadOnly="true" Status="Valid">1</Adjust.Default>)",
    R"(<Adjust.Min IsReadOnly="true" Status="Valid">0.05</Adjust.Min>)",
    R"(<Adjust.Max IsReadOnly="true" Status="Valid">2</Adjust.Max>)",
    R"(<Adjust.Increment IsReadOnly="true" Status="Valid">0.05</Adjust.Increment>)",
    R"(<Adjust.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</Adjust.GuiHint>)",
    R"(<Adjust.Enabled IsReadOnly="true" Status="Valid">true</Adjust.Enabled>)",
    R"(<AdjustFrameRate Status="Valid">false</AdjustFrameRate>)",
    R"(<AdjustFrameRate.Type IsReadOnly="true" Status="Valid">BoolType</AdjustFrameRate.Type>)",
    R"(<AdjustFrameRate.Default IsReadOnly="true" Status="Valid">false</AdjustFrameRate.Default>)",
    R"(<AdjustFrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Adjust Live Frame Rate</AdjustFrameRate.Label>)",
    R"(<AdjustFrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AdjustFrameRate.GuiHint>)",
    R"(<AdjustFrameRate.Mode IsReadOnly="true" Status="Valid">Expert</AdjustFrameRate.Mode>)",
    R"(<AdjustFrameRate.Enabled IsReadOnly="true" Status="Valid">true</AdjustFrameRate.Enabled>)",
    R"(<AdjustFrameRate.DisplayOrder IsReadOnly="true" Status="Valid">200</AdjustFrameRate.DisplayOrder>)",
    R"(<AutoExposure Status="Valid">false</AutoExposure>)",
    R"(<AutoExposure.Type IsReadOnly="true" Status="Valid">BoolType</AutoExposure.Type>)",
    R"(<AutoExposure.Default IsReadOnly="true" Status="Valid">false</AutoExposure.Default>)",
    R"(<AutoExposure.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</AutoExposure.GuiHint>)",
    R"(<AutoExposure.Enabled IsReadOnly="true" Status="Valid">true</AutoExposure.Enabled>)",
    R"(<WhiteBalanceOffset Status="Valid">0</WhiteBalanceOffset>)",
    R"(<WhiteBalance.GuiHint IsReadOnly="true" Status="Valid">GuiWhiteBalanceValues</WhiteBalance.GuiHint>)",
    R"(<WhiteBalance Status="Valid">0.38886577758484,0.57139313740009,1</WhiteBalance>)",
    R"(<WhiteBalance.Default IsReadOnly="true" Status="Valid">1,1,1</WhiteBalance.Default>)",
    R"(<WhiteBalance.Type IsReadOnly="true" Status="Valid">StringType</WhiteBalance.Type>)",
    R"(<Binning Status="Valid">1,1</Binning>)",
    R"(<Binning.Type IsReadOnly="true" Status="Valid">SizeType</Binning.Type>)",
    R"(<Binning.Default IsReadOnly="true" Status="Valid">1,1</Binning.Default>)",
    R"(<Binning.GuiHint IsReadOnly="true" Status="Valid">GuiNone</Binning.GuiHint>)",
    R"(<BinningList Status="Valid">0</BinningList>)",
    R"(<BinningList.Type IsReadOnly="true" Status="Valid">IntegerType</BinningList.Type>)",
    R"(<BinningList.ListItems IsReadOnly="true" Status="Valid">0|1x1|1x1</BinningList.ListItems>)",
    R"(<BinningList.Default IsReadOnly="true" Status="Valid">0</BinningList.Default>)",
    R"(<BinningList.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</BinningList.GuiHint>)",
    R"(<CameraIdentifier.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</CameraIdentifier.GuiHint>)",
    R"(<CameraIdentifier IsReadOnly="true" Status="Valid">MILCamHitachi_FD202SCL_S0</CameraIdentifier>)",
    R"(<CameraIdentifier.Type IsReadOnly="true" Status="Valid">StringType</CameraIdentifier.Type>)",
    R"(<DualCameraCalibration Status="Valid">false</DualCameraCalibration>)",
    R"(<DualCameraCalibration.Type IsReadOnly="true" Status="Valid">BoolType</DualCameraCalibration.Type>)",
    R"(<DualCameraCalibration.Default IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Default>)",
    R"(<DualCameraCalibration.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</DualCameraCalibration.GuiHint>)",
    R"(<DualCameraCalibrationId Status="Valid">None</DualCameraCalibrationId>)",
    R"(<DualCameraCalibrationId.Type IsReadOnly="true" Status="Valid">StringType</DualCameraCalibrationId.Type>)",
    R"(<DualCameraCalibrationId.Default IsReadOnly="true" Status="Valid" />)",
    R"(<DualCameraCalibrationId.Mode IsReadOnly="true" Status="Valid">Normal</DualCameraCalibrationId.Mode>)",
    R"(<DualCameraCalibration.Enabled IsReadOnly="true" Status="Valid">false</DualCameraCalibration.Enabled>)",
    R"(<ExposureDelay Status="Valid">0</ExposureDelay>)",
    R"(<ExposureDelay.Type IsReadOnly="true" Status="Valid">DoubleType</ExposureDelay.Type>)",
    R"(<ExposureDelay.Default IsReadOnly="true" Status="Valid">0</ExposureDelay.Default>)",
    R"(<ExposureDelay.Label IsReadOnly="true" Status="Valid">Acquire|Acquisition Delay</ExposureDelay.Label>)",
    R"(<ExposureDelay.GuiHint IsReadOnly="true" Status="Valid">GuiNone</ExposureDelay.GuiHint>)",
    R"(<ExposureDelay.Min IsReadOnly="true" Status="Valid">0</ExposureDelay.Min>)",
    R"(<ExposureDelay.Max IsReadOnly="true" Status="Valid">1000</ExposureDelay.Max>)",
    R"(<ExposureDelay.Increment IsReadOnly="true" Status="Valid">1</ExposureDelay.Increment>)",
    R"(<FrameRate IsReadOnly="true" Status="Valid">0</FrameRate>)",
    R"(<FrameRate.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRate.Type>)",
    R"(<FrameRate.Default IsReadOnly="true" Status="Valid">0</FrameRate.Default>)",
    R"(<FrameRate.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate</FrameRate.Label>)",
    R"(<FrameRate.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</FrameRate.GuiHint>)",
    R"(<FrameRate.Mode IsReadOnly="true" Status="Valid">Expert</FrameRate.Mode>)",
    R"(<FrameRate.DisplayOrder IsReadOnly="true" Status="Valid">202</FrameRate.DisplayOrder>)",
    R"(<FrameRateTarget Status="Valid">30</FrameRateTarget>)",
    R"(<FrameRateTarget.Type IsReadOnly="true" Status="Valid">DoubleType</FrameRateTarget.Type>)",
    R"(<FrameRateTarget.Default IsReadOnly="true" Status="Valid">30</FrameRateTarget.Default>)",
    R"(<FrameRateTarget.Min IsReadOnly="true" Status="Valid">1</FrameRateTarget.Min>)",
    R"(<FrameRateTarget.Max IsReadOnly="true" Status="Valid">100</FrameRateTarget.Max>)",
    R"(<FrameRateTarget.Increment IsReadOnly="true" Status="Valid">1</FrameRateTarget.Increment>)",
    R"(<FrameRateTarget.Label IsReadOnly="true" Status="Valid">Acquire|Live Frame Rate Max</FrameRateTarget.Label>)",
    R"(<FrameRateTarget.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</FrameRateTarget.GuiHint>)",
    R"(<FrameRateTarget.Mode IsReadOnly="true" Status="Valid">Expert</FrameRateTarget.Mode>)",
    R"(<FrameRateTarget.Enabled IsReadOnly="true" Status="Valid">false</FrameRateTarget.Enabled>)",
    R"(<FrameRateTarget.DisplayOrder IsReadOnly="true" Status="Valid">201</FrameRateTarget.DisplayOrder>)",
    R"(<ImageOrientation.Enabled IsReadOnly="true" Status="Valid">true</ImageOrientation.Enabled>)",
    R"(<NoiseFilter Status="Valid">false</NoiseFilter>)",
    R"(<NoiseFilter.Type IsReadOnly="true" Status="Valid">BoolType</NoiseFilter.Type>)",
    R"(<NoiseFilter.Default IsReadOnly="true" Status="Valid">false</NoiseFilter.Default>)",
    R"(<NoiseFilter.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</NoiseFilter.GuiHint>)",
    R"(<NoiseFilterThreshold Status="Valid">1.5</NoiseFilterThreshold>)",
    R"(<NoiseFilterThreshold.Type IsReadOnly="true" Status="Valid">DoubleType</NoiseFilterThreshold.Type>)",
    R"(<NoiseFilterThreshold.Default IsReadOnly="true" Status="Valid">1.5</NoiseFilterThreshold.Default>)",
    R"(<NoiseFilterThreshold.Min IsReadOnly="true" Status="Valid">0</NoiseFilterThreshold.Min>)",
    R"(<NoiseFilterThreshold.Max IsReadOnly="true" Status="Valid">10</NoiseFilterThreshold.Max>)",
    R"(<NoiseFilterThreshold.Increment IsReadOnly="true" Status="Valid">0.1</NoiseFilterThreshold.Increment>)",
    R"(<NoiseFilterThreshold.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</NoiseFilterThreshold.GuiHint>)",
    R"(<NoiseFilter.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilter.Enabled>)",
    R"(<NoiseFilterThreshold.Enabled IsReadOnly="true" Status="Valid">true</NoiseFilterThreshold.Enabled>)",
    R"(<ReferenceImageCount Status="Valid">3</ReferenceImageCount>)",
    R"(<ReferenceImageCount.Type IsReadOnly="true" Status="Valid">IntegerType</ReferenceImageCount.Type>)",
    R"(<ReferenceImageCount.Default IsReadOnly="true" Status="Valid">3</ReferenceImageCount.Default>)",
    R"(<ReferenceImageCount.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ReferenceImageCount.GuiHint>)",
    R"(<ReferenceImageCount.Label IsReadOnly="true" Status="Valid">Post Processing|Reference Image Count</ReferenceImageCount.Label>)",
    R"(<ReferenceImageCount.Enabled IsReadOnly="true" Status="Valid">true</ReferenceImageCount.Enabled>)",
    R"(<ReferenceImageCount.Min IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Min>)",
    R"(<ReferenceImageCount.Max IsReadOnly="true" Status="Valid">5</ReferenceImageCount.Max>)",
    R"(<ReferenceImageCount.Increment IsReadOnly="true" Status="Valid">1</ReferenceImageCount.Increment>)",
    R"(<ReferenceImageCount.Mode IsReadOnly="true" Status="Valid">Expert</ReferenceImageCount.Mode>)",
    R"(<ShadingReference Status="Valid">true</ShadingReference>)",
    R"(<ShadingReference.Type IsReadOnly="true" Status="Valid">BoolType</ShadingReference.Type>)",
    R"(<ShadingReference.Default IsReadOnly="true" Status="Valid">false</ShadingReference.Default>)",
    R"(<ShadingReference.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingReference.GuiHint>)",
    R"(<ShadingReference.Enabled IsReadOnly="true" Status="Valid">false</ShadingReference.Enabled>)",
    R"(<ShadingStrength Status="Valid">1</ShadingStrength>)",
    R"(<ShadingStrength.Type IsReadOnly="true" Status="Valid">DoubleType</ShadingStrength.Type>)",
    R"(<ShadingStrength.Default IsReadOnly="true" Status="Valid">1</ShadingStrength.Default>)",
    R"(<ShadingStrength.Min IsReadOnly="true" Status="Valid">0.1</ShadingStrength.Min>)",
    R"(<ShadingStrength.Max IsReadOnly="true" Status="Valid">2</ShadingStrength.Max>)",
    R"(<ShadingStrength.Increment IsReadOnly="true" Status="Valid">0.01</ShadingStrength.Increment>)",
    R"(<ShadingStrength.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</ShadingStrength.GuiHint>)",
    R"(<ShadingStrength.Mode IsReadOnly="true" Status="Valid">Expert</ShadingStrength.Mode>)",
    R"(<ShadingStrength.Enabled IsReadOnly="true" Status="Valid">true</ShadingStrength.Enabled>)",
    R"(<ShadingStrength.Label IsReadOnly="true" Status="Valid">Shading Strength</ShadingStrength.Label>)",
    R"(<ShadingUseBias Status="Valid">false</ShadingUseBias>)",
    R"(<ShadingUseBias.Type IsReadOnly="true" Status="Valid">BoolType</ShadingUseBias.Type>)",
    R"(<ShadingUseBias.Default IsReadOnly="true" Status="Valid">false</ShadingUseBias.Default>)",
    R"(<ShadingUseBias.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ShadingUseBias.GuiHint>)",
    R"(<ShadingUseBias.Mode IsReadOnly="true" Status="Valid">Expert</ShadingUseBias.Mode>)",
    R"(<ShadingUseBias.Enabled IsReadOnly="true" Status="Valid">true</ShadingUseBias.Enabled>)",
    R"(<ShadingUseBias.Label IsReadOnly="true" Status="Valid">Shading Use Bias</ShadingUseBias.Label>)",
    R"(<LiveImageFrame.Max IsReadOnly="true" Status="Valid">0,0,1600,1200</LiveImageFrame.Max>)",
    R"(<Resolution.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</Resolution.GuiHint>)",
    R"(<Resolution Status="Valid">0</Resolution>)",
    R"(<Resolution.Type IsReadOnly="true" Status="Valid">IntegerType</Resolution.Type>)",
    R"(<Resolution.Default IsReadOnly="true" Status="Valid">0</Resolution.Default>)",
    R"(<Resolution.ListItems IsReadOnly="true" Status="Valid">0|1600 x 1200  8bit|1600 x 1200  8bit,1|1600 x 1200 10bit|1600 x 1200 10bit,2|1600 x 1200 12bit|1600 x 1200 12bit</Resolution.ListItems>)",
    R"(<Resolution.Enabled IsReadOnly="true" Status="Valid">true</Resolution.Enabled>)",
    R"(<UnsharpMask Status="Valid">false</UnsharpMask>)",
    R"(<UnsharpMask.Type IsReadOnly="true" Status="Valid">BoolType</UnsharpMask.Type>)",
    R"(<UnsharpMask.Default IsReadOnly="true" Status="Valid">false</UnsharpMask.Default>)",
    R"(<UnsharpMask.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</UnsharpMask.GuiHint>)",
    R"(<UnsharpMaskState Status="Valid" />)",
    R"(<UnsharpMaskState.Type IsReadOnly="true" Status="Valid">StringType</UnsharpMaskState.Type>)",
    R"(<UnsharpMaskState.Default IsReadOnly="true" Status="Valid" />)",
    R"(<UnsharpMask.Enabled IsReadOnly="true" Status="Valid">true</UnsharpMask.Enabled>)",
    R"(<ImageFrame.Max IsReadOnly="true" Status="Valid">0,0,1600,1200</ImageFrame.Max>)",
    R"(<Frame.Enabled IsReadOnly="true" Status="Valid">false</Frame.Enabled>)",
    R"(<ExposureTime.Enabled IsReadOnly="true" Status="Valid">true</ExposureTime.Enabled>)",
    R"(<WhiteBalanceOffset.Type IsReadOnly="true" Status="Valid">DoubleType</WhiteBalanceOffset.Type>)",
    R"(<WhiteBalanceOffset.Default IsReadOnly="true" Status="Valid">0</WhiteBalanceOffset.Default>)",
    R"(<WhiteBalanceOffset.Min IsReadOnly="true" Status="Valid">-2</WhiteBalanceOffset.Min>)",
    R"(<WhiteBalanceOffset.Max IsReadOnly="true" Status="Valid">2</WhiteBalanceOffset.Max>)",
    R"(<WhiteBalanceOffset.Increment IsReadOnly="true" Status="Valid">0.1</WhiteBalanceOffset.Increment>)",
    R"(<WhiteBalanceOffset.GuiHint IsReadOnly="true" Status="Valid">GuiSlider</WhiteBalanceOffset.GuiHint>)",
    R"(<WhiteBalance.Enabled IsReadOnly="true" Status="Valid">true</WhiteBalance.Enabled>)",
    R"(<WhiteBalanceOffset.Enabled IsReadOnly="true" Status="Valid">true</WhiteBalanceOffset.Enabled>)",
    R"(<WhiteBalance.ListItems IsReadOnly="true" Status="Valid">0|1&amp;comma;0.73186874338281&amp;comma;0.35563224299028|3200K,1|0.401465&amp;comma;0.5458076&amp;comma;1|6500K</WhiteBalance.ListItems>)",
    R"(<ColorMode Status="Valid">1</ColorMode>)",
    R"(<ColorMode.Type IsReadOnly="true" Status="Valid">IntegerType</ColorMode.Type>)",
    R"(<ColorMode.ListItems IsReadOnly="true" Status="Valid">0|0|Monochrome,1|1|Color</ColorMode.ListItems>)",
    R"(<ColorMode.Default IsReadOnly="true" Status="Valid">1</ColorMode.Default>)",
    R"(<ColorMode.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</ColorMode.GuiHint>)",
    R"(<ColorMode.Enabled IsReadOnly="true" Status="Valid">true</ColorMode.Enabled>)",
    R"(<ApplyCameraProfile.Enabled IsReadOnly="true" Status="Valid">true</ApplyCameraProfile.Enabled>)",
    R"(<ApplyCameraProfile.Label IsReadOnly="true" Status="Valid">Post Processing|Apply Color Profile</ApplyCameraProfile.Label>)",
    R"(<ApplyCameraProfile.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</ApplyCameraProfile.GuiHint>)",
    R"(<ApplyCameraProfile.Mode IsReadOnly="true" Status="Valid">Expert</ApplyCameraProfile.Mode>)",
    R"(<CameraProfile Status="Valid">*srgb</CameraProfile>)",
    R"(<CameraProfile.Type IsReadOnly="true" Status="Valid">StringType</CameraProfile.Type>)",
    R"(<CameraProfile.Default IsReadOnly="true" Status="Valid">*srgb</CameraProfile.Default>)",
    R"(<CameraProfile.GuiHint IsReadOnly="true" Status="Valid">GuiTextBox</CameraProfile.GuiHint>)",
    R"(<CameraProfile.Enabled IsReadOnly="true" Status="Valid">true</CameraProfile.Enabled>)",
    R"(<CameraProfile.Label IsReadOnly="true" Status="Valid">Post Processing|Color Profile</CameraProfile.Label>)",
    R"(<CameraProfile.Mode IsReadOnly="true" Status="Valid">Expert</CameraProfile.Mode>)",
    R"(<Trigger Status="Valid">true</Trigger>)",
    R"(<Trigger.Type IsReadOnly="true" Status="Valid">BoolType</Trigger.Type>)",
    R"(<Trigger.Default IsReadOnly="true" Status="Valid">false</Trigger.Default>)",
    R"(<Trigger.GuiHint IsReadOnly="true" Status="Valid">GuiCheckBox</Trigger.GuiHint>)",
    R"(<Trigger.Enabled IsReadOnly="true" Status="Valid">true</Trigger.Enabled>)",
    R"(<TriggerMode Status="Valid">2</TriggerMode>)",
    R"(<TriggerMode.Type IsReadOnly="true" Status="Valid">IntegerType</TriggerMode.Type>)",
    R"(<TriggerMode.ListItems IsReadOnly="true" Status="Valid">0|Active High - Camera Link|Active High - Camera Link,1|Active Low  - Camera Link|Active Low  - Camera Link,2|Active High - 12 Pin Connector|Active High - 12 Pin Connector,3|Active Low  - 12 Pin Connector|Active Low  - 12 Pin Connector</TriggerMode.ListItems>)",
    R"(<TriggerMode.Default IsReadOnly="true" Status="Valid">0</TriggerMode.Default>)",
    R"(<TriggerMode.GuiHint IsReadOnly="true" Status="Valid">GuiComboBox</TriggerMode.GuiHint>)",
    R"(<TriggerMode.Label IsReadOnly="true" Status="Valid">Acquire|Trigger Mode</TriggerMode.Label>)",
    R"(<TriggerMode.Mode IsReadOnly="true" Status="Valid">Expert</TriggerMode.Mode>)",
    R"(<TriggerMode.Enabled IsReadOnly="true" Status="Valid">true</TriggerMode.Enabled>)",
    R"(<CameraIdentifier.DisplayOrder IsReadOnly="true" Status="Valid">-999</CameraIdentifier.DisplayOrder>)",
    R"(<CameraIdentifier.Label IsReadOnly="true" Status="Valid">Camera Identifier</CameraIdentifier.Label>)",
    R"(<AutomaticTriggerControl Status="Valid">false</AutomaticTriggerControl>)",
    R"(<AutomaticTriggerControl.Type IsReadOnly="true" Status="Valid">BoolType</AutomaticTriggerControl.Type>)",
    R"(<AutomaticTriggerControl.Default IsReadOnly="true" Status="Valid">true</AutomaticTriggerControl.Default>)",
    R"(<AutomaticTriggerControl.Enabled IsReadOnly="true" Status="Valid">true</AutomaticTriggerControl.Enabled>)",
    R"(<AutomaticTriggerControl.Label IsReadOnly="true" Status="Valid">Acquire|Auto Trigger Control</AutomaticTriggerControl.Label>)",
    R"(<AutomaticTriggerControl.Mode IsReadOnly="true" Status="Valid">Expert</AutomaticTriggerControl.Mode>)",
    R"(<AutomaticTriggerControl.GuiHint IsReadOnly="true" Status="Valid">GuiNone</AutomaticTriggerControl.GuiHint>)",
    R"(<TheoreticalTotalMagnification IsActivated="true" IsReadOnly="true" Status="Valid">40</TheoreticalTotalMagnification>)",
    R"(<TotalMagnification IsActivated="true" IsReadOnly="true" Status="Valid">39.55571</TotalMagnification>)",
    R"(<BlackReference.Enabled IsReadOnly="true" Status="Valid">false</BlackReference.Enabled>)",
    R"(<CameraGeometry Status="Valid" />)",
    R"(<JpegCompression Status="Valid">true</JpegCompression>)",
    R"(<HDRBits Status="Valid">0</HDRBits>)",
    R"(<SnapMode Status="Valid">0</SnapMode>)",
    R"(<ThreadCount Status="Valid" />)",
    R"(<LiveSpeed Status="Valid">0</LiveSpeed>)",
    R"(<HDRBits.Enabled IsReadOnly="true" Status="Valid">false</HDRBits.Enabled>)",
   R"(</Camera>)",
   R"(<Autofocus Id="Autofocus" Name="Autofocus" UniqueName="Autofocus" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<IsRunning Status="Valid">false</IsRunning>)",
   R"(</Autofocus>)",
   R"(<IOCard Id="IOCard" Name="IOCard" UniqueName="IOCard" Model="" IsAvailable="true" IsBroken="false" IsBrokenReason="" Motorization="Motorized" IsLightSource="false" IsLightSink="false">)",
    R"(<DigitalInLabels>)",
     R"(<DigitalInLabel Label="Digital In 7" />)",
     R"(<DigitalInLabel Label="Digital In 8" />)",
    R"(</DigitalInLabels>)",
    R"(<DigitalOutLabels>)",
     R"(<DigitalOutLabel Label="TL Shutter" />)",
     R"(<DigitalOutLabel Label="RL Shutter" />)",
     R"(<DigitalOutLabel Label="Digital Out 7" />)",
     R"(<DigitalOutLabel Label="Digital Out 8" />)",
    R"(</DigitalOutLabels>)",
    R"(<AnalogInLabels />)",
    R"(<AnalogOutLabels>)",
     R"(<AnalogOutLabel Label="Analog Out 1" />)",
     R"(<AnalogOutLabel Label="Analog Out 3" />)",
     R"(<AnalogOutLabel Label="Analog Out 4" />)",
    R"(</AnalogOutLabels>)",
    R"(<AnalogOutMinimums>)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
     R"(<AnalogOutMinimum Value="0" />)",
    R"(</AnalogOutMinimums>)",
    R"(<AnalogOutMaximums>)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
     R"(<AnalogOutMaximum Value="10" />)",
    R"(</AnalogOutMaximums>)",
    R"(<DigitalOutValue1 Status="Valid">false</DigitalOutValue1>)",
    R"(<DigitalOutMode1 Status="Valid">RisingEdge</DigitalOutMode1>)",
    R"(<DigitalOutDelay1 Status="Valid">0</DigitalOutDelay1>)",
    R"(<DigitalOutValue2 Status="Valid">false</DigitalOutValue2>)",
    R"(<DigitalOutMode2 Status="Valid">RisingEdge</DigitalOutMode2>)",
    R"(<DigitalOutDelay2 Status="Valid">0</DigitalOutDelay2>)",
    R"(<DigitalOutValue3 Status="Valid">false</DigitalOutValue3>)",
    R"(<DigitalOutMode3 Status="Valid">RisingEdge</DigitalOutMode3>)",
    R"(<DigitalOutDelay3 Status="Valid">0</DigitalOutDelay3>)",
    R"(<DigitalOutValue4 Status="Valid">false</DigitalOutValue4>)",
    R"(<DigitalOutMode4 Status="Valid">RisingEdge</DigitalOutMode4>)",
    R"(<DigitalOutDelay4 Status="Valid">0</DigitalOutDelay4>)",
    R"(<DigitalIn1 Status="Valid">false</DigitalIn1>)",
    R"(<DigitalIn2 Status="Valid">false</DigitalIn2>)",
    R"(<AnalogOut1 Status="Valid">0</AnalogOut1>)",
    R"(<AnalogOut2 Status="Valid">0</AnalogOut2>)",
    R"(<AnalogOut3 Status="Valid">0</AnalogOut3>)",
   R"(</IOCard>)",
  R"(</HardwareSetting>)",
  R"(<Experiment Version="1.1">)",
   R"(<RunMode>ValidateAndAdaptBeforePerformEnabled, OptimizeBeforePerformEnabled, PreventMissingCalibrationDataInformation</RunMode>)",
   R"(<BeforeHardwareSetting>Before Exp [FITC, TexRe] Smart</BeforeHardwareSetting>)",
   R"(<AfterHardwareSetting>After Exp [FITC, TexRe] Smart</AfterHardwareSetting>)",
   R"(<ExperimentBlockIndex>0</ExperimentBlockIndex>)",
   R"(<IsSegmented>false</IsSegmented>)",
   R"(<IsStandardMode>true</IsStandardMode>)",
   R"(<ImageName>D:\Images\qDOT Slides 40x\TMA1_Zeb1_SPRR2_Ck19_S100-1-1-1-1.czi</ImageName>)",
   R"(<AutoSave IsActivated="false">)",
    R"(<StorageFolder>D:\Users\zeiss\Pictures</StorageFolder>)",
    R"(<Name />)",
    R"(<IsAutoSubFolder>false</IsAutoSubFolder>)",
    R"(<IsClosedOnCompletion>false</IsClosedOnCompletion>)",
    R"(<AdditionalFilesFormat />)",
   R"(</AutoSave>)",
   R"(<BlockExecutionOrder>1</BlockExecutionOrder>)",
   R"(<LoopDefinitions />)",
   R"(<ExperimentBlocks>)",
    R"(<AcquisitionBlock IsActivated="true" BlockReference="635073549579201178">)",
     R"(<RunMode>ComponentUpdateEnabled, StateUpdateEnabled, HighLevelProtocolCreationEnabled, LowLevelProtocolCreationEnabled, DisableCheckFreeRAM, PauseAcquisitionOnProcessingGraphOverload</RunMode>)",
     R"(<SubDimensionSetups>)",
      R"(<RegionsSetup IsActivated="true">)",
       R"(<SampleHolder>)",
        R"(<Overlap>0.1</Overlap>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<IsConstantTiles>false</IsConstantTiles>)",
        R"(<TileRegionAnchorMode>Center</TileRegionAnchorMode>)",
        R"(<StageReferencePoint>0,0</StageReferencePoint>)",
        R"(<UseBacklashCorrection>false</UseBacklashCorrection>)",
        R"(<IsOptimizedTravelMovement>true</IsOptimizedTravelMovement>)",
        R"(<UseStageContinualSpeed>false</UseStageContinualSpeed>)",
        R"(<StageContinualSpeed>1</StageContinualSpeed>)",
        R"(<MoveToLoadPositionBetweenShapes>false</MoveToLoadPositionBetweenShapes>)",
        R"(<SplitScenesInSeparateFiles>false</SplitScenesInSeparateFiles>)",
        R"(<TileRegionCoveringMode>AlignedToGlobalGrid</TileRegionCoveringMode>)",
        R"(<TileRegionAllowedScanArea>-59000,-1000,60000,27400</TileRegionAllowedScanArea>)",
        R"(<TemplateShapesFillGrade>1</TemplateShapesFillGrade>)",
        R"(<IsOnlineStitchingEnabled>false</IsOnlineStitchingEnabled>)",
        R"(<IsOnlinePyramidEnabled>true</IsOnlinePyramidEnabled>)",
        R"(<GlobalInterpolationExpansionDegree>3</GlobalInterpolationExpansionDegree>)",
        R"(<LocalInterpolationExpansionDegree>3</LocalInterpolationExpansionDegree>)",
        R"(<IsFocusSurfaceOutlierRemovingEnabled>true</IsFocusSurfaceOutlierRemovingEnabled>)",
        R"(<FocusSurfaceOutlierExpansionDegree>2</FocusSurfaceOutlierExpansionDegree>)",
        R"(<FocusSurfaceOutlierThresholdSigma>2</FocusSurfaceOutlierThresholdSigma>)",
        R"(<TileRegions>)",
         R"(<TileRegion Name="ScanRegion0" Id="635145958921324051">)",
          R"(<CenterPosition>-26169.273,7260.055</CenterPosition>)",
          R"(<ContourSize>3277.715,2952.579</ContourSize>)",
          R"(<Columns>12</Columns>)",
          R"(<Rows>10</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-25410.438,8736.345 -24741.283,8068.543 -24741.924,7260.1 -24671.459,7224.955 -24565.635,7330.411 -24530.416,7295.263 -24530.584,7084.365 -24671.654,6978.907 -24636.686,6627.412 -25483.275,5783.766 -25588.931,5889.209 -25765.136,5924.347 -26505.371,5854.002 -27385.84,6732.689 -27456.11,7013.882 -27526.547,7084.177 -27597.068,7049.023 -27596.873,7295.071 -27667.255,7435.665 -27808.13,7576.255 -27667.087,7646.563 -27631.562,7998.063 -26926.071,8701.101 -25410.438,8736.345 -25410.438,8736.345</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650012">)",
            R"(<X>-26468.358</X>)",
            R"(<Y>5958.528</Y>)",
            R"(<Z>3783.344</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650019">)",
            R"(<X>-27491.626</X>)",
            R"(<Y>7693.856</Y>)",
            R"(<Z>3783.344</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650020">)",
            R"(<X>-25773.014</X>)",
            R"(<Y>7693.856</Y>)",
            R"(<Z>3782.421</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion1" Id="635145958921334052">)",
          R"(<CenterPosition>-22591.615,7488.716</CenterPosition>)",
          R"(<ContourSize>2607.887,2636.221</ContourSize>)",
          R"(<Columns>10</Columns>)",
          R"(<Rows>10</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-22520.159,8806.826 -21498.816,7787.549 -21287.671,7365.766 -21393.634,7084.562 -21358.527,6908.815 -22099.292,6170.625 -22416.511,6170.605 -22557.414,6276.045 -22733.647,6276.034 -23790.209,7330.459 -23825.289,7541.355 -23754.712,7646.809 -23860.396,7717.101 -23895.559,7822.548 -23719.242,7928.008 -23683.912,8033.46 -22943.147,8771.65 -22520.159,8806.826 -22520.159,8806.826</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650014">)",
            R"(<X>-22646.072</X>)",
            R"(<Y>6343.872</Y>)",
            R"(<Z>3776.33</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650021">)",
            R"(<X>-23509.407</X>)",
            R"(<Y>8061.104</Y>)",
            R"(<Z>3776.33</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650022">)",
            R"(<X>-21919.713</X>)",
            R"(<Y>8061.104</Y>)",
            R"(<Z>3776.33</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion2" Id="635145958921344054">)",
          R"(<CenterPosition>-30134.686,7312.493</CenterPosition>)",
          R"(<ContourSize>3384.121,3620.442</ContourSize>)",
          R"(<Columns>12</Columns>)",
          R"(<Rows>13</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-29851.207,9122.714 -29604.536,9052.43 -29322.563,9052.447 -28547.751,8279.203 -28618.774,7611.354 -28442.625,7505.916 -28442.708,7400.467 -28654.272,7295.004 -28689.965,6732.607 -29607.102,5818.657 -29889.159,5713.191 -30206.546,5502.272 -30382.64,5678.01 -30770.185,5888.884 -31826.747,6943.309 -31826.05,7822.051 -31755.306,8138.403 -31790.385,8349.299 -31049.62,9087.489 -29851.207,9122.714 -29851.207,9122.714</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650011">)",
            R"(<X>-30275.314</X>)",
            R"(<Y>5680.8</Y>)",
            R"(<Z>3790.05</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650017">)",
            R"(<X>-31658.167</X>)",
            R"(<Y>7460.816</Y>)",
            R"(<Z>3792.77</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650018">)",
            R"(<X>-29995.984</X>)",
            R"(<Y>7460.816</Y>)",
            R"(<Z>3790.05</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion3" Id="635145958921364056">)",
          R"(<CenterPosition>-18308.982,7770.189</CenterPosition>)",
          R"(<ContourSize>3771.25,3269.015</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-17338.431,9404.696 -16528.4,8596.304 -16423.357,7717.568 -16529.404,7330.914 -17622.942,6206.056 -17904.915,6206.038 -18151.586,6276.322 -18363.149,6170.859 -18433.614,6206.005 -18821.383,6135.681 -18997.533,6241.119 -19103.3,6205.963 -19913.331,7014.355 -20124.504,7400.988 -20124.225,7752.485 -20194.607,7893.08 -20053.453,8103.987 -20053.286,8314.885 -19100.874,9263.987 -17338.431,9404.696 -17338.431,9404.696</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650013">)",
            R"(<X>-19056.408</X>)",
            R"(<Y>6311.68</Y>)",
            R"(<Z>3768.732</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650023">)",
            R"(<X>-18128.32</X>)",
            R"(<Y>8061.872</Y>)",
            R"(<Z>3768.227</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion4" Id="635145958921374058">)",
          R"(<CenterPosition>-14202.164,8086.815</CenterPosition>)",
          R"(<ContourSize>3312.876,2917.445</ContourSize>)",
          R"(<Columns>12</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-13461.19,9545.538 -12615.968,8666.848 -12545.726,8350.505 -12546.2,7752.961 -12652.163,7471.756 -13498.752,6628.11 -13780.725,6628.093 -14027.395,6698.377 -15120.013,6733.458 -15824.36,7471.557 -15858.602,8736.944 -15153.112,9439.983 -14025.22,9440.053 -13461.19,9545.538 -13461.19,9545.538</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650015">)",
            R"(<X>-15038.907</X>)",
            R"(<Y>6801.118</Y>)",
            R"(<Z>3765.792</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650016">)",
            R"(<X>-13470.109</X>)",
            R"(<Y>6801.118</Y>)",
            R"(<Z>3765.792</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650024">)",
            R"(<X>-15678.209</X>)",
            R"(<Y>8515.36</Y>)",
            R"(<Z>3765.792</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650025">)",
            R"(<X>-13923.653</X>)",
            R"(<Y>8515.36</Y>)",
            R"(<Z>3764.619</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion5" Id="635145958921384059">)",
          R"(<CenterPosition>-26360.424,11319.775</CenterPosition>)",
          R"(<ContourSize>3454.559,3058.117</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-25830.135,12848.834 -25618.738,12743.398 -25407.259,12743.411 -24738.103,12075.609 -24633.144,11091.424 -25903.054,9790.806 -27312.92,9790.717 -28087.704,10599.112 -27945.713,11864.51 -27980.792,12075.406 -27240.028,12813.596 -26499.876,12778.492 -25830.135,12848.834 -25830.135,12848.834</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650026">)",
            R"(<X>-27325.946</X>)",
            R"(<Y>9970.736</Y>)",
            R"(<Z>3785.087</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650032">)",
            R"(<X>-27782.891</X>)",
            R"(<Y>11769.408</Y>)",
            R"(<Z>3786.447</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650033">)",
            R"(<X>-26040.571</X>)",
            R"(<Y>11769.408</Y>)",
            R"(<Z>3783.727</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion6" Id="635145958921394061">)",
          R"(<CenterPosition>-30377.912,11583.16</CenterPosition>)",
          R"(<ContourSize>3101.562,2952.613</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-30059.562,13059.467 -30024.372,12989.17 -29671.905,12989.192 -28897.093,12215.947 -28862.014,12005.051 -28968.061,11618.398 -28827.13,11548.107 -29003.447,11442.647 -28968.591,10950.553 -29744.63,10177.211 -30555.247,10247.46 -30625.796,10177.156 -30590.605,10106.859 -30661.098,10106.854 -30837.248,10212.293 -31013.509,10177.132 -31823.54,10985.524 -31717.353,11547.926 -31858.2,11723.666 -31928.694,11723.661 -31928.61,11829.11 -31822.87,11829.117 -31822.703,12040.015 -31046.663,12813.357 -30694.141,12883.679 -30623.536,13024.282 -30482.606,12953.991 -30200.633,12954.009 -30059.562,13059.467 -30059.562,13059.467</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650027">)",
            R"(<X>-30958.84</X>)",
            R"(<Y>10281.616</Y>)",
            R"(<Z>3791.806</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650034">)",
            R"(<X>-31648.666</X>)",
            R"(<Y>12016.976</Y>)",
            R"(<Z>3791.126</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650035">)",
            R"(<X>-29906.942</X>)",
            R"(<Y>12016.976</Y>)",
            R"(<Z>3790.446</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion7" Id="635145958921414063">)",
          R"(<CenterPosition>-22711.38,12023.018</CenterPosition>)",
          R"(<ContourSize>2432.381,2284.769</ContourSize>)",
          R"(<Columns>9</Columns>)",
          R"(<Rows>8</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-22305.221,13165.402 -21495.19,12357.01 -21495.58,11864.914 -22236.345,11126.724 -22518.346,11091.557 -22624.17,10986.101 -22835.677,10950.938 -22906.226,10880.634 -23927.57,11899.911 -23751.225,12040.521 -23750.862,12497.467 -23080.646,13165.354 -22305.221,13165.402 -22305.221,13165.402</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650031">)",
            R"(<X>-22939.937</X>)",
            R"(<Y>11049.84</Y>)",
            R"(<Z>3777.754</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650043">)",
            R"(<X>-23378.503</X>)",
            R"(<Y>12718.144</Y>)",
            R"(<Z>3777.754</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion8" Id="635145958921424064">)",
          R"(<CenterPosition>-18922.535,11847.518</CenterPosition>)",
          R"(<ContourSize>3806.105,3269.024</ContourSize>)",
          R"(<Columns>14</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-17828.648,13482.03 -17089.054,12743.933 -17089.835,11759.741 -17019.482,11583.997 -17055.119,11091.899 -17866.405,10318.555 -18077.885,10318.542 -18148.434,10248.238 -18218.899,10283.383 -18289.448,10213.079 -19452.587,10213.006 -20403.493,11161.989 -20403.27,11443.186 -20508.786,11724.377 -20614.526,11724.371 -20684.74,12075.863 -20825.588,12251.603 -20825.392,12497.651 -19908.255,13411.6 -18604.13,13411.682 -17828.648,13482.03 -17828.648,13482.03</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650028">)",
            R"(<X>-19463.735</X>)",
            R"(<Y>10389.008</Y>)",
            R"(<Z>3770.449</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650036">)",
            R"(<X>-20568.063</X>)",
            R"(<Y>12139.232</Y>)",
            R"(<Z>3770.449</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650037">)",
            R"(<X>-18768.392</X>)",
            R"(<Y>12139.232</Y>)",
            R"(<Z>3770.449</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion9" Id="635145958921434066">)",
          R"(<CenterPosition>-30357.513,15590.203</CenterPosition>)",
          R"(<ContourSize>3207.553,3304.044</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-30972.656,17242.224 -30232.644,17031.372 -30021.164,17031.385 -29105.477,16117.55 -29105.728,15801.203 -28894.415,15590.318 -28965.132,15309.116 -28753.736,15203.68 -28753.931,14957.632 -29635.794,14078.835 -30199.74,14078.799 -30340.838,13938.192 -30517.071,13938.181 -30658.002,14008.471 -31045.743,13973.297 -31750.118,14676.247 -31749.894,14957.445 -31961.29,15062.88 -31961.179,15203.479 -31855.355,15308.935 -31855.3,15379.234 -31960.705,15801.024 -31960.203,16433.719 -31148.889,17242.213 -30972.656,17242.224 -30972.656,17242.224</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650045">)",
            R"(<X>-31043.778</X>)",
            R"(<Y>14115.76</Y>)",
            R"(<Z>3790.433</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650053">)",
            R"(<X>-31791.361</X>)",
            R"(<Y>15885.072</Y>)",
            R"(<Z>3792.473</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650054">)",
            R"(<X>-30113.747</X>)",
            R"(<Y>15885.072</Y>)",
            R"(<Z>3789.753</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion10" Id="635145958921454068">)",
          R"(<CenterPosition>-14534.246,12128.975</CenterPosition>)",
          R"(<ContourSize>3841.352,3339.318</ContourSize>)",
          R"(<Columns>14</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-13739.788,13798.634 -13634.076,13763.491 -12647.951,12779.361 -12683.505,12392.712 -12789.217,12427.855 -12859.85,12252.102 -12648.426,12181.816 -12613.57,11689.723 -12719.337,11654.566 -12825.217,11478.811 -13848.15,10494.555 -14130.095,10529.687 -15293.29,10459.315 -16173.758,11338.002 -16244.14,11478.597 -16173.367,11830.098 -16454.922,12357.326 -16454.615,12743.973 -16348.624,13060.327 -15643.134,13763.365 -13739.788,13798.634 -13739.788,13798.634</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650029">)",
            R"(<X>-15312.563</X>)",
            R"(<Y>10638.53</Y>)",
            R"(<Z>3766.974</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650038">)",
            R"(<X>-16282.136</X>)",
            R"(<Y>12427.04</Y>)",
            R"(<Z>3766.974</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650039">)",
            R"(<X>-14566.927</X>)",
            R"(<Y>12427.04</Y>)",
            R"(<Z>3766.974</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650040">)",
            R"(<X>-12851.719</X>)",
            R"(<Y>12427.04</Y>)",
            R"(<Z>3766.974</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion11" Id="635145958921464070">)",
          R"(<CenterPosition>-10674.238,12217.105</CenterPosition>)",
          R"(<ContourSize>3031.572,3304.107</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-10144.576,13869.159 -9968.371,13834.02 -9228.778,13095.923 -9158.452,12885.029 -9159.01,12182.035 -9265,11865.681 -9265.224,11584.483 -10111.812,10740.838 -10711.144,10565.052 -10816.856,10600.195 -10887.238,10740.789 -11204.513,10670.47 -12155.419,11619.452 -12119.866,12006.101 -12049.261,12146.704 -12190.024,12427.893 -12084.173,12568.498 -12084.089,12673.947 -12013.568,12709.101 -12013.484,12814.551 -11061.072,13763.652 -10638.029,13869.128 -10144.576,13869.159 -10144.576,13869.159</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650030">)",
            R"(<X>-11119.957</X>)",
            R"(<Y>10742.656</Y>)",
            R"(<Z>3763.054</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650041">)",
            R"(<X>-11962.317</X>)",
            R"(<Y>12511.968</Y>)",
            R"(<Z>3763.054</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650042">)",
            R"(<X>-10210.259</X>)",
            R"(<Y>12511.968</Y>)",
            R"(<Z>3762.374</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion12" Id="635145958921474071">)",
          R"(<CenterPosition>-26497.227,15537.753</CenterPosition>)",
          R"(<ContourSize>3524.16,3761.021</ContourSize>)",
          R"(<Columns>12</Columns>)",
          R"(<Rows>13</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-26319.961,17418.264 -25791.29,17383.147 -24840.384,16434.165 -24770.058,16223.271 -24805.528,15942.072 -24735.146,15801.477 -24735.285,15625.729 -24911.714,15379.67 -24876.635,15168.774 -25017.649,15133.615 -25017.733,15028.166 -24947.295,14957.871 -24947.463,14746.973 -25864.6,13833.023 -26252.285,13868.149 -26393.439,13657.242 -26534.286,13832.981 -27027.767,13797.801 -27908.235,14676.488 -27943.37,14817.084 -28154.683,15027.969 -28153.929,15977.011 -28259.307,16433.951 -27377.444,17312.749 -27095.415,17383.066 -26319.961,17418.264 -26319.961,17418.264</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650044">)",
            R"(<X>-26920.798</X>)",
            R"(<Y>13829.745</Y>)",
            R"(<Z>3784.319</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650051">)",
            R"(<X>-27980.17</X>)",
            R"(<Y>15253.056</Y>)",
            R"(<Z>3785.857</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650052">)",
            R"(<X>-26229.649</X>)",
            R"(<Y>15253.056</Y>)",
            R"(<Z>3782.959</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650063">)",
            R"(<X>-27570.297</X>)",
            R"(<Y>16961.072</Y>)",
            R"(<Z>3784.319</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650064">)",
            R"(<X>-25820.125</X>)",
            R"(<Y>16961.072</Y>)",
            R"(<Z>3782.279</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion13" Id="635145958921494073">)",
          R"(<CenterPosition>-30265.826,19456.726</CenterPosition>)",
          R"(<ContourSize>2891.227,3233.799</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-29630.243,21073.625 -28820.212,20265.233 -28891.152,19702.833 -28856.435,19034.991 -30055.769,17839.826 -30161.397,17980.418 -30302.384,17980.409 -30408.18,17910.103 -30513.78,18085.845 -30725.315,18015.533 -30901.521,18050.671 -31641.114,18788.769 -31711.44,18999.662 -31570.175,19351.168 -31499.04,20159.616 -30758.275,20897.806 -30582.042,20897.817 -30158.943,21073.592 -29630.243,21073.625 -29630.243,21073.625</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650067">)",
            R"(<X>-30311.71</X>)",
            R"(<Y>18014.256</Y>)",
            R"(<Z>3788.344</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660075">)",
            R"(<X>-31387.369</X>)",
            R"(<Y>19456.736</Y>)",
            R"(<Z>3789.704</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660076">)",
            R"(<X>-29636.138</X>)",
            R"(<Y>19456.736</Y>)",
            R"(<Z>3788.344</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion14" Id="635145958921504075">)",
          R"(<CenterPosition>-22496.847,15942.235</CenterPosition>)",
          R"(<ContourSize>3488.857,3725.847</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>13</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-22372.032,17805.158 -21209.786,16680.441 -21174.651,16539.844 -20857.682,16223.517 -20893.291,15766.568 -20752.417,15625.978 -20752.584,15415.08 -20928.929,15274.47 -20964.287,15133.869 -22022.522,14079.312 -23220.88,14114.387 -23890.008,14817.339 -23924.808,15379.732 -24135.87,15906.964 -24135.563,16293.611 -24206,16363.906 -24241.275,16328.754 -24241.219,16399.053 -24170.67,16469.357 -24205.861,16539.654 -24099.982,16715.409 -23077.02,17734.815 -22971.225,17805.121 -22372.032,17805.158 -22372.032,17805.158</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650046">)",
            R"(<X>-23188.553</X>)",
            R"(<Y>14250.463</Y>)",
            R"(<Z>3777.284</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650055">)",
            R"(<X>-23962.157</X>)",
            R"(<Y>15942.208</Y>)",
            R"(<Z>3778.644</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650056">)",
            R"(<X>-22216.181</X>)",
            R"(<Y>15942.208</Y>)",
            R"(<Z>3776.604</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650065">)",
            R"(<X>-23044.713</X>)",
            R"(<Y>17634.016</Y>)",
            R"(<Z>3777.284</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion15" Id="635145958921524077">)",
          R"(<CenterPosition>-14760.085,16118.461</CenterPosition>)",
          R"(<ContourSize>3735.501,3304.051</ContourSize>)",
          R"(<Columns>14</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-14758.788,17770.486 -14124.851,17137.831 -13842.99,16997.25 -12927.303,16083.415 -12892.335,15731.92 -12962.884,15661.617 -12962.995,15521.018 -14021.231,14466.46 -14197.464,14466.449 -14267.874,14571.894 -14408.944,14466.436 -14620.368,14536.722 -15113.849,14501.542 -15219.533,14571.834 -15748.26,14536.652 -16487.826,15309.899 -16417.026,15696.55 -16487.435,15801.994 -16487.268,16012.893 -16627.836,16540.129 -16592.394,16786.179 -15745.805,17629.825 -15534.326,17629.838 -15393.256,17735.296 -15181.748,17770.459 -14758.788,17770.486 -14758.788,17770.486</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650048">)",
            R"(<X>-15694.315</X>)",
            R"(<Y>14644.016</Y>)",
            R"(<Z>3767.659</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650049">)",
            R"(<X>-13998.994</X>)",
            R"(<Y>14644.016</Y>)",
            R"(<Z>3767.659</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650059">)",
            R"(<X>-16418.665</X>)",
            R"(<Y>16413.344</Y>)",
            R"(<Z>3767.659</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650060">)",
            R"(<X>-14620.922</X>)",
            R"(<Y>16413.344</Y>)",
            R"(<Z>3766.979</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion16" Id="635145958921544079">)",
          R"(<CenterPosition>-18830.932,16083.037</CenterPosition>)",
          R"(<ContourSize>3489.332,3936.778</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>14</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-18882.421,18051.425 -17086.265,16258.903 -17086.516,15942.556 -17192.312,15872.25 -17192.451,15696.501 -17686.295,15204.374 -17721.681,15028.624 -18603.544,14149.826 -19061.778,14114.647 -20541.02,15520.543 -20540.658,15977.489 -20399.616,16047.797 -20399.56,16118.096 -20470.053,16118.092 -20469.941,16258.691 -20575.598,16364.133 -20258.127,16680.5 -20222.685,16926.551 -19093.9,18051.412 -18882.421,18051.425 -18882.421,18051.425</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650047">)",
            R"(<X>-19109.187</X>)",
            R"(<Y>14293.92</Y>)",
            R"(<Z>3771.734</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650057">)",
            R"(<X>-20221.891</X>)",
            R"(<Y>16083.008</Y>)",
            R"(<Z>3773.094</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650058">)",
            R"(<X>-18447.217</X>)",
            R"(<Y>16083.008</Y>)",
            R"(<Z>3770.84</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650066">)",
            R"(<X>-19123.608</X>)",
            R"(<Y>17872.176</Y>)",
            R"(<Z>3771.734</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion17" Id="635145958921564081">)",
          R"(<CenterPosition>-22582.047,19826.249</CenterPosition>)",
          R"(<ContourSize>3172.281,3128.385</ContourSize>)",
          R"(<Columns>12</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-22157.707,21390.441 -22016.832,21249.851 -21840.655,21179.563 -21030.624,20371.17 -20995.907,19703.328 -21066.512,19562.725 -21066.652,19386.976 -21137.312,19176.074 -21948.626,18367.58 -22865.066,18332.373 -22935.615,18262.069 -23147.095,18262.056 -24062.782,19175.89 -24062.643,19351.639 -24133.08,19421.934 -24168.188,19597.68 -24167.992,19843.728 -24097.415,19949.182 -24167.881,19984.327 -24167.658,20265.524 -24061.806,20406.13 -24061.639,20617.028 -23320.874,21355.218 -22298.749,21320.133 -22157.707,21390.441 -22157.707,21390.441</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410660070">)",
            R"(<X>-23162.653</X>)",
            R"(<Y>18431.682</Y>)",
            R"(<Z>3776.337</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660079">)",
            R"(<X>-23999.181</X>)",
            R"(<Y>20105.168</Y>)",
            R"(<Z>3776.337</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660080">)",
            R"(<X>-22310.484</X>)",
            R"(<Y>20105.168</Y>)",
            R"(<Z>3775.657</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion18" Id="635145958921574083">)",
          R"(<CenterPosition>-26494.869,19667.804</CenterPosition>)",
          R"(<ContourSize>3666.234,3655.627</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>13</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-26492.959,21495.618 -26422.521,21425.323 -26175.795,21425.339 -26034.864,21355.048 -25999.729,21214.451 -25858.743,21214.46 -24837.399,20195.183 -24767.018,20054.589 -24661.752,19457.05 -24662.087,19035.254 -24767.938,18894.649 -24768.022,18789.199 -25649.885,17910.402 -26989.285,17875.168 -26812.968,17980.628 -26848.159,18050.925 -27306.365,18050.897 -27482.682,17945.437 -27412.244,17875.142 -27130.271,17875.159 -27447.519,17839.99 -28327.987,18718.677 -28327.373,19491.97 -28221.131,20124.672 -27303.994,21038.621 -27057.184,21144.086 -26986.607,21249.539 -26739.852,21284.704 -26492.959,21495.618 -26492.959,21495.618</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650068">)",
            R"(<X>-27487.952</X>)",
            R"(<Y>18019.964</Y>)",
            R"(<Z>3783.259</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660077">)",
            R"(<X>-28099.721</X>)",
            R"(<Y>19817.568</Y>)",
            R"(<Z>3784.619</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660078">)",
            R"(<X>-26353.393</X>)",
            R"(<Y>19817.568</Y>)",
            R"(<Z>3781.899</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion19" Id="635145958921594085">)",
          R"(<CenterPosition>-19056.562,19984.623</CenterPosition>)",
          R"(<ContourSize>3031.098,3163.496</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-19267.344,21566.371 -18703.51,21425.807 -18315.825,21390.682 -17541.013,20617.437 -17577.347,19246.596 -18282.837,18543.558 -18846.756,18578.673 -19622.321,18402.875 -20432.352,19211.268 -20396.408,20090.012 -20572.111,20757.846 -19760.797,21566.34 -19267.344,21566.371 -19267.344,21566.371</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410660072">)",
            R"(<X>-19653.682</X>)",
            R"(<Y>18574.098</Y>)",
            R"(<Z>3770.457</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660081">)",
            R"(<X>-20276.39</X>)",
            R"(<Y>20266.736</Y>)",
            R"(<Z>3772.414</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660082">)",
            R"(<X>-18572.235</X>)",
            R"(<Y>20266.736</Y>)",
            R"(<Z>3769.777</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion20" Id="635145958921604087">)",
          R"(<CenterPosition>-11023.706,16698.647</CenterPosition>)",
          R"(<ContourSize>2891.617,3971.99</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>14</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-10528.468,18684.643 -9683.218,17841.104 -9577.897,17313.865 -9578.12,17032.667 -9648.808,16786.615 -9649.199,16294.519 -9578.929,16013.326 -9614.566,15521.228 -10284.782,14853.341 -10425.824,14783.033 -10637.332,14747.87 -10743.016,14818.163 -10954.496,14818.15 -11201.278,14747.835 -11236.496,14782.982 -11730.005,14712.652 -12434.38,15415.602 -12469.515,15556.198 -12398.045,16786.442 -12291.971,17208.245 -12185.98,17524.599 -11163.019,18544.005 -10916.292,18544.02 -10528.468,18684.643 -10528.468,18684.643</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410650050">)",
            R"(<X>-11759.619</X>)",
            R"(<Y>14893.28</Y>)",
            R"(<Z>3762.914</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650061">)",
            R"(<X>-12224.209</X>)",
            R"(<Y>16698.608</Y>)",
            R"(<Z>3762.914</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410650062">)",
            R"(<X>-10427.56</X>)",
            R"(<Y>16698.608</Y>)",
            R"(<Z>3761.554</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660071">)",
            R"(<X>-11050.537</X>)",
            R"(<Y>18504.032</Y>)",
            R"(<Z>3761.554</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion21" Id="635145958921624089">)",
          R"(<CenterPosition>-15214.917,20125.466</CenterPosition>)",
          R"(<ContourSize>3454.533,3304.111</ContourSize>)",
          R"(<Columns>13</Columns>)",
          R"(<Rows>12</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-15249.061,21777.521 -14896.651,21707.243 -14755.748,21601.803 -14473.831,21531.521 -14403.421,21426.077 -14262.463,21390.936 -13487.65,20617.691 -13488.292,19809.248 -14370.155,18930.45 -14616.937,18860.135 -14757.924,18860.127 -14863.608,18930.419 -15110.362,18895.254 -15392.502,18684.338 -15639.257,18649.173 -15850.876,18473.411 -16837.028,19422.391 -16836.805,19703.589 -16942.183,20160.528 -16836.359,20265.984 -16800.861,20582.334 -16624.544,20687.794 -16659.735,20758.091 -15672.049,21742.345 -15425.35,21707.21 -15249.061,21777.521 -15249.061,21777.521</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410660073">)",
            R"(<X>-15904.03</X>)",
            R"(<Y>18651.04</Y>)",
            R"(<Z>3766.165</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660083">)",
            R"(<X>-16642.388</X>)",
            R"(<Y>20420.355</Y>)",
            R"(<Z>3767.525</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660084">)",
            R"(<X>-14856.614</X>)",
            R"(<Y>20420.355</Y>)",
            R"(<Z>3765.09</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
         R"(<TileRegion Name="ScanRegion22" Id="635145958921634090">)",
          R"(<CenterPosition>-11179.261,20547.534</CenterPosition>)",
          R"(<ContourSize>2995.824,3022.861</ContourSize>)",
          R"(<Columns>11</Columns>)",
          R"(<Rows>11</Rows>)",
          R"(<Z>3775.285</Z>)",
          R"(<TemplateShapeId />)",
          R"(<IsUsedForAcquisition>true</IsUsedForAcquisition>)",
          R"(<IsProtected>false</IsProtected>)",
          R"(<AreSupportPointsOutsideContourAllowed>true</AreSupportPointsOutsideContourAllowed>)",
          R"(<Contour Type="Polygon">)",
           R"(<Points>-11336.462,22058.964 -11230.806,21953.521 -10666.86,21953.557 -9892.048,21180.312 -9857.08,20828.817 -9681.349,20196.133 -9681.433,20090.684 -10492.747,19282.19 -10951.037,19176.713 -11127.382,19036.103 -11444.573,19071.233 -12430.698,20055.362 -12677.173,20371.694 -12641.313,21144.99 -11724.175,22058.939 -11512.751,21988.653 -11336.462,22058.964 -11336.462,22058.964</Points>)",
          R"(</Contour>)",
          R"(<SupportPoints>)",
           R"(<SupportPoint Id="635145962410660074">)",
            R"(<X>-11436.466</X>)",
            R"(<Y>19214.416</Y>)",
            R"(<Z>3760.285</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660085">)",
            R"(<X>-12483.573</X>)",
            R"(<Y>20991.888</Y>)",
            R"(<Z>3760.285</Z>)",
           R"(</SupportPoint>)",
           R"(<SupportPoint Id="635145962410660086">)",
            R"(<X>-10852.762</X>)",
            R"(<Y>20991.888</Y>)",
            R"(<Z>3759.605</Z>)",
           R"(</SupportPoint>)",
          R"(</SupportPoints>)",
         R"(</TileRegion>)",
        R"(</TileRegions>)",
        R"(<SingleTileRegions />)",
        R"(<SingleTileRegionArrays />)",
        R"(<Template Name="Untitled404">)",
         R"(<AnchorPoint>0,0</AnchorPoint>)",
         R"(<BoundsSize>0,0</BoundsSize>)",
         R"(<Category>Slide</Category>)",
         R"(<HasRegularShapes>true</HasRegularShapes>)",
         R"(<ReferencePoint>0,0</ReferencePoint>)",
         R"(<ReferencePointLocationMode>Center</ReferencePointLocationMode>)",
         R"(<ShapeType>Ellipse</ShapeType>)",
         R"(<ShapeColumns>0</ShapeColumns>)",
         R"(<ShapeRows>0</ShapeRows>)",
         R"(<ShapeWidth>10000</ShapeWidth>)",
         R"(<ShapeHeight>10000</ShapeHeight>)",
         R"(<ShapeDistanceX>20000</ShapeDistanceX>)",
         R"(<ShapeDistanceY>20000</ShapeDistanceY>)",
         R"(<OriginalShapeWidth>0</OriginalShapeWidth>)",
         R"(<OriginalShapeHeight>0</OriginalShapeHeight>)",
         R"(<OriginalShapeDistanceX>0</OriginalShapeDistanceX>)",
         R"(<OriginalShapeDistanceY>0</OriginalShapeDistanceY>)",
         R"(<AreShapesCenteredX>true</AreShapesCenteredX>)",
         R"(<AreShapesCenteredY>true</AreShapesCenteredY>)",
         R"(<ShapesOffsetX>0</ShapesOffsetX>)",
         R"(<ShapesOffsetY>0</ShapesOffsetY>)",
         R"(<Rotation>0</Rotation>)",
         R"(<SupportPoints>)",
          R"(<SupportPoint Id="635145962410650011">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3790.05</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650012">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3783.344</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650013">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3768.732</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650014">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.33</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650015">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3765.792</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650016">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3765.792</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650017">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3792.77</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650018">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3790.05</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650019">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3783.344</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650020">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3782.421</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650021">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.33</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650022">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.33</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650023">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3768.227</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650024">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3765.792</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650025">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3764.619</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650026">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3785.087</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650027">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3791.806</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650028">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3770.449</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650029">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.974</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650030">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3763.054</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650031">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3777.754</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650032">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3786.447</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650033">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3783.727</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650034">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3791.126</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650035">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3790.446</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650036">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3770.449</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650037">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3770.449</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650038">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.974</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650039">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.974</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650040">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.974</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650041">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3763.054</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650042">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3762.374</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650043">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3777.754</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650044">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3784.319</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650045">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3790.433</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650046">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3777.284</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650047">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3771.734</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650048">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3767.659</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650049">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3767.659</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650050">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3762.914</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650051">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3785.857</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650052">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3782.959</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650053">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3792.473</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650054">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3789.753</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650055">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3778.644</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650056">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.604</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650057">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3773.094</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650058">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3770.84</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650059">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3767.659</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650060">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.979</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650061">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3762.914</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650062">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3761.554</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650063">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3784.319</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650064">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3782.279</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650065">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3777.284</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650066">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3771.734</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650067">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3788.344</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410650068">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3783.259</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660070">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.337</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660071">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3761.554</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660072">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3770.457</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660073">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3766.165</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660074">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3760.285</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660075">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3789.704</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660076">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3788.344</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660077">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3784.619</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660078">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3781.899</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660079">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3776.337</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660080">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3775.657</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660081">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3772.414</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660082">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3769.777</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660083">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3767.525</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660084">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3765.09</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660085">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3760.285</Z>)",
          R"(</SupportPoint>)",
          R"(<SupportPoint Id="635145962410660086">)",
           R"(<X>0</X>)",
           R"(<Y>0</Y>)",
           R"(<Z>3759.605</Z>)",
          R"(</SupportPoint>)",
         R"(</SupportPoints>)",
        R"(</Template>)",
       R"(</SampleHolder>)",
       R"(<SubDimensionSetups>)",
        R"(<TilesSetup IsActivated="true">)",
         R"(<SubDimensionSetups>)",
          R"(<MultiTrackSetup IsActivated="true">)",
           R"(<IsTIRFAngleSynchronized>true</IsTIRFAngleSynchronized>)",
           R"(<Track IsActivated="true" IsSelected="false">)",
            R"(<BeforeHardwareSetting>Before [DAPI]_2</BeforeHardwareSetting>)",
            R"(<AfterHardwareSetting>After [DAPI]_2</AfterHardwareSetting>)",
            R"(<ZStackMode>Yes</ZStackMode>)",
            R"(<UseZStackCenter>true</UseZStackCenter>)",
            R"(<ZStackSliceIndex>1</ZStackSliceIndex>)",
            R"(<IsDisabledForApoTome>false</IsDisabledForApoTome>)",
            R"(<FocusOffset>0</FocusOffset>)",
            R"(<DirectDevice>MTBHXPLamp</DirectDevice>)",
            R"(<Channels>)",
             R"(<Channel IsActivated="true" IsSelected="false" ChannelSetupId="635145930996145747" InstanceId="635145930996145747" Name="DAPI" Description="">)",
              R"(<Color>#FF0000FF</Color>)",
              R"(<PixelShiftX>0</PixelShiftX>)",
              R"(<PixelShiftY>0</PixelShiftY>)",
              R"(<DataGrabberSetup>)",
               R"(<CameraFrameSetup Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
                R"(<ExposureTime IsActivated="true">15</ExposureTime>)",
                R"(<Adjust IsActivated="true">0.5</Adjust>)",
                R"(<AutoExposure IsActivated="true">false</AutoExposure>)",
                R"(<EMGain>50</EMGain>)",
                R"(<ShadingReference IsActivated="true">false</ShadingReference>)",
               R"(</CameraFrameSetup>)",
              R"(</DataGrabberSetup>)",
              R"(<AdditionalDyeInformation>)",
               R"(<ShortName>DAPI</ShortName>)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<DyeMaxEmission>465</DyeMaxEmission>)",
               R"(<DyeMaxExcitation>353</DyeMaxExcitation>)",
               R"(<DyeId>McNamara-Boswell-0434</DyeId>)",
               R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
              R"(</AdditionalDyeInformation>)",
              R"(<FluorescenceDye>)",
               R"(<Name>DAPI</Name>)",
               R"(<ShortName>DAPI</ShortName>)",
               R"(<Description />)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<ContrastMethods>ReflectedLightFluorescence</ContrastMethods>)",
               R"(<ContrastTypes>Unknown</ContrastTypes>)",
               R"(<Color>#FF00A0FF</Color>)",
               R"(<dye_id>McNamara-Boswell-0434</dye_id>)",
               R"(<collection_id>66071726-cbd4-4c41-b371-0a6eee4ae9c5</collection_id>)",
              R"(</FluorescenceDye>)",
             R"(</Channel>)",
            R"(</Channels>)",
           R"(</Track>)",
           R"(<Track IsActivated="true" IsSelected="false">)",
            R"(<BeforeHardwareSetting>Before [QD655]</BeforeHardwareSetting>)",
            R"(<AfterHardwareSetting>After [QD655]</AfterHardwareSetting>)",
            R"(<ZStackMode>Yes</ZStackMode>)",
            R"(<UseZStackCenter>true</UseZStackCenter>)",
            R"(<ZStackSliceIndex>1</ZStackSliceIndex>)",
            R"(<IsDisabledForApoTome>false</IsDisabledForApoTome>)",
            R"(<FocusOffset>0</FocusOffset>)",
            R"(<DirectDevice>MTBHXPLamp</DirectDevice>)",
            R"(<Channels>)",
             R"(<Channel IsActivated="true" IsSelected="false" ChannelSetupId="635134860654880767" InstanceId="635134860654880767" Name="QD655" Description="">)",
              R"(<Color>#FFFF0000</Color>)",
              R"(<PixelShiftX>0</PixelShiftX>)",
              R"(<PixelShiftY>0</PixelShiftY>)",
              R"(<DataGrabberSetup>)",
               R"(<CameraFrameSetup Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
                R"(<ExposureTime IsActivated="true">100</ExposureTime>)",
                R"(<Adjust IsActivated="true">0.5</Adjust>)",
                R"(<AutoExposure IsActivated="true">false</AutoExposure>)",
                R"(<EMGain>50</EMGain>)",
                R"(<ShadingReference IsActivated="true">false</ShadingReference>)",
               R"(</CameraFrameSetup>)",
              R"(</DataGrabberSetup>)",
              R"(<AdditionalDyeInformation>)",
               R"(<ShortName>QD655</ShortName>)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<DyeMaxEmission>654</DyeMaxEmission>)",
               R"(<DyeMaxExcitation>250</DyeMaxExcitation>)",
               R"(<DyeId>McNamara-Boswell-0985</DyeId>)",
               R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
              R"(</AdditionalDyeInformation>)",
              R"(<FluorescenceDye>)",
               R"(<Name>QD655</Name>)",
               R"(<ShortName>QD655</ShortName>)",
               R"(<Description />)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<ContrastMethods>ReflectedLightFluorescence</ContrastMethods>)",
               R"(<ContrastTypes>Unknown</ContrastTypes>)",
               R"(<Color>#FFFF0015</Color>)",
               R"(<dye_id>McNamara-Boswell-0985</dye_id>)",
               R"(<collection_id>66071726-cbd4-4c41-b371-0a6eee4ae9c5</collection_id>)",
              R"(</FluorescenceDye>)",
             R"(</Channel>)",
            R"(</Channels>)",
           R"(</Track>)",
           R"(<Track IsActivated="true" IsSelected="false">)",
            R"(<BeforeHardwareSetting>Before [QD605]</BeforeHardwareSetting>)",
            R"(<AfterHardwareSetting>After [QD605]</AfterHardwareSetting>)",
            R"(<ZStackMode>Yes</ZStackMode>)",
            R"(<UseZStackCenter>true</UseZStackCenter>)",
            R"(<ZStackSliceIndex>1</ZStackSliceIndex>)",
            R"(<IsDisabledForApoTome>false</IsDisabledForApoTome>)",
            R"(<FocusOffset>0</FocusOffset>)",
            R"(<DirectDevice>MTBHXPLamp</DirectDevice>)",
            R"(<Channels>)",
             R"(<Channel IsActivated="true" IsSelected="false" ChannelSetupId="635134860640499940" InstanceId="635134860640499940" Name="QD605" Description="">)",
              R"(<Color>#FF00FF00</Color>)",
              R"(<PixelShiftX>0</PixelShiftX>)",
              R"(<PixelShiftY>0</PixelShiftY>)",
              R"(<DataGrabberSetup>)",
               R"(<CameraFrameSetup Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
                R"(<ExposureTime IsActivated="true">20</ExposureTime>)",
                R"(<Adjust IsActivated="true">0.5</Adjust>)",
                R"(<AutoExposure IsActivated="true">false</AutoExposure>)",
                R"(<EMGain>50</EMGain>)",
                R"(<ShadingReference IsActivated="true">false</ShadingReference>)",
               R"(</CameraFrameSetup>)",
              R"(</DataGrabberSetup>)",
              R"(<AdditionalDyeInformation>)",
               R"(<ShortName>QD605</ShortName>)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<DyeMaxEmission>604</DyeMaxEmission>)",
               R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
               R"(<DyeId>McNamara-Boswell-0983</DyeId>)",
               R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
              R"(</AdditionalDyeInformation>)",
              R"(<FluorescenceDye>)",
               R"(<Name>QD605</Name>)",
               R"(<ShortName>QD605</ShortName>)",
               R"(<Description />)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<ContrastMethods>ReflectedLightFluorescence</ContrastMethods>)",
               R"(<ContrastTypes>Unknown</ContrastTypes>)",
               R"(<Color>#FFFF1200</Color>)",
               R"(<dye_id>McNamara-Boswell-0983</dye_id>)",
               R"(<collection_id>66071726-cbd4-4c41-b371-0a6eee4ae9c5</collection_id>)",
              R"(</FluorescenceDye>)",
             R"(</Channel>)",
            R"(</Channels>)",
           R"(</Track>)",
           R"(<Track IsActivated="true" IsSelected="false">)",
            R"(<BeforeHardwareSetting>Before [QD700]</BeforeHardwareSetting>)",
            R"(<AfterHardwareSetting>After [QD700]</AfterHardwareSetting>)",
            R"(<ZStackMode>Yes</ZStackMode>)",
            R"(<UseZStackCenter>true</UseZStackCenter>)",
            R"(<ZStackSliceIndex>1</ZStackSliceIndex>)",
            R"(<IsDisabledForApoTome>false</IsDisabledForApoTome>)",
            R"(<FocusOffset>0</FocusOffset>)",
            R"(<DirectDevice>MTBHXPLamp</DirectDevice>)",
            R"(<Channels>)",
             R"(<Channel IsActivated="true" IsSelected="false" ChannelSetupId="635145931372977308" InstanceId="635145931372977308" Name="QD705" Description="">)",
              R"(<Color>#FFFFFF00</Color>)",
              R"(<PixelShiftX>0</PixelShiftX>)",
              R"(<PixelShiftY>0</PixelShiftY>)",
              R"(<DataGrabberSetup>)",
               R"(<CameraFrameSetup Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
                R"(<ExposureTime IsActivated="true">20</ExposureTime>)",
                R"(<Adjust IsActivated="true">0.5</Adjust>)",
                R"(<AutoExposure IsActivated="true">false</AutoExposure>)",
                R"(<EMGain>50</EMGain>)",
                R"(<ShadingReference IsActivated="true">false</ShadingReference>)",
               R"(</CameraFrameSetup>)",
              R"(</DataGrabberSetup>)",
              R"(<AdditionalDyeInformation>)",
               R"(<ShortName>QD700</ShortName>)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<DyeMaxEmission>702</DyeMaxEmission>)",
               R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
               R"(<DyeId>McNamara-Boswell-1538</DyeId>)",
               R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
              R"(</AdditionalDyeInformation>)",
              R"(<FluorescenceDye>)",
               R"(<Name>QD705</Name>)",
               R"(<ShortName>QD700</ShortName>)",
               R"(<Description />)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<ContrastMethods>ReflectedLightFluorescence</ContrastMethods>)",
               R"(<ContrastTypes>Unknown</ContrastTypes>)",
               R"(<Color>#FFFF0008</Color>)",
               R"(<dye_id>McNamara-Boswell-1538</dye_id>)",
               R"(<collection_id>66071726-cbd4-4c41-b371-0a6eee4ae9c5</collection_id>)",
              R"(</FluorescenceDye>)",
             R"(</Channel>)",
            R"(</Channels>)",
           R"(</Track>)",
           R"(<Track IsActivated="true" IsSelected="true">)",
            R"(<BeforeHardwareSetting>Before [QD565]</BeforeHardwareSetting>)",
            R"(<AfterHardwareSetting>After [QD565]</AfterHardwareSetting>)",
            R"(<ZStackMode>Yes</ZStackMode>)",
            R"(<UseZStackCenter>true</UseZStackCenter>)",
            R"(<ZStackSliceIndex>1</ZStackSliceIndex>)",
            R"(<IsDisabledForApoTome>false</IsDisabledForApoTome>)",
            R"(<FocusOffset>0</FocusOffset>)",
            R"(<DirectDevice>MTBHXPLamp</DirectDevice>)",
            R"(<Channels>)",
             R"(<Channel IsActivated="true" IsSelected="true" ChannelSetupId="635145931246240056" InstanceId="635145931246240056" Name="QD565" Description="">)",
              R"(<Color>#FF00FFFF</Color>)",
              R"(<PixelShiftX>0</PixelShiftX>)",
              R"(<PixelShiftY>0</PixelShiftY>)",
              R"(<DataGrabberSetup>)",
               R"(<CameraFrameSetup Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
                R"(<ExposureTime IsActivated="true">25</ExposureTime>)",
                R"(<Adjust IsActivated="true">0.5</Adjust>)",
                R"(<AutoExposure IsActivated="true">false</AutoExposure>)",
                R"(<EMGain>50</EMGain>)",
                R"(<ShadingReference IsActivated="true">false</ShadingReference>)",
               R"(</CameraFrameSetup>)",
              R"(</DataGrabberSetup>)",
              R"(<AdditionalDyeInformation>)",
               R"(<ShortName>QD565</ShortName>)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<DyeMaxEmission>564</DyeMaxEmission>)",
               R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
               R"(<DyeId>McNamara-Boswell-0979</DyeId>)",
               R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
              R"(</AdditionalDyeInformation>)",
              R"(<FluorescenceDye>)",
               R"(<Name>QD565</Name>)",
               R"(<ShortName>QD565</ShortName>)",
               R"(<Description />)",
               R"(<IlluminationType>Fluorescence</IlluminationType>)",
               R"(<ContrastMethods>ReflectedLightFluorescence</ContrastMethods>)",
               R"(<ContrastTypes>Unknown</ContrastTypes>)",
               R"(<Color>#FFBFFF00</Color>)",
               R"(<dye_id>McNamara-Boswell-0979</dye_id>)",
               R"(<collection_id>66071726-cbd4-4c41-b371-0a6eee4ae9c5</collection_id>)",
              R"(</FluorescenceDye>)",
             R"(</Channel>)",
            R"(</Channels>)",
           R"(</Track>)",
           R"(<SubDimensionSetups />)",
          R"(</MultiTrackSetup>)",
         R"(</SubDimensionSetups>)",
        R"(</TilesSetup>)",
       R"(</SubDimensionSetups>)",
      R"(</RegionsSetup>)",
      R"(<ZStackSetup IsActivated="false">)",
       R"(<IsCenterMode>true</IsCenterMode>)",
       R"(<IsIntervalKept>true</IsIntervalKept>)",
       R"(<Offset>0</Offset>)",
       R"(<First>)",
        R"(<Distance>)",
         R"(<Value>0.0037844999999999997</Value>)",
         R"(<DefaultUnitFormat>um</DefaultUnitFormat>)",
        R"(</Distance>)",
       R"(</First>)",
       R"(<Last>)",
        R"(<Distance>)",
         R"(<Value>0.0037855</Value>)",
         R"(<DefaultUnitFormat>um</DefaultUnitFormat>)",
        R"(</Distance>)",
       R"(</Last>)",
       R"(<Interval>)",
        R"(<Distance>)",
         R"(<Value>5E-07</Value>)",
         R"(<DefaultUnitFormat>um</DefaultUnitFormat>)",
        R"(</Distance>)",
       R"(</Interval>)",
       R"(<SubDimensionSetups />)",
      R"(</ZStackSetup>)",
      R"(<FocusMapSetup IsActivated="false">)",
       R"(<IsRelativ>false</IsRelativ>)",
       R"(<StoreImageAtFocus>false</StoreImageAtFocus>)",
       R"(<StoreImageZstack>false</StoreImageZstack>)",
       R"(<SkipSharpnessMeasure>false</SkipSharpnessMeasure>)",
       R"(<LowerLimit>0</LowerLimit>)",
       R"(<UpperLimit>0</UpperLimit>)",
       R"(<StepSize>0</StepSize>)",
       R"(<ScanFrameRate>0</ScanFrameRate>)",
       R"(<PositioningForerun>0</PositioningForerun>)",
       R"(<TlFlashIntensity>0</TlFlashIntensity>)",
       R"(<SharpnessMeasureSet Name="" Aggregator="RootMeanSquare" />)",
       R"(<FocusPointStrategySet>)",
        R"(<PreferBorder>false</PreferBorder>)",
        R"(<MaxNumberOfPoints>24</MaxNumberOfPoints>)",
        R"(<FocusPointStrategy Strategy="" Param="" />)",
       R"(</FocusPointStrategySet>)",
       R"(<SubDimensionSetups />)",
      R"(</FocusMapSetup>)",
      R"(<LineScanSetup IsActivated="false">)",
       R"(<IsBrightfield>false</IsBrightfield>)",
       R"(<ScanFrameRate>0</ScanFrameRate>)",
       R"(<PositioningForerun>0</PositioningForerun>)",
       R"(<TlFlashIntensity>0</TlFlashIntensity>)",
       R"(<FmapMagicOffset>0</FmapMagicOffset>)",
       R"(<SubDimensionSetups />)",
      R"(</LineScanSetup>)",
      R"(<PanoramaSetup IsActivated="false">)",
       R"(<SubDimensionSetups />)",
      R"(</PanoramaSetup>)",
      R"(<TimeSeriesSetup IsActivated="false">)",
       R"(<IsInteractive>true</IsInteractive>)",
       R"(<IsLongAsPossible>false</IsLongAsPossible>)",
       R"(<Duration>)",
        R"(<Cycles>1</Cycles>)",
       R"(</Duration>)",
       R"(<IsFastAsPossible>false</IsFastAsPossible>)",
       R"(<Interval>)",
        R"(<TimeSpan>)",
         R"(<Value>0</Value>)",
         R"(<DefaultUnitFormat>ms</DefaultUnitFormat>)",
        R"(</TimeSpan>)",
       R"(</Interval>)",
       R"(<BeforeHardwareSetting />)",
       R"(<AfterHardwareSetting />)",
       R"(<StartMode>)",
        R"(<Manual />)",
       R"(</StartMode>)",
       R"(<StopMode>)",
        R"(<Manual />)",
       R"(</StopMode>)",
       R"(<PauseBeginMode>)",
        R"(<Manual />)",
       R"(</PauseBeginMode>)",
       R"(<PauseEndMode>)",
        R"(<Manual />)",
       R"(</PauseEndMode>)",
       R"(<SubDimensionSetups />)",
      R"(</TimeSeriesSetup>)",
      R"(<ApoTomePhaseSetup IsActivated="false">)",
       R"(<PhaseCount>0</PhaseCount>)",
       R"(<SubDimensionSetups />)",
      R"(</ApoTomePhaseSetup>)",
      R"(<CorrelativeSetup IsActivated="false">)",
       R"(<HolderDocument>)",
        R"(<Image />)",
        R"(<Delta1To2>)",
         R"(<X>0</X>)",
         R"(<Y>0</Y>)",
        R"(</Delta1To2>)",
        R"(<Delta2To3>)",
         R"(<X>0</X>)",
         R"(<Y>0</Y>)",
        R"(</Delta2To3>)",
        R"(<HolderName />)",
        R"(<IsReadOnly>false</IsReadOnly>)",
        R"(<Calibration>)",
         R"(<Marker1>)",
          R"(<X>NaN</X>)",
          R"(<Y>NaN</Y>)",
          R"(<Z>NaN</Z>)",
         R"(</Marker1>)",
         R"(<Marker2>)",
          R"(<X>NaN</X>)",
          R"(<Y>NaN</Y>)",
          R"(<Z>NaN</Z>)",
         R"(</Marker2>)",
         R"(<Marker3>)",
          R"(<X>NaN</X>)",
          R"(<Y>NaN</Y>)",
          R"(<Z>NaN</Z>)",
         R"(</Marker3>)",
        R"(</Calibration>)",
       R"(</HolderDocument>)",
       R"(<SubDimensionSetups />)",
      R"(</CorrelativeSetup>)",
     R"(</SubDimensionSetups>)",
     R"(<HelperSetups>)",
      R"(<AcquisitionModeSetup IsActivated="false">)",
       R"(<SelectedDetector>MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C</SelectedDetector>)",
       R"(<Detectors>)",
        R"(<Camera Id="MTBCamera_MTBCameraPortChanger_Left.AxioCamMR3">)",
         R"(<ApplyCameraProfile Status="Valid">false</ApplyCameraProfile>)",
         R"(<ApplyImageOrientation Status="Valid">true</ApplyImageOrientation>)",
         R"(<ExposureTime Status="Valid">118.4</ExposureTime>)",
         R"(<Frame Status="Valid">0,0,1388,1040</Frame>)",
         R"(<ImageOrientation Status="Valid">1</ImageOrientation>)",
         R"(<ContinuousAcquisitionSafeMode Status="Valid" />)",
         R"(<TTLOut2Mode Status="Valid">0</TTLOut2Mode>)",
         R"(<NIRMode Status="Valid">false</NIRMode>)",
         R"(<ShadingReference Status="Valid">false</ShadingReference>)",
         R"(<Resolution Status="Valid">0</Resolution>)",
         R"(<BinningList Status="Valid">0</BinningList>)",
         R"(<AnalogGainModeList Status="Valid">0</AnalogGainModeList>)",
         R"(<LiveSpeed Status="Valid">0</LiveSpeed>)",
         R"(<Trigger Status="Valid">false</Trigger>)",
         R"(<TriggerMode Status="Valid">1</TriggerMode>)",
         R"(<SnapShutter Status="Valid">true</SnapShutter>)",
         R"(<LiveShutter Status="Valid">false</LiveShutter>)",
         R"(<ShutterMode Status="Valid">0</ShutterMode>)",
         R"(<ShutterOpenDelay Status="Valid">0</ShutterOpenDelay>)",
         R"(<Adjust Status="Valid">0.9</Adjust>)",
         R"(<AdjustFrameRate Status="Valid">true</AdjustFrameRate>)",
         R"(<AutoExposure Status="Valid">false</AutoExposure>)",
         R"(<AutoExposureFrame Status="Valid">Empty</AutoExposureFrame>)",
         R"(<Binning Status="Valid">1,1</Binning>)",
         R"(<BlackReference Status="Valid">false</BlackReference>)",
         R"(<DualCameraCalibration Status="Valid">false</DualCameraCalibration>)",
         R"(<DualCameraCalibrationId Status="Valid">None</DualCameraCalibrationId>)",
         R"(<ExposureDelay Status="Valid">0</ExposureDelay>)",
         R"(<FrameRateTarget Status="Valid">30</FrameRateTarget>)",
         R"(<NoiseFilter Status="Valid">false</NoiseFilter>)",
         R"(<NoiseFilterThreshold Status="Valid">1.5</NoiseFilterThreshold>)",
         R"(<ReferenceImageCount Status="Valid">3</ReferenceImageCount>)",
         R"(<UnsharpMask Status="Valid">false</UnsharpMask>)",
         R"(<UnsharpMaskState Status="Valid" />)",
        R"(</Camera>)",
        R"(<Camera Id="MTBCamera_MTBCameraPortChanger_Left.HDCamC11440-22C" IsActivated="true">)",
         R"(<ApplyCameraProfile Status="Valid">false</ApplyCameraProfile>)",
         R"(<ApplyImageOrientation Status="Valid">true</ApplyImageOrientation>)",
         R"(<ExposureTime IsActivated="true" Status="Valid">15</ExposureTime>)",
         R"(<Frame Status="Valid">4,4,2040,2040</Frame>)",
         R"(<ImageOrientation Status="Valid">2</ImageOrientation>)",
         R"(<Adjust IsActivated="true" Status="Valid">0.5</Adjust>)",
         R"(<AdjustFrameRate Status="Valid">false</AdjustFrameRate>)",
         R"(<AutoExposure IsActivated="true" Status="Valid">false</AutoExposure>)",
         R"(<AutoExposureFrame Status="Valid">Empty</AutoExposureFrame>)",
         R"(<Binning Status="Valid">1,1</Binning>)",
         R"(<BinningList Status="Valid">0</BinningList>)",
         R"(<BlackReference Status="Valid">false</BlackReference>)",
         R"(<DualCameraCalibration Status="Valid">false</DualCameraCalibration>)",
         R"(<DualCameraCalibrationId Status="Valid">None</DualCameraCalibrationId>)",
         R"(<ExposureDelay Status="Valid">0</ExposureDelay>)",
         R"(<FrameRateTarget Status="Valid">30</FrameRateTarget>)",
         R"(<NoiseFilter Status="Valid">false</NoiseFilter>)",
         R"(<NoiseFilterThreshold Status="Valid">1.5</NoiseFilterThreshold>)",
         R"(<ReferenceImageCount Status="Valid">3</ReferenceImageCount>)",
         R"(<ShadingReference IsActivated="true" Status="Valid">false</ShadingReference>)",
         R"(<UnsharpMask Status="Valid">false</UnsharpMask>)",
         R"(<UnsharpMaskState Status="Valid" />)",
         R"(<DefectPixelCorrection Status="Valid">true</DefectPixelCorrection>)",
         R"(<TriggerMode Status="Valid">3</TriggerMode>)",
         R"(<Trigger Status="Valid">false</Trigger>)",
         R"(<EMGain Status="Valid">50</EMGain>)",
        R"(</Camera>)",
       R"(</Detectors>)",
       R"(<Devices />)",
      R"(</AcquisitionModeSetup>)",
      R"(<FocusSetup IsActivated="false">)",
       R"(<FocusStrategy IsActivated="false">)",
        R"(<StrategyMode>LocalFocusSurface</StrategyMode>)",
        R"(<ReferenceChannelId>635145930996145747</ReferenceChannelId>)",
        R"(<IsFocussingOnAllTracks>false</IsFocussingOnAllTracks>)",
        R"(<IsFocussingBeforeAcquisition>true</IsFocussingBeforeAcquisition>)",
        R"(<FixedZPosition>0</FixedZPosition>)",
        R"(<UseZPositionFromTilesSetup>false</UseZPositionFromTilesSetup>)",
        R"(<SupportPointsMode>FixedZPosition</SupportPointsMode>)",
        R"(<FocusZStackMode>FixedZ</FocusZStackMode>)",
        R"(<WaitPositionMode>CenterOfFirstRegion</WaitPositionMode>)",
        R"(<WaitPositionX>0</WaitPositionX>)",
        R"(<WaitPositionY>0</WaitPositionY>)",
        R"(<WaitPositionZ>0</WaitPositionZ>)",
        R"(<IsUpdatePositionedRegionsToClientEnabled>false</IsUpdatePositionedRegionsToClientEnabled>)",
        R"(<PositionedRegionsUpdateMode>CumulatedPositions</PositionedRegionsUpdateMode>)",
        R"(<BreakAfterSupportPointsDetermination>false</BreakAfterSupportPointsDetermination>)",
        R"(<TimeSeriesIntervalInfo>)",
         R"(<IsStartWithEnabled>false</IsStartWithEnabled>)",
         R"(<IsMainIntervalEnabled>true</IsMainIntervalEnabled>)",
         R"(<IsSubIntervalEnabled>false</IsSubIntervalEnabled>)",
         R"(<MainInterval>1</MainInterval>)",
         R"(<SubInterval>1</SubInterval>)",
        R"(</TimeSeriesIntervalInfo>)",
        R"(<RegionsIntervalInfo>)",
         R"(<IsStartWithEnabled>false</IsStartWithEnabled>)",
         R"(<IsMainIntervalEnabled>false</IsMainIntervalEnabled>)",
         R"(<IsSubIntervalEnabled>false</IsSubIntervalEnabled>)",
         R"(<MainInterval>1</MainInterval>)",
         R"(<SubInterval>1</SubInterval>)",
        R"(</RegionsIntervalInfo>)",
        R"(<TilesIntervalInfo>)",
         R"(<IsStartWithEnabled>true</IsStartWithEnabled>)",
         R"(<IsMainIntervalEnabled>true</IsMainIntervalEnabled>)",
         R"(<IsSubIntervalEnabled>false</IsSubIntervalEnabled>)",
         R"(<MainInterval>1</MainInterval>)",
         R"(<SubInterval>1</SubInterval>)",
        R"(</TilesIntervalInfo>)",
       R"(</FocusStrategy>)",
       R"(<SoftwareAutofocus>)",
        R"(<ActiveCamera>MTBCamera_MTBIntermediateTube_Left</ActiveCamera>)",
        R"(<AutofocusRegion>)",
         R"(<X>0</X>)",
         R"(<Y>0</Y>)",
         R"(<DX>1</DX>)",
         R"(<DY>1</DY>)",
        R"(</AutofocusRegion>)",
        R"(<BacklashStepsize>2</BacklashStepsize>)",
        R"(<DepthOfFocus>0.34</DepthOfFocus>)",
        R"(<DoBacklashCorrection>-1</DoBacklashCorrection>)",
        R"(<DoBleachingCorrection>true</DoBleachingCorrection>)",
        R"(<FactorGaussianFitStepsize>10</FactorGaussianFitStepsize>)",
        R"(<InitialDirection>1</InitialDirection>)",
        R"(<IsAutomaticRange>true</IsAutomaticRange>)",
        R"(<LowerLimit>100</LowerLimit>)",
        R"(<Precision>4</Precision>)",
        R"(<RequestedFocusPositionIsReachedPrecision>0.01</RequestedFocusPositionIsReachedPrecision>)",
        R"(<PredefinedStepsize>Optimal</PredefinedStepsize>)",
        R"(<SharpnessMeasureMethod>Auto</SharpnessMeasureMethod>)",
        R"(<RelativeSearchRange>13.600000000000001</RelativeSearchRange>)",
        R"(<SearchingMode>FarFromFocus</SearchingMode>)",
        R"(<SearchRangeType>SearchRangeAroundStartingPosition</SearchRangeType>)",
        R"(<SearchStrategy>Exhaustive</SearchStrategy>)",
        R"(<SharpnessMeasureSet Name="Best">)",
         R"(<SharpnessMeasure DecisionWeight="1" ScoreType="HaarGradient">)",
          R"(<Parameters>)",
           R"(<Parameter>1</Parameter>)",
          R"(</Parameters>)",
         R"(</SharpnessMeasure>)",
         R"(<SharpnessMeasure DecisionWeight="2" ScoreType="HaarGradient">)",
          R"(<Parameters>)",
           R"(<Parameter>2</Parameter>)",
          R"(</Parameters>)",
         R"(</SharpnessMeasure>)",
         R"(<SharpnessMeasure DecisionWeight="3" ScoreType="HaarGradient">)",
          R"(<Parameters>)",
           R"(<Parameter>3</Parameter>)",
          R"(</Parameters>)",
         R"(</SharpnessMeasure>)",
        R"(</SharpnessMeasureSet>)",
        R"(<UpperLimit>-100</UpperLimit>)",
        R"(<UseAcquisitionROI>true</UseAcquisitionROI>)",
        R"(<UseCameraParameters>false</UseCameraParameters>)",
        R"(<UseChannelInformation>true</UseChannelInformation>)",
       R"(</SoftwareAutofocus>)",
       R"(<DefiniteFocus IsActivated="false">)",
        R"(<IsPeriodicStabilization>false</IsPeriodicStabilization>)",
        R"(<Period>10</Period>)",
        R"(<IsPausedDuringAcquisition>true</IsPausedDuringAcquisition>)",
       R"(</DefiniteFocus>)",
      R"(</FocusSetup>)",
      R"(<BleachingSetup IsActivated="false">)",
       R"(<HasStartIndex>false</HasStartIndex>)",
       R"(<StartIndex>0</StartIndex>)",
       R"(<HasRepetition>false</HasRepetition>)",
       R"(<Repetition>1</Repetition>)",
       R"(<PulseDuration>0</PulseDuration>)",
       R"(<UsedAOTF>MTBAOTF2</UsedAOTF>)",
       R"(<AOTF />)",
      R"(</BleachingSetup>)",
      R"(<PhysiologyROISetup IsActivated="false" GraphicScale="1">)",
       R"(<ROI Name="DynamicRois">)",
        R"(<Usage>Annotation</Usage>)",
        R"(<IsProtected>false</IsProtected>)",
        R"(<LayerFlags>1</LayerFlags>)",
        R"(<Elements />)",
       R"(</ROI>)",
       R"(<Background Name="BackgroundRois">)",
        R"(<Usage>BackgroundRois</Usage>)",
        R"(<IsProtected>false</IsProtected>)",
        R"(<LayerFlags>1</LayerFlags>)",
        R"(<Elements />)",
       R"(</Background>)",
       R"(<Appliances>)",
        R"(<MeasurementSettings />)",
       R"(</Appliances>)",
      R"(</PhysiologyROISetup>)",
      R"(<RatioSetup IsActivated="false">)",
       R"(<IsRatioImageGenerated>false</IsRatioImageGenerated>)",
       R"(<SelectedColorMode>Palette</SelectedColorMode>)",
       R"(<Color>#00FFFFFF</Color>)",
       R"(<PaletteName>rainbow</PaletteName>)",
       R"(<RatioDisplayMinimum>0</RatioDisplayMinimum>)",
       R"(<RatioDisplayMaximum>10</RatioDisplayMaximum>)",
       R"(<RatioMethod>SingleWavelength</RatioMethod>)",
       R"(<SelectedReferenceIdChannel1>0</SelectedReferenceIdChannel1>)",
       R"(<SelectedReferenceIdChannel2>0</SelectedReferenceIdChannel2>)",
       R"(<RatioBackgroundCorrection>None</RatioBackgroundCorrection>)",
       R"(<BackgroundCorrectionChannel1>0</BackgroundCorrectionChannel1>)",
       R"(<BackgroundCorrectionChannel2>0</BackgroundCorrectionChannel2>)",
       R"(<SelectedRoiId>0</SelectedRoiId>)",
       R"(<AverageFrame>5</AverageFrame>)",
       R"(<Multiplicator>1</Multiplicator>)",
       R"(<IsOptionalThreshold>false</IsOptionalThreshold>)",
       R"(<OptionalThresholdChannel1>0</OptionalThresholdChannel1>)",
       R"(<OptionalThresholdChannel2>0</OptionalThresholdChannel2>)",
       R"(<IsRatioCalculationEnabled>true</IsRatioCalculationEnabled>)",
       R"(<IsRatioReferenceImageDefined>true</IsRatioReferenceImageDefined>)",
       R"(<AverageStartFrame>1</AverageStartFrame>)",
       R"(<AverageEndFrame>1</AverageEndFrame>)",
       R"(<IsBackgroundROIDefined>false</IsBackgroundROIDefined>)",
      R"(</RatioSetup>)",
     R"(</HelperSetups>)",
     R"(<ProcessingGraph FullTypeName="Zeiss.Micro.Acquisition.Processing.ProcessingGraph" Name="ProcessingFilterGraph" IsActive="true">)",
      R"(<Filters>)",
       R"(<Filter FullTypeName="Zeiss.Micro.Acquisition.Processing.CziFileSinkFilter" Name="FileSink" IsActive="true" IsUpdateEnabled="false" IsSplitScenes="false" IsCompressWithJpegXr="true" IsJpegXrLossless="false" JpegXrQualityLevel="85" Instrumentation="0" />)",
       R"(<Filter FullTypeName="Zeiss.Micro.Acquisition.Processing.MultiTilePyramidFilter" Name="Pyramid" IsActive="true" BackgroundFillType="FillBlack" ExcessiveLogging="false" PurgeAfterNRows="20" Type="Auto">)",
        R"(<Connections>)",
         R"(<Connection To="FileSink" />)",
        R"(</Connections>)",
        R"(<SwapFolder />)",
        R"(<TilingMode>1</TilingMode>)",
        R"(<CalcDiffMeasure>false</CalcDiffMeasure>)",
        R"(<PendingTasksFactor>NaN</PendingTasksFactor>)",
       R"(</Filter>)",
       R"(<Filter FullTypeName="Zeiss.Micro.Acquisition.Processing.CompositeDispatchFilterStitching2" Name="Stitching" IsActive="true" Type="Auto">)",
        R"(<ReferencePlane>)",
         R"(<dim_0 ImageDimension="C" Position="0" StitchingMethodHint="MostlyBlack" />)",
        R"(</ReferencePlane>)",
        R"(<CameraRotationInDegree>0</CameraRotationInDegree>)",
        R"(<CameraRotationErrorInDegree>1</CameraRotationErrorInDegree>)",
        R"(<JitterErrorXLeftRight>0.15</JitterErrorXLeftRight>)",
        R"(<JitterErrorYLeftRight>0.1</JitterErrorYLeftRight>)",
        R"(<JitterErrorXTopBottom>0.15</JitterErrorXTopBottom>)",
        R"(<JitterErrorYTopBottom>0.1</JitterErrorYTopBottom>)",
        R"(<JitterErrorYTopBottomFirstRow>0.28</JitterErrorYTopBottomFirstRow>)",
        R"(<PurgeTilesAfterNRows>20</PurgeTilesAfterNRows>)",
        R"(<LastCallAfterNRows>2</LastCallAfterNRows>)",
        R"(<BurnInStitchingInfo>false</BurnInStitchingInfo>)",
        R"(<CorrelationGateConstant1>NaN</CorrelationGateConstant1>)",
        R"(<CorrelationGateConstant2>5E-05</CorrelationGateConstant2>)",
        R"(<CorrelationGateConstant3>0.8</CorrelationGateConstant3>)",
        R"(<CorrelationGateConstant4>1</CorrelationGateConstant4>)",
        R"(<CorrelationGateConstant5>NaN</CorrelationGateConstant5>)",
        R"(<CorrelationGateConstant6>NaN</CorrelationGateConstant6>)",
        R"(<DisplacementGateConstantX>0.2</DisplacementGateConstantX>)",
        R"(<DisplacementGateConstantY>0.2</DisplacementGateConstantY>)",
        R"(<FL_CorrelationGateConstant1>NaN</FL_CorrelationGateConstant1>)",
        R"(<FL_CorrelationGateConstant2>1.5E-06</FL_CorrelationGateConstant2>)",
        R"(<FL_CorrelationGateConstant3>0.8</FL_CorrelationGateConstant3>)",
        R"(<FL_CorrelationGateConstant4>1</FL_CorrelationGateConstant4>)",
        R"(<FL_CorrelationGateConstant5>NaN</FL_CorrelationGateConstant5>)",
        R"(<FL_CorrelationGateConstant6>NaN</FL_CorrelationGateConstant6>)",
        R"(<FL_DisplacementGateConstantX>0.2</FL_DisplacementGateConstantX>)",
        R"(<FL_DisplacementGateConstantY>0.2</FL_DisplacementGateConstantY>)",
        R"(<StitchingStrategy>5</StitchingStrategy>)",
        R"(<CameraBias>0</CameraBias>)",
        R"(<UseStitchingHint>true</UseStitchingHint>)",
        R"(<MaxMemory>NaN</MaxMemory>)",
        R"(<PendingTasksFactor>NaN</PendingTasksFactor>)",
        R"(<Connections>)",
         R"(<Connection To="Pyramid" />)",
        R"(</Connections>)",
       R"(</Filter>)",
       R"(<Filter FullTypeName="Zeiss.Micro.Acquisition.Processing.RawDecodeImageFilter" Name="DecodeImage" IsActive="true">)",
        R"(<Connections>)",
         R"(<Connection To="Stitching" />)",
        R"(</Connections>)",
       R"(</Filter>)",
      R"(</Filters>)",
      R"(<StopPriorities>)",
       R"(<StopPriority Name="FileSink" Value="1" />)",
       R"(<StopPriority Name="Pyramid" Value="2" />)",
       R"(<StopPriority Name="Stitching" Value="3" />)",
       R"(<StopPriority Name="DecodeImage" Value="8" />)",
      R"(</StopPriorities>)",
     R"(</ProcessingGraph>)",
    R"(</AcquisitionBlock>)",
   R"(</ExperimentBlocks>)",
   R"(<HardwareSettingsPool>)",
    R"(<HardwareSetting Name="HardwareBefore">)",
     R"(<TriggerCard Id="MTBTrigger" IsActivated="true">)",
      R"(<Reset IsActivated="true">true</Reset>)",
      R"(<Outp1 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">15</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp1>)",
      R"(<Outp2 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">9</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp2>)",
      R"(<Outp3 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">10</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp3>)",
      R"(<Outp4 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">11</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp4>)",
      R"(<Outp5 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">13</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp5>)",
      R"(<Outp6 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">14</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp6>)",
      R"(<Outp7 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">29</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp7>)",
      R"(<Outp8 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">30</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp8>)",
      R"(<Outp9 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">2</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp9>)",
      R"(<Outp10 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">3</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp10>)",
      R"(<Outp11 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">4</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp11>)",
      R"(<Outp13 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">5</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp13>)",
      R"(<Outp14 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">6</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp14>)",
      R"(<Outp15 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">7</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">4</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp15>)",
      R"(<Outp21 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">1</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp21>)",
      R"(<Outp29 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">8</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp29>)",
     R"(</TriggerCard>)",
     R"(<TransmittedLightShutter Id="MTBTLShutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true" Status="Valid">true</IsClosed>)",
     R"(</TransmittedLightShutter>)",
     R"(<TransmittedLightHalogenLamp Id="MTBTLHalogenLamp" IsActivated="true">)",
      R"(<Intensity IsActivated="true" Status="Valid">0.0</Intensity>)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
     R"(</TransmittedLightHalogenLamp>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before Exp [FITC, TexRe] Smart">)",
     R"(<TransmittedLightShutter Id="MTBTLShutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true" Status="Valid">true</IsClosed>)",
     R"(</TransmittedLightShutter>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true">false</IsEnabled>)",
      R"(<Intensity IsActivated="true">0</Intensity>)",
     R"(</LED4>)",
     R"(<ObjectiveChanger Id="MTBObjectiveChanger" IsActivated="true">)",
      R"(<Position IsActivated="true" Status="Valid">4</Position>)",
      R"(<PositionName IsActivated="true" Status="Valid">Objective.420660-9970-000</PositionName>)",
     R"(</ObjectiveChanger>)",
     R"(<TransmittedLightHalogenLamp Id="MTBTLHalogenLamp" IsActivated="true">)",
      R"(<Intensity IsActivated="true" Status="Valid">0.0</Intensity>)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
     R"(</TransmittedLightHalogenLamp>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<TriggerCard Id="MTBTrigger" IsActivated="true">)",
      R"(<Reset IsActivated="true">true</Reset>)",
      R"(<Outp1 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">15</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp1>)",
      R"(<Outp2 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">9</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp2>)",
      R"(<Outp3 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">10</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp3>)",
      R"(<Outp4 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">11</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp4>)",
      R"(<Outp5 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">13</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp5>)",
      R"(<Outp6 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">14</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp6>)",
      R"(<Outp7 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">29</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp7>)",
      R"(<Outp8 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">30</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp8>)",
      R"(<Outp9 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">2</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp9>)",
      R"(<Outp10 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">3</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp10>)",
      R"(<Outp11 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">4</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp11>)",
      R"(<Outp13 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">5</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp13>)",
      R"(<Outp14 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">6</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp14>)",
      R"(<Outp15 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">7</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">4</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp15>)",
      R"(<Outp21 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">1</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp21>)",
      R"(<Outp29 IsActivated="true">)",
       R"(<Output>)",
        R"(<InputSignalMode IsActivated="true">DirectInput</InputSignalMode>)",
        R"(<DirectInput IsActivated="true">8</DirectInput>)",
        R"(<AndMask IsActivated="true">0</AndMask>)",
        R"(<OrMask IsActivated="true">0</OrMask>)",
        R"(<InvertSignal IsActivated="true">False</InvertSignal>)",
       R"(</Output>)",
      R"(</Outp29>)",
     R"(</TriggerCard>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After Exp [FITC, TexRe] Smart">)",
     R"(<TransmittedLightShutter Id="MTBTLShutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</TransmittedLightShutter>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true">false</IsEnabled>)",
      R"(<Intensity IsActivated="true">0</Intensity>)",
     R"(</LED4>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before [QD605]">)",
     R"(<ReflectorChanger Id="MTBReflectorChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">4</Position>)",
      R"(<PositionName IsActivated="true">User.Chroma 605</PositionName>)",
     R"(</ReflectorChanger>)",
     R"(<Colibri Id="MTBFLLEDController" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">External</LampMode>)",
     R"(</Colibri>)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">false</IsClosed>)",
     R"(</HXP120Shutter>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<HXP120V Id="MTBHXPLamp" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
      R"(<Intensity IsActivated="true" Status="Valid">75</Intensity>)",
     R"(</HXP120V>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true" Status="Valid">false</IsEnabled>)",
      R"(<Intensity IsActivated="true" Status="Valid">0</Intensity>)",
     R"(</LED4>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After [QD605]">)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</HXP120Shutter>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before [QD655]">)",
     R"(<ReflectorChanger Id="MTBReflectorChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">3</Position>)",
      R"(<PositionName IsActivated="true">User.Chroma 655</PositionName>)",
     R"(</ReflectorChanger>)",
     R"(<Colibri Id="MTBFLLEDController" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">External</LampMode>)",
     R"(</Colibri>)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true" Status="Valid">false</IsClosed>)",
     R"(</HXP120Shutter>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true" Status="Valid">false</IsEnabled>)",
      R"(<Intensity IsActivated="true" Status="Valid">100</Intensity>)",
     R"(</LED4>)",
     R"(<HXP120V Id="MTBHXPLamp" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
      R"(<Intensity IsActivated="true" Status="Valid">75</Intensity>)",
     R"(</HXP120V>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After [QD655]">)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</HXP120Shutter>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before [DAPI]_2">)",
     R"(<ReflectorChanger Id="MTBReflectorChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Reflector.488049-9901-000</PositionName>)",
     R"(</ReflectorChanger>)",
     R"(<Colibri Id="MTBFLLEDController" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">External</LampMode>)",
     R"(</Colibri>)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true" Status="Valid">false</IsClosed>)",
     R"(</HXP120Shutter>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true" Status="Valid">false</IsEnabled>)",
      R"(<Intensity IsActivated="true" Status="Valid">100</Intensity>)",
     R"(</LED4>)",
     R"(<HXP120V Id="MTBHXPLamp" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
      R"(<Intensity IsActivated="true" Status="Valid">75</Intensity>)",
     R"(</HXP120V>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After [DAPI]_2">)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</HXP120Shutter>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before [QD565]">)",
     R"(<ReflectorChanger Id="MTBReflectorChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">6</Position>)",
      R"(<PositionName IsActivated="true">User.Chroma 565</PositionName>)",
     R"(</ReflectorChanger>)",
     R"(<Colibri Id="MTBFLLEDController" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">External</LampMode>)",
     R"(</Colibri>)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true" Status="Valid">false</IsClosed>)",
     R"(</HXP120Shutter>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<HXP120V Id="MTBHXPLamp" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
      R"(<Intensity IsActivated="true" Status="Valid">75</Intensity>)",
     R"(</HXP120V>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true" Status="Valid">false</IsEnabled>)",
      R"(<Intensity IsActivated="true" Status="Valid">45.474158756189</Intensity>)",
     R"(</LED4>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After [QD565]">)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</HXP120Shutter>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="Before [QD700]">)",
     R"(<ReflectorChanger Id="MTBReflectorChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">5</Position>)",
      R"(<PositionName IsActivated="true">User.Chroma 705</PositionName>)",
     R"(</ReflectorChanger>)",
     R"(<Colibri Id="MTBFLLEDController" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">External</LampMode>)",
     R"(</Colibri>)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">false</IsClosed>)",
     R"(</HXP120Shutter>)",
     R"(<CameraPortChanger Id="MTBCameraPortChanger" IsActivated="true">)",
      R"(<Position IsActivated="true">2</Position>)",
      R"(<PositionName IsActivated="true">Pollux.CameraChangerElement_Left</PositionName>)",
     R"(</CameraPortChanger>)",
     R"(<HXP120V Id="MTBHXPLamp" IsActivated="true">)",
      R"(<LampMode IsActivated="true" Status="Valid">Manual</LampMode>)",
      R"(<Intensity IsActivated="true" Status="Valid">75</Intensity>)",
     R"(</HXP120V>)",
     R"(<LED4 Id="MTBLED4" IsActivated="true">)",
      R"(<IsEnabled IsActivated="true" Status="Valid">false</IsEnabled>)",
      R"(<Intensity IsActivated="true" Status="Valid">0</Intensity>)",
     R"(</LED4>)",
    R"(</HardwareSetting>)",
    R"(<HardwareSetting Name="After [QD700]">)",
     R"(<HXP120Shutter Id="MTBHXP120Shutter" IsActivated="true">)",
      R"(<IsClosed IsActivated="true">true</IsClosed>)",
     R"(</HXP120Shutter>)",
    R"(</HardwareSetting>)",
   R"(</HardwareSettingsPool>)",
   R"(<ExperimentFeedback IsActivated="false">)",
    R"(<PreScript />)",
    R"(<LoopScript />)",
    R"(<PostScript />)",
    R"(<Actions />)",
   R"(</ExperimentFeedback>)",
  R"(</Experiment>)",
  R"(<CustomAttributes />)",
  R"(<Information>)",
   R"(<User Id="0" />)",
   R"(<Application>)",
    R"(<Name>ZEN 2012 (blue edition)</Name>)",
    R"(<Version>1.1.1.0</Version>)",
   R"(</Application>)",
   R"(<Document>)",
    R"(<UserName>zeiss</UserName>)",
    R"(<CreationDate>2013-09-12T11:18:12.2152881-04:00</CreationDate>)",
   R"(</Document>)",
   R"(<Image>)",
    R"(<AcquisitionDateAndTime>2013-09-12T15:24:02.4033177Z</AcquisitionDateAndTime>)",
    R"(<ComponentBitCount>16</ComponentBitCount>)",
    R"(<PixelType>Gray16</PixelType>)",
    R"(<SizeC>5</SizeC>)",
    R"(<SizeS>23</SizeS>)",
    R"(<SizeX>139758</SizeX>)",
    R"(<SizeY>103021</SizeY>)",
    R"(<SizeT>1</SizeT>)",
    R"(<SizeB>1</SizeB>)",
    R"(<SizeM>141</SizeM>)",
    R"(<AcquisitionDuration>7761956.9586000005</AcquisitionDuration>)",
    R"(<OriginalEncodingQuality>100</OriginalEncodingQuality>)",
    R"(<Dimensions>)",
     R"(<Channels>)",
      R"(<Channel Id="Channel:1">)",
       R"(<ExcitationWavelength>353</ExcitationWavelength>)",
       R"(<EmissionWavelength>465</EmissionWavelength>)",
       R"(<DyeId>McNamara-Boswell-0434</DyeId>)",
       R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
       R"(<Color>#FF0000FF</Color>)",
       R"(<Fluor>DAPI</Fluor>)",
       R"(<AcquisitionMode>WideField</AcquisitionMode>)",
       R"(<ExposureTime>15000000</ExposureTime>)",
       R"(<Reflector>49 DAPI</Reflector>)",
       R"(<IlluminationType>Epifluorescence</IlluminationType>)",
       R"(<ContrastMethod>Fluorescence</ContrastMethod>)",
       R"(<PixelType>Gray16</PixelType>)",
       R"(<ComponentBitCount>16</ComponentBitCount>)",
       R"(<DetectorSettings>)",
        R"(<Binning>1,1</Binning>)",
        R"(<Detector Id="Detector:Orca Flash 4.0" />)",
       R"(</DetectorSettings>)",
       R"(<LightSourcesSettings>)",
        R"(<LightSourceSettings>)",
         R"(<Intensity>73.95 %</Intensity>)",
         R"(<LightSource Id="LightSource:1" />)",
        R"(</LightSourceSettings>)",
       R"(</LightSourcesSettings>)",
       R"(<FilterSetRef Id="FilterSet:1" />)",
      R"(</Channel>)",
      R"(<Channel Id="Channel:2">)",
       R"(<ExcitationWavelength>250</ExcitationWavelength>)",
       R"(<EmissionWavelength>654</EmissionWavelength>)",
       R"(<DyeId>McNamara-Boswell-0985</DyeId>)",
       R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
       R"(<Color>#FFFF0000</Color>)",
       R"(<Fluor>QD655</Fluor>)",
       R"(<AcquisitionMode>WideField</AcquisitionMode>)",
       R"(<ExposureTime>100000000</ExposureTime>)",
       R"(<Reflector>Chroma 655</Reflector>)",
       R"(<IlluminationType>Epifluorescence</IlluminationType>)",
       R"(<ContrastMethod>Fluorescence</ContrastMethod>)",
       R"(<PixelType>Gray16</PixelType>)",
       R"(<ComponentBitCount>16</ComponentBitCount>)",
       R"(<DetectorSettings>)",
        R"(<Binning>1,1</Binning>)",
        R"(<Detector Id="Detector:Orca Flash 4.0" />)",
       R"(</DetectorSettings>)",
       R"(<LightSourcesSettings>)",
        R"(<LightSourceSettings>)",
         R"(<Intensity>73.95 %</Intensity>)",
         R"(<LightSource Id="LightSource:2" />)",
        R"(</LightSourceSettings>)",
       R"(</LightSourcesSettings>)",
       R"(<FilterSetRef Id="FilterSet:2" />)",
      R"(</Channel>)",
      R"(<Channel Id="Channel:3">)",
       R"(<ExcitationWavelength>300</ExcitationWavelength>)",
       R"(<EmissionWavelength>604</EmissionWavelength>)",
       R"(<DyeId>McNamara-Boswell-0983</DyeId>)",
       R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
       R"(<Color>#FF00FF00</Color>)",
       R"(<Fluor>QD605</Fluor>)",
       R"(<AcquisitionMode>WideField</AcquisitionMode>)",
       R"(<ExposureTime>20000000</ExposureTime>)",
       R"(<Reflector>Chroma 605</Reflector>)",
       R"(<IlluminationType>Epifluorescence</IlluminationType>)",
       R"(<ContrastMethod>Fluorescence</ContrastMethod>)",
       R"(<PixelType>Gray16</PixelType>)",
       R"(<ComponentBitCount>16</ComponentBitCount>)",
       R"(<DetectorSettings>)",
        R"(<Binning>1,1</Binning>)",
        R"(<Detector Id="Detector:Orca Flash 4.0" />)",
       R"(</DetectorSettings>)",
       R"(<LightSourcesSettings>)",
        R"(<LightSourceSettings>)",
         R"(<Intensity>73.95 %</Intensity>)",
         R"(<LightSource Id="LightSource:3" />)",
        R"(</LightSourceSettings>)",
       R"(</LightSourcesSettings>)",
       R"(<FilterSetRef Id="FilterSet:3" />)",
      R"(</Channel>)",
      R"(<Channel Id="Channel:4">)",
       R"(<ExcitationWavelength>300</ExcitationWavelength>)",
       R"(<EmissionWavelength>702</EmissionWavelength>)",
       R"(<DyeId>McNamara-Boswell-1538</DyeId>)",
       R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
       R"(<Color>#FFFFFF00</Color>)",
       R"(<Fluor>QD705</Fluor>)",
       R"(<AcquisitionMode>WideField</AcquisitionMode>)",
       R"(<ExposureTime>20000000</ExposureTime>)",
       R"(<Reflector>Chroma 705</Reflector>)",
       R"(<IlluminationType>Epifluorescence</IlluminationType>)",
       R"(<ContrastMethod>Fluorescence</ContrastMethod>)",
       R"(<PixelType>Gray16</PixelType>)",
       R"(<ComponentBitCount>16</ComponentBitCount>)",
       R"(<DetectorSettings>)",
        R"(<Binning>1,1</Binning>)",
        R"(<Detector Id="Detector:Orca Flash 4.0" />)",
       R"(</DetectorSettings>)",
       R"(<LightSourcesSettings>)",
        R"(<LightSourceSettings>)",
         R"(<Intensity>73.95 %</Intensity>)",
         R"(<LightSource Id="LightSource:4" />)",
        R"(</LightSourceSettings>)",
       R"(</LightSourcesSettings>)",
       R"(<FilterSetRef Id="FilterSet:4" />)",
      R"(</Channel>)",
      R"(<Channel Id="Channel:5">)",
       R"(<ExcitationWavelength>300</ExcitationWavelength>)",
       R"(<EmissionWavelength>564</EmissionWavelength>)",
       R"(<DyeId>McNamara-Boswell-0979</DyeId>)",
       R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
       R"(<Color>#FF00FFFF</Color>)",
       R"(<Fluor>QD565</Fluor>)",
       R"(<AcquisitionMode>WideField</AcquisitionMode>)",
       R"(<ExposureTime>25000000</ExposureTime>)",
       R"(<Reflector>Chroma 565</Reflector>)",
       R"(<IlluminationType>Epifluorescence</IlluminationType>)",
       R"(<ContrastMethod>Fluorescence</ContrastMethod>)",
       R"(<PixelType>Gray16</PixelType>)",
       R"(<ComponentBitCount>16</ComponentBitCount>)",
       R"(<DetectorSettings>)",
        R"(<Binning>1,1</Binning>)",
        R"(<Detector Id="Detector:Orca Flash 4.0" />)",
       R"(</DetectorSettings>)",
       R"(<LightSourcesSettings>)",
        R"(<LightSourceSettings>)",
         R"(<Intensity>73.95 %</Intensity>)",
         R"(<LightSource Id="LightSource:5" />)",
        R"(</LightSourceSettings>)",
       R"(</LightSourcesSettings>)",
       R"(<FilterSetRef Id="FilterSet:5" />)",
      R"(</Channel>)",
     R"(</Channels>)",
     R"(<Tracks>)",
      R"(<Track Id="Track:1">)",
       R"(<ChannelRefs>)",
        R"(<ChannelRef Id="Channel:1" />)",
       R"(</ChannelRefs>)",
      R"(</Track>)",
      R"(<Track Id="Track:2">)",
       R"(<ChannelRefs>)",
        R"(<ChannelRef Id="Channel:2" />)",
       R"(</ChannelRefs>)",
      R"(</Track>)",
      R"(<Track Id="Track:3">)",
       R"(<ChannelRefs>)",
        R"(<ChannelRef Id="Channel:3" />)",
       R"(</ChannelRefs>)",
      R"(</Track>)",
      R"(<Track Id="Track:4">)",
       R"(<ChannelRefs>)",
        R"(<ChannelRef Id="Channel:4" />)",
       R"(</ChannelRefs>)",
      R"(</Track>)",
      R"(<Track Id="Track:5">)",
       R"(<ChannelRefs>)",
        R"(<ChannelRef Id="Channel:5" />)",
       R"(</ChannelRefs>)",
      R"(</Track>)",
     R"(</Tracks>)",
     R"(<T>)",
      R"(<StartTime>2013-09-12T15:24:02.4033177Z</StartTime>)",
      R"(<Positions>)",
       R"(<BinaryList>)",
        R"(<AttachmentName>TimeStamps</AttachmentName>)",
       R"(</BinaryList>)",
      R"(</Positions>)",
     R"(</T>)",
     R"(<S>)",
      R"(<Scenes>)",
       R"(<Scene Index="0" Name="ScanRegion0">)",
        R"(<RegionId>635145958921324051</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-26169.273,7260.055</CenterPosition>)",
        R"(<ContourSize>3277.715,2952.579</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="1" Name="ScanRegion2">)",
        R"(<RegionId>635145958921344054</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-30134.686,7312.493</CenterPosition>)",
        R"(<ContourSize>3384.121,3620.442</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="2" Name="ScanRegion1">)",
        R"(<RegionId>635145958921334052</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-22591.615,7488.716</CenterPosition>)",
        R"(<ContourSize>2607.887,2636.221</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="3" Name="ScanRegion3">)",
        R"(<RegionId>635145958921364056</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-18308.982,7770.189</CenterPosition>)",
        R"(<ContourSize>3771.25,3269.015</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="4" Name="ScanRegion4">)",
        R"(<RegionId>635145958921374058</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-14202.164,8086.815</CenterPosition>)",
        R"(<ContourSize>3312.876,2917.445</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="5" Name="ScanRegion5">)",
        R"(<RegionId>635145958921384059</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-26360.424,11319.775</CenterPosition>)",
        R"(<ContourSize>3454.559,3058.117</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="6" Name="ScanRegion6">)",
        R"(<RegionId>635145958921394061</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-30377.912,11583.16</CenterPosition>)",
        R"(<ContourSize>3101.562,2952.613</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="7" Name="ScanRegion8">)",
        R"(<RegionId>635145958921424064</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-18922.535,11847.518</CenterPosition>)",
        R"(<ContourSize>3806.105,3269.024</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="8" Name="ScanRegion7">)",
        R"(<RegionId>635145958921414063</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-22711.38,12023.018</CenterPosition>)",
        R"(<ContourSize>2432.381,2284.769</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="9" Name="ScanRegion10">)",
        R"(<RegionId>635145958921454068</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-14534.246,12128.975</CenterPosition>)",
        R"(<ContourSize>3841.352,3339.318</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="10" Name="ScanRegion11">)",
        R"(<RegionId>635145958921464070</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-10674.238,12217.105</CenterPosition>)",
        R"(<ContourSize>3031.572,3304.107</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="11" Name="ScanRegion12">)",
        R"(<RegionId>635145958921474071</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-26497.227,15537.753</CenterPosition>)",
        R"(<ContourSize>3524.16,3761.021</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="12" Name="ScanRegion9">)",
        R"(<RegionId>635145958921434066</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-30357.513,15590.203</CenterPosition>)",
        R"(<ContourSize>3207.553,3304.044</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="13" Name="ScanRegion14">)",
        R"(<RegionId>635145958921504075</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-22496.847,15942.235</CenterPosition>)",
        R"(<ContourSize>3488.857,3725.847</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="14" Name="ScanRegion16">)",
        R"(<RegionId>635145958921544079</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-18830.932,16083.037</CenterPosition>)",
        R"(<ContourSize>3489.332,3936.778</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="15" Name="ScanRegion15">)",
        R"(<RegionId>635145958921524077</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-14760.085,16118.461</CenterPosition>)",
        R"(<ContourSize>3735.501,3304.051</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="16" Name="ScanRegion20">)",
        R"(<RegionId>635145958921604087</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-11023.706,16698.647</CenterPosition>)",
        R"(<ContourSize>2891.617,3971.99</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="17" Name="ScanRegion13">)",
        R"(<RegionId>635145958921494073</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-30265.826,19456.726</CenterPosition>)",
        R"(<ContourSize>2891.227,3233.799</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="18" Name="ScanRegion18">)",
        R"(<RegionId>635145958921574083</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-26494.869,19667.804</CenterPosition>)",
        R"(<ContourSize>3666.234,3655.627</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="19" Name="ScanRegion17">)",
        R"(<RegionId>635145958921564081</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-22582.047,19826.249</CenterPosition>)",
        R"(<ContourSize>3172.281,3128.385</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="20" Name="ScanRegion19">)",
        R"(<RegionId>635145958921594085</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-19056.562,19984.623</CenterPosition>)",
        R"(<ContourSize>3031.098,3163.496</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="21" Name="ScanRegion21">)",
        R"(<RegionId>635145958921624089</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-15214.917,20125.466</CenterPosition>)",
        R"(<ContourSize>3454.533,3304.111</ContourSize>)",
       R"(</Scene>)",
       R"(<Scene Index="22" Name="ScanRegion22">)",
        R"(<RegionId>635145958921634090</RegionId>)",
        R"(<ScanMode>Meander</ScanMode>)",
        R"(<CenterPosition>-11179.261,20547.534</CenterPosition>)",
        R"(<ContourSize>2995.824,3022.861</ContourSize>)",
       R"(</Scene>)",
      R"(</Scenes>)",
     R"(</S>)",
    R"(</Dimensions>)",
    R"(<ObjectiveSettings>)",
     R"(<ObjectiveRef Id="Objective:1" />)",
    R"(</ObjectiveSettings>)",
    R"(<MicroscopeRef Id="Microscope:1" />)",
    R"(<TubeLenses />)",
   R"(</Image>)",
   R"(<Instrument>)",
    R"(<Microscopes>)",
     R"(<Microscope Id="Microscope:1" Name="Axio Scan.Z1">)",
      R"(<Type>Upright</Type>)",
     R"(</Microscope>)",
    R"(</Microscopes>)",
    R"(<LightSources>)",
     R"(<LightSource Id="LightSource:1" Name="HXP 120 V">)",
      R"(<LightSourceType>)",
       R"(<Arc>)",
        R"(<Type>Other</Type>)",
       R"(</Arc>)",
      R"(</LightSourceType>)",
     R"(</LightSource>)",
     R"(<LightSource Id="LightSource:2" Name="HXP 120 V">)",
      R"(<LightSourceType>)",
       R"(<Arc>)",
        R"(<Type>Other</Type>)",
       R"(</Arc>)",
      R"(</LightSourceType>)",
     R"(</LightSource>)",
     R"(<LightSource Id="LightSource:3" Name="HXP 120 V">)",
      R"(<LightSourceType>)",
       R"(<Arc>)",
        R"(<Type>Other</Type>)",
       R"(</Arc>)",
      R"(</LightSourceType>)",
     R"(</LightSource>)",
     R"(<LightSource Id="LightSource:4" Name="HXP 120 V">)",
      R"(<LightSourceType>)",
       R"(<Arc>)",
        R"(<Type>Other</Type>)",
       R"(</Arc>)",
      R"(</LightSourceType>)",
     R"(</LightSource>)",
     R"(<LightSource Id="LightSource:5" Name="HXP 120 V">)",
      R"(<LightSourceType>)",
       R"(<Arc>)",
        R"(<Type>Other</Type>)",
       R"(</Arc>)",
      R"(</LightSourceType>)",
     R"(</LightSource>)",
    R"(</LightSources>)",
    R"(<Detectors>)",
     R"(<Detector Id="Detector:Orca Flash 4.0" Name="Orca Flash 4.0">)",
      R"(<Manufacturer>)",
       R"(<Model>HDCamC11440-22C</Model>)",
      R"(</Manufacturer>)",
      R"(<Adapter>)",
       R"(<Manufacturer>)",
        R"(<Model>1x Camera Adapter</Model>)",
       R"(</Manufacturer>)",
      R"(</Adapter>)",
     R"(</Detector>)",
    R"(</Detectors>)",
    R"(<Objectives>)",
     R"(<Objective Id="Objective:1" Name="Plan-Apochromat 40x/0.95 Korr M27">)",
      R"(<LensNA>0.95</LensNA>)",
      R"(<NominalMagnification>40</NominalMagnification>)",
      R"(<WorkingDistance>160</WorkingDistance>)",
      R"(<PupilGeometry>Circular</PupilGeometry>)",
      R"(<Immersion>Air</Immersion>)",
     R"(</Objective>)",
    R"(</Objectives>)",
    R"(<FilterSets>)",
     R"(<FilterSet Id="FilterSet:1">)",
      R"(<DichroicRef Id="Dichroic:1" />)",
      R"(<EmissionFilters>)",
       R"(<EmissionFilterRef Id="Filter:2" />)",
      R"(</EmissionFilters>)",
      R"(<ExcitationFilters>)",
       R"(<ExcitationFilterRef Id="Filter:1" />)",
      R"(</ExcitationFilters>)",
     R"(</FilterSet>)",
     R"(<FilterSet Id="FilterSet:2">)",
      R"(<DichroicRef Id="Dichroic:2" />)",
      R"(<EmissionFilters>)",
       R"(<EmissionFilterRef Id="Filter:4" />)",
      R"(</EmissionFilters>)",
      R"(<ExcitationFilters>)",
       R"(<ExcitationFilterRef Id="Filter:3" />)",
      R"(</ExcitationFilters>)",
     R"(</FilterSet>)",
     R"(<FilterSet Id="FilterSet:3">)",
      R"(<DichroicRef Id="Dichroic:3" />)",
      R"(<EmissionFilters>)",
       R"(<EmissionFilterRef Id="Filter:6" />)",
      R"(</EmissionFilters>)",
      R"(<ExcitationFilters>)",
       R"(<ExcitationFilterRef Id="Filter:5" />)",
      R"(</ExcitationFilters>)",
     R"(</FilterSet>)",
     R"(<FilterSet Id="FilterSet:4">)",
      R"(<DichroicRef Id="Dichroic:4" />)",
      R"(<EmissionFilters>)",
       R"(<EmissionFilterRef Id="Filter:8" />)",
      R"(</EmissionFilters>)",
      R"(<ExcitationFilters>)",
       R"(<ExcitationFilterRef Id="Filter:7" />)",
      R"(</ExcitationFilters>)",
     R"(</FilterSet>)",
     R"(<FilterSet Id="FilterSet:5">)",
      R"(<DichroicRef Id="Dichroic:5" />)",
      R"(<EmissionFilters>)",
       R"(<EmissionFilterRef Id="Filter:10" />)",
      R"(</EmissionFilters>)",
      R"(<ExcitationFilters>)",
       R"(<ExcitationFilterRef Id="Filter:9" />)",
      R"(</ExcitationFilters>)",
     R"(</FilterSet>)",
    R"(</FilterSets>)",
    R"(<Filters>)",
     R"(<Filter Id="Filter:1">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>335</CutIn>)",
       R"(<CutOut>383</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:2">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>420</CutIn>)",
       R"(<CutOut>470</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:3">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>350</CutIn>)",
       R"(<CutOut>450</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:4">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>635</CutIn>)",
       R"(<CutOut>675</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:5">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>350</CutIn>)",
       R"(<CutOut>450</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:6">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>585</CutIn>)",
       R"(<CutOut>625</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:7">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>350</CutIn>)",
       R"(<CutOut>450</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:8">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>685</CutIn>)",
       R"(<CutOut>725</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:9">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>350</CutIn>)",
       R"(<CutOut>450</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
     R"(<Filter Id="Filter:10">)",
      R"(<TransmittanceRange>)",
       R"(<CutIn>545</CutIn>)",
       R"(<CutOut>585</CutOut>)",
      R"(</TransmittanceRange>)",
     R"(</Filter>)",
    R"(</Filters>)",
    R"(<Dichroics>)",
     R"(<Dichroic Id="Dichroic:1">)",
      R"(<Wavelengths>)",
       R"(<Wavelength>395</Wavelength>)",
      R"(</Wavelengths>)",
     R"(</Dichroic>)",
     R"(<Dichroic Id="Dichroic:2">)",
      R"(<Wavelengths>)",
       R"(<Wavelength>475</Wavelength>)",
      R"(</Wavelengths>)",
     R"(</Dichroic>)",
     R"(<Dichroic Id="Dichroic:3">)",
      R"(<Wavelengths>)",
       R"(<Wavelength>475</Wavelength>)",
      R"(</Wavelengths>)",
     R"(</Dichroic>)",
     R"(<Dichroic Id="Dichroic:4">)",
      R"(<Wavelengths>)",
       R"(<Wavelength>475</Wavelength>)",
      R"(</Wavelengths>)",
     R"(</Dichroic>)",
     R"(<Dichroic Id="Dichroic:5">)",
      R"(<Wavelengths>)",
       R"(<Wavelength>475</Wavelength>)",
      R"(</Wavelengths>)",
     R"(</Dichroic>)",
    R"(</Dichroics>)",
   R"(</Instrument>)",
   R"(<TimelineTracks>)",
    R"(<TimelineTrack Name="ExecutionInformation Track" Id="Track:1">)",
     R"(<TimelineElements>)",
      R"(<TimelineElement Id="1">)",
       R"(<Time>2013-09-12T16:55:48.9262728Z</Time>)",
       R"(<Duration>0.00</Duration>)",
       R"(<EventInformation>)",
        R"(<ExecutionInformation>)",
         R"(<Type>Pause</Type>)",
        R"(</ExecutionInformation>)",
       R"(</EventInformation>)",
       R"(<Bounds StartB="0" />)",
      R"(</TimelineElement>)",
      R"(<TimelineElement Id="2">)",
       R"(<Time>2013-09-12T16:55:55.1516289Z</Time>)",
       R"(<Duration>0.00</Duration>)",
       R"(<EventInformation>)",
        R"(<ExecutionInformation>)",
         R"(<Type>Resume</Type>)",
        R"(</ExecutionInformation>)",
       R"(</EventInformation>)",
       R"(<Bounds StartB="0" />)",
      R"(</TimelineElement>)",
      R"(<TimelineElement Id="3">)",
       R"(<Time>2013-09-12T17:22:30.6728875Z</Time>)",
       R"(<Duration>0.00</Duration>)",
       R"(<EventInformation>)",
        R"(<ExecutionInformation>)",
         R"(<Type>Pause</Type>)",
        R"(</ExecutionInformation>)",
       R"(</EventInformation>)",
       R"(<Bounds StartB="0" />)",
      R"(</TimelineElement>)",
      R"(<TimelineElement Id="4">)",
       R"(<Time>2013-09-12T17:22:32.833011Z</Time>)",
       R"(<Duration>0.00</Duration>)",
       R"(<EventInformation>)",
        R"(<ExecutionInformation>)",
         R"(<Type>Resume</Type>)",
        R"(</ExecutionInformation>)",
       R"(</EventInformation>)",
       R"(<Bounds StartB="0" />)",
      R"(</TimelineElement>)",
     R"(</TimelineElements>)",
    R"(</TimelineTrack>)",
   R"(</TimelineTracks>)",
  R"(</Information>)",
  R"(<Scaling>)",
   R"(<AutoScaling>)",
    R"(<Type>Measured</Type>)",
    R"(<CameraName>Orca Flash 4.0</CameraName>)",
    R"(<CameraPixelDistance>6.5,6.5</CameraPixelDistance>)",
    R"(<CreationDateTime>01/01/0001 00:00:00</CreationDateTime>)",
   R"(</AutoScaling>)",
   R"(<Items>)",
    R"(<Distance Id="X">)",
     R"(<Value>1.6432520108980473E-07</Value>)",
     R"(<DefaultUnitFormat>um</DefaultUnitFormat>)",
    R"(</Distance>)",
    R"(<Distance Id="Y">)",
     R"(<Value>1.6432520108980473E-07</Value>)",
     R"(<DefaultUnitFormat>um</DefaultUnitFormat>)",
    R"(</Distance>)",
   R"(</Items>)",
  R"(</Scaling>)",
  R"(<DisplaySetting>)",
   R"(<Channels>)",
    R"(<Channel Id="Channel:1" Name="DAPI">)",
     R"(<Low>7.8207124024629588E-05</Low>)",
     R"(<High>0.018017242489859593</High>)",
     R"(<BitCountRange>16</BitCountRange>)",
     R"(<DyeName>DAPI</DyeName>)",
     R"(<ShortName>DAPI</ShortName>)",
     R"(<IlluminationType>Fluorescence</IlluminationType>)",
     R"(<DyeMaxEmission>465</DyeMaxEmission>)",
     R"(<DyeMaxExcitation>353</DyeMaxExcitation>)",
     R"(<DyeId>McNamara-Boswell-0434</DyeId>)",
     R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
     R"(<Color>#FF0000FF</Color>)",
     R"(<OriginalColor>#FF0000FF</OriginalColor>)",
    R"(</Channel>)",
    R"(<Channel Id="Channel:2" Name="QD655">)",
     R"(<High>0.8125027635666251</High>)",
     R"(<BitCountRange>16</BitCountRange>)",
     R"(<DyeName>QD655</DyeName>)",
     R"(<ShortName>QD655</ShortName>)",
     R"(<IlluminationType>Fluorescence</IlluminationType>)",
     R"(<DyeMaxEmission>654</DyeMaxEmission>)",
     R"(<DyeMaxExcitation>250</DyeMaxExcitation>)",
     R"(<DyeId>McNamara-Boswell-0985</DyeId>)",
     R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
     R"(<Color>#FFFF0000</Color>)",
     R"(<OriginalColor>#FFFF0000</OriginalColor>)",
    R"(</Channel>)",
    R"(<Channel Id="Channel:3" Name="QD605">)",
     R"(<Low>7.9927379265427585E-05</Low>)",
     R"(<High>0.57034487892876062</High>)",
     R"(<BitCountRange>16</BitCountRange>)",
     R"(<DyeName>QD605</DyeName>)",
     R"(<ShortName>QD605</ShortName>)",
     R"(<IlluminationType>Fluorescence</IlluminationType>)",
     R"(<DyeMaxEmission>604</DyeMaxEmission>)",
     R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
     R"(<DyeId>McNamara-Boswell-0983</DyeId>)",
     R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
     R"(<Color>#FF00FF00</Color>)",
     R"(<OriginalColor>#FF00FF00</OriginalColor>)",
    R"(</Channel>)",
    R"(<Channel Id="Channel:4" Name="QD705">)",
     R"(<Low>7.9921241849660872E-05</Low>)",
     R"(<High>0.70003913148493635</High>)",
     R"(<BitCountRange>16</BitCountRange>)",
     R"(<DyeName>QD705</DyeName>)",
     R"(<ShortName>QD700</ShortName>)",
     R"(<IlluminationType>Fluorescence</IlluminationType>)",
     R"(<DyeMaxEmission>702</DyeMaxEmission>)",
     R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
     R"(<DyeId>McNamara-Boswell-1538</DyeId>)",
     R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
     R"(<Color>#FFFFFF00</Color>)",
     R"(<OriginalColor>#FFFFFF00</OriginalColor>)",
    R"(</Channel>)",
    R"(<Channel Id="Channel:5" Name="QD565">)",
     R"(<Low>7.9711256548762319E-05</Low>)",
     R"(<High>0.22009835086521193</High>)",
     R"(<BitCountRange>16</BitCountRange>)",
     R"(<DyeName>QD565</DyeName>)",
     R"(<ShortName>QD565</ShortName>)",
     R"(<IlluminationType>Fluorescence</IlluminationType>)",
     R"(<DyeMaxEmission>564</DyeMaxEmission>)",
     R"(<DyeMaxExcitation>300</DyeMaxExcitation>)",
     R"(<DyeId>McNamara-Boswell-0979</DyeId>)",
     R"(<DyeDatabaseId>66071726-cbd4-4c41-b371-0a6eee4ae9c5</DyeDatabaseId>)",
     R"(<Color>#FF00FFFF</Color>)",
     R"(<OriginalColor>#FF00FFFF</OriginalColor>)",
    R"(</Channel>)",
   R"(</Channels>)",
  R"(</DisplaySetting>)",
  R"(<Layers />)",
  R"(<AttachmentInfos>)",
   R"(<AttachmentInfo Id="Label:1">)",
    R"(<Label>)",
     R"(<Barcodes>)",
      R"(<Barcode Id="Barcode:1">)",
       R"(<Type />)",
       R"(<Content />)",
      R"(</Barcode>)",
     R"(</Barcodes>)",
     R"(<OCRs>)",
      R"(<OCR Id="OCR:1">)",
       R"(<Content />)",
      R"(</OCR>)",
     R"(</OCRs>)",
    R"(</Label>)",
   R"(</AttachmentInfo>)",
  R"(</AttachmentInfos>)",
 R"(</Metadata>)",
R"(</ImageDocument>)"
};
// editorconfig-checker-enable

// editorconfig-checker-disable
static const char* test2Metadata[] =
{
R"_(<?xml version="1.0"?>)_",
R"_(<ImageDocument>)_",
R"_( <Metadata>)_",
R"_(  <CustomAttributes>)_",
R"_(   <GrabberName>Carl Zeiss AxioCam</GrabberName>)_",
R"_(   <TransmittedLightFieldstop>0</TransmittedLightFieldstop>)_",
R"_(   <CameraImageAcquisitionTo8Bit>0</CameraImageAcquisitionTo8Bit>)_",
R"_(   <MonoReferenceLow>0</MonoReferenceLow>)_",
R"_(   <MonoReferenceHigh>0</MonoReferenceHigh>)_",
R"_(   <RedReferenceLow>0</RedReferenceLow>)_",
R"_(   <RedReferenceHigh>0</RedReferenceHigh>)_",
R"_(   <GreenReferenceLow>0</GreenReferenceLow>)_",
R"_(   <GreenReferenceHigh>0</GreenReferenceHigh>)_",
R"_(   <BlueReferenceLow>0</BlueReferenceLow>)_",
R"_(   <BlueReferenceHigh>0</BlueReferenceHigh>)_",
R"_(   <CameraUniqueId>0</CameraUniqueId>)_",
R"_(   <CameraTriggerSignalType>0</CameraTriggerSignalType>)_",
R"_(   <CameraTriggerEnable>0</CameraTriggerEnable>)_",
R"_(   <GrabberTimeout>0</GrabberTimeout>)_",
R"_(   <ScaleOriginWidth>0</ScaleOriginWidth>)_",
R"_(   <ScaleOriginHeight>0</ScaleOriginHeight>)_",
R"_(   <ReflectedLightAperture>0</ReflectedLightAperture>)_",
R"_(   <FluoLampIntensity>0</FluoLampIntensity>)_",
R"_(  </CustomAttributes>)_",
R"_(  <Information>)_",
R"_(   <HardwareSetting>)_",
R"_(    <Camera Name="Axio Camera (Type = 1, SNO = 1234567)">)_",
R"_(     <ExposureTime>0</ExposureTime>)_",
R"_(    </Camera>)_",
R"_(    <Microscope>)_",
R"_(     <Model>Axioskop 2</Model>)_",
R"_(    </Microscope>)_",
R"_(    <OptovarChanger>)_",
R"_(     <ChangerElements>)_",
R"_(      <OptovarLens Type="Optovar">)_",
R"_(       <Magnification>0</Magnification>)_",
R"_(      </OptovarLens>)_",
R"_(     </ChangerElements>)_",
R"_(    </OptovarChanger>)_",
R"_(    <TransmittedLightApertureStop>)_",
R"_(     <PositionUnit>NA</PositionUnit>)_",
R"_(     <Position>0</Position>)_",
R"_(    </TransmittedLightApertureStop>)_",
R"_(   </HardwareSetting>)_",
R"_(   <User Id="0" />)_",
R"_(   <Institution />)_",
R"_(   <Document>)_",
R"_(    <Title>DCV</Title>)_",
R"_(    <Author>Carl Zeiss</Author>)_",
R"_(    <Keywords>DCV</Keywords>)_",
R"_(    <Comment>DCV</Comment>)_",
R"_(    <Description>DCV</Description>)_",
R"_(    <UserName>zhjbo</UserName>)_",
R"_(    <Name>DCV.zvi</Name>)_",
R"_(    <CreationDate>2016-02-15T14:23:28.5656661+01:00</CreationDate>)_",
R"_(   </Document>)_",
R"_(   <Image>)_",
R"_(    <SizeX>512</SizeX>)_",
R"_(    <SizeY>512</SizeY>)_",
R"_(    <SizeZ>20</SizeZ>)_",
R"_(    <SizeC>3</SizeC>)_",
R"_(    <PixelType>Gray16</PixelType>)_",
R"_(    <ComponentBitCount>14</ComponentBitCount>)_",
R"_(    <ComponentHighValue>16383</ComponentHighValue>)_",
R"_(    <SizeT>1</SizeT>)_",
R"_(    <OriginalCompressionMethod>Uncompressed</OriginalCompressionMethod>)_",
R"_(    <OriginalEncodingQuality>100</OriginalEncodingQuality>)_",
R"_(    <Dimensions>)_",
R"_(     <Channels>)_",
R"_(      <Channel Id="Channel:0">)_",
R"_(       <PixelType>Gray16</PixelType>)_",
R"_(       <ComponentBitCount>14</ComponentBitCount>)_",
R"_(       <IlluminationWavelength />)_",
R"_(       <DetectionWavelength />)_",
R"_(       <AiryscanSettings />)_",
R"_(      </Channel>)_",
R"_(      <Channel Id="Channel:1">)_",
R"_(       <PixelType>Gray16</PixelType>)_",
R"_(       <ComponentBitCount>14</ComponentBitCount>)_",
R"_(       <IlluminationWavelength />)_",
R"_(       <DetectionWavelength />)_",
R"_(       <AiryscanSettings />)_",
R"_(      </Channel>)_",
R"_(      <Channel Id="Channel:2">)_",
R"_(       <PixelType>Gray16</PixelType>)_",
R"_(       <ComponentBitCount>14</ComponentBitCount>)_",
R"_(       <IlluminationWavelength />)_",
R"_(       <DetectionWavelength />)_",
R"_(       <AiryscanSettings />)_",
R"_(      </Channel>)_",
R"_(     </Channels>)_",
R"_(     <T>)_",
R"_(      <StartTime>0001-01-01T00:00:00</StartTime>)_",
R"_(     </T>)_",
R"_(     <Z>)_",
R"_(      <StartPosition>-11.25</StartPosition>)_",
R"_(      <Positions>)_",
R"_(       <Interval>)_",
R"_(        <Start>0</Start>)_",
R"_(        <Increment>9.525</Increment>)_",
R"_(       </Interval>)_",
R"_(      </Positions>)_",
R"_(     </Z>)_",
R"_(    </Dimensions>)_",
R"_(   </Image>)_",
R"_(   <Instrument />)_",
R"_(   <Processing>)_",
R"_(    <CustomAttributes />)_",
R"_(    <PSF>)_",
R"_(     <ZStackDirection>Descending</ZStackDirection>)_",
R"_(     <Dimension>ThreeDimensional</Dimension>)_",
R"_(     <WorkingDistance>0</WorkingDistance>)_",
R"_(     <UsedImmersionIndex>1.4</UsedImmersionIndex>)_",
R"_(     <UsedCoverglassIndex>1.515</UsedCoverglassIndex>)_",
R"_(     <UsedCoverglassThickness>170</UsedCoverglassThickness>)_",
R"_(     <EmbeddingIndex>1</EmbeddingIndex>)_",
R"_(     <EmbeddingThickness>5</EmbeddingThickness>)_",
R"_(     <NAObjective>1</NAObjective>)_",
R"_(     <Channels>)_",
R"_(      <c0 Id="Channel:0">)_",
R"_(       <AttachedPsfName>None</AttachedPsfName>)_",
R"_(       <IlluminationWavelength>500</IlluminationWavelength>)_",
R"_(       <Source>ScalarTheory</Source>)_",
R"_(       <DetectionWavelength>550</DetectionWavelength>)_",
R"_(      </c0>)_",
R"_(      <c1 Id="Channel:1">)_",
R"_(       <AttachedPsfName>None</AttachedPsfName>)_",
R"_(       <IlluminationWavelength>550</IlluminationWavelength>)_",
R"_(       <Source>ScalarTheory</Source>)_",
R"_(       <DetectionWavelength>580</DetectionWavelength>)_",
R"_(      </c1>)_",
R"_(      <c2 Id="Channel:2">)_",
R"_(       <AttachedPsfName>None</AttachedPsfName>)_",
R"_(       <IlluminationWavelength>600</IlluminationWavelength>)_",
R"_(       <Source>ScalarTheory</Source>)_",
R"_(       <DetectionWavelength>630</DetectionWavelength>)_",
R"_(      </c2>)_",
R"_(     </Channels>)_",
R"_(    </PSF>)_",
R"_(   </Processing>)_",
R"_(  </Information>)_",
R"_(  <Scaling>)_",
R"_(   <AutoScaling>)_",
R"_(    <Type>Measured</Type>)_",
R"_(    <CreationDateTime>02/15/2016 12:57:07</CreationDateTime>)_",
R"_(   </AutoScaling>)_",
R"_(   <Items>)_",
R"_(    <Distance Id="X">)_",
R"_(     <Value>1.06822E-07</Value>)_",
R"_(     <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(    </Distance>)_",
R"_(    <Distance Id="Y">)_",
R"_(     <Value>1.06822E-07</Value>)_",
R"_(     <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(    </Distance>)_",
R"_(    <Distance Id="Z">)_",
R"_(     <Value>5E-07</Value>)_",
R"_(     <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(    </Distance>)_",
R"_(   </Items>)_",
R"_(  </Scaling>)_",
R"_(  <DisplaySetting>)_",
R"_(   <Channels>)_",
R"_(    <Channel Id="Channel:0" Name="FITC">)_",
R"_(     <High>0.245260663507109</High>)_",
R"_(     <Gamma>0.32119738387296742</Gamma>)_",
R"_(     <BitCountRange>14</BitCountRange>)_",
R"_(     <DyeName>FITC</DyeName>)_",
R"_(     <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(     <DyeMaxEmission>525</DyeMaxEmission>)_",
R"_(     <DyeMaxExcitation>490</DyeMaxExcitation>)_",
R"_(     <Color>#FF41FF00</Color>)_",
R"_(     <ColorMode>Custom</ColorMode>)_",
R"_(     <OriginalColor>#FF41FF00</OriginalColor>)_",
R"_(    </Channel>)_",
R"_(    <Channel Id="Channel:1" Name="Rhodamin">)_",
R"_(     <Low>0.026066350710900438</Low>)_",
R"_(     <High>0.10130331753554503</High>)_",
R"_(     <Gamma>0.45</Gamma>)_",
R"_(     <BitCountRange>14</BitCountRange>)_",
R"_(     <Mode>Spline</Mode>)_",
R"_(     <Points>0.196850393700788,0.757142857142857 0.346456692913386,0.0428571428571428</Points>)_",
R"_(     <DyeName>Rhodamin</DyeName>)_",
R"_(     <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(     <DyeMaxEmission>580</DyeMaxEmission>)_",
R"_(     <DyeMaxExcitation>550</DyeMaxExcitation>)_",
R"_(     <Color>#FFFFD200</Color>)_",
R"_(     <OriginalColor>#FFFFD200</OriginalColor>)_",
R"_(    </Channel>)_",
R"_(    <Channel Id="Channel:2" Name="DAPI">)_",
R"_(     <High>0.09597156398104266</High>)_",
R"_(     <BitCountRange>14</BitCountRange>)_",
R"_(     <DyeName>DAPI</DyeName>)_",
R"_(     <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(     <DyeMaxEmission>461</DyeMaxEmission>)_",
R"_(     <DyeMaxExcitation>359</DyeMaxExcitation>)_",
R"_(     <Color>#FF4178FF</Color>)_",
R"_(     <OriginalColor>#FF4178FF</OriginalColor>)_",
R"_(    </Channel>)_",
R"_(   </Channels>)_",
R"_(  </DisplaySetting>)_",
R"_(  <Layers />)_",
R"_(  <MetadataNodes>)_",
R"_(   <MetadataNode StartC="0">)_",
R"_(    <Information>)_",
R"_(     <HardwareSetting>)_",
R"_(      <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(       <ExposureTime>997</ExposureTime>)_",
R"_(      </Camera>)_",
R"_(      <TransmittedLightApertureStop>)_",
R"_(       <PositionUnit>NA</PositionUnit>)_",
R"_(       <Position>0.07</Position>)_",
R"_(      </TransmittedLightApertureStop>)_",
R"_(     </HardwareSetting>)_",
R"_(    </Information>)_",
R"_(   </MetadataNode>)_",
R"_(   <MetadataNode StartC="1">)_",
R"_(    <Information>)_",
R"_(     <HardwareSetting>)_",
R"_(      <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(       <ExposureTime>476</ExposureTime>)_",
R"_(      </Camera>)_",
R"_(      <TransmittedLightApertureStop>)_",
R"_(       <PositionUnit>NA</PositionUnit>)_",
R"_(       <Position>0.07</Position>)_",
R"_(      </TransmittedLightApertureStop>)_",
R"_(     </HardwareSetting>)_",
R"_(    </Information>)_",
R"_(   </MetadataNode>)_",
R"_(   <MetadataNode StartC="2">)_",
R"_(    <Information>)_",
R"_(     <HardwareSetting>)_",
R"_(      <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(       <ExposureTime>72</ExposureTime>)_",
R"_(      </Camera>)_",
R"_(      <TransmittedLightApertureStop>)_",
R"_(       <PositionUnit>NA</PositionUnit>)_",
R"_(       <Position>0.07</Position>)_",
R"_(      </TransmittedLightApertureStop>)_",
R"_(     </HardwareSetting>)_",
R"_(    </Information>)_",
R"_(   </MetadataNode>)_",
R"_(  </MetadataNodes>)_",
R"_( </Metadata>)_",
R"_(</ImageDocument>)_"
};
// editorconfig-checker-enable

// editorconfig-checker-disable
static const char* test3Metadata[] =
{
R"_(<ImageDocument>)_",
R"_(  <Metadata>)_",
R"_(    <CustomAttributes>)_",
R"_(      <GrabberName>Carl Zeiss AxioCam</GrabberName>)_",
R"_(      <TransmittedLightFieldstop>0</TransmittedLightFieldstop>)_",
R"_(      <CameraImageAcquisitionTo8Bit>0</CameraImageAcquisitionTo8Bit>)_",
R"_(      <MonoReferenceLow>0</MonoReferenceLow>)_",
R"_(      <MonoReferenceHigh>0</MonoReferenceHigh>)_",
R"_(      <RedReferenceLow>0</RedReferenceLow>)_",
R"_(      <RedReferenceHigh>0</RedReferenceHigh>)_",
R"_(      <GreenReferenceLow>0</GreenReferenceLow>)_",
R"_(      <GreenReferenceHigh>0</GreenReferenceHigh>)_",
R"_(      <BlueReferenceLow>0</BlueReferenceLow>)_",
R"_(      <BlueReferenceHigh>0</BlueReferenceHigh>)_",
R"_(      <CameraUniqueId>0</CameraUniqueId>)_",
R"_(      <CameraTriggerSignalType>0</CameraTriggerSignalType>)_",
R"_(      <CameraTriggerEnable>0</CameraTriggerEnable>)_",
R"_(      <GrabberTimeout>0</GrabberTimeout>)_",
R"_(      <ScaleOriginWidth>0</ScaleOriginWidth>)_",
R"_(      <ScaleOriginHeight>0</ScaleOriginHeight>)_",
R"_(      <ReflectedLightAperture>0</ReflectedLightAperture>)_",
R"_(      <FluoLampIntensity>0</FluoLampIntensity>)_",
R"_(    </CustomAttributes>)_",
R"_(    <Information>)_",
R"_(      <HardwareSetting>)_",
R"_(        <Camera Name="Axio Camera (Type = 1, SNO = 1234567)">)_",
R"_(          <ExposureTime>0</ExposureTime>)_",
R"_(        </Camera>)_",
R"_(        <Microscope>)_",
R"_(          <Model>Axioskop 2</Model>)_",
R"_(        </Microscope>)_",
R"_(        <OptovarChanger>)_",
R"_(          <ChangerElements>)_",
R"_(            <OptovarLens Type="Optovar">)_",
R"_(              <Magnification>0</Magnification>)_",
R"_(            </OptovarLens>)_",
R"_(          </ChangerElements>)_",
R"_(        </OptovarChanger>)_",
R"_(        <TransmittedLightApertureStop>)_",
R"_(          <PositionUnit>NA</PositionUnit>)_",
R"_(          <Position>0</Position>)_",
R"_(        </TransmittedLightApertureStop>)_",
R"_(      </HardwareSetting>)_",
R"_(      <User Id="0" />)_",
R"_(      <Institution />)_",
R"_(      <Document>)_",
R"_(        <Title>DCV</Title>)_",
R"_(        <Author>Carl Zeiss</Author>)_",
R"_(        <Keywords>DCV</Keywords>)_",
R"_(        <Comment>DCV</Comment>)_",
R"_(        <Description>DCV</Description>)_",
R"_(        <UserName>zhwba</UserName>)_",
R"_(        <Name>DCV.zvi</Name>)_",
R"_(        <CreationDate>2012-07-23T10:55:51.8124499+02:00</CreationDate>)_",
R"_(      </Document>)_",
R"_(      <Image>)_",
R"_(        <SizeX>512</SizeX>)_",
R"_(        <SizeY>512</SizeY>)_",
R"_(        <SizeZ>20</SizeZ>)_",
R"_(        <SizeC>3</SizeC>)_",
R"_(        <PixelType>Gray16</PixelType>)_",
R"_(        <ComponentBitCount>14</ComponentBitCount>)_",
R"_(        <ComponentHighValue>16383</ComponentHighValue>)_",
R"_(        <SizeT>1</SizeT>)_",
R"_(        <Dimensions>)_",
R"_(          <Channels>)_",
R"_(            <Channel Id="Channel:0">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(            <Channel Id="Channel:1">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(            <Channel Id="Channel:2">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(          </Channels>)_",
R"_(          <T>)_",
R"_(            <StartTime>0001-01-01T00:00:00</StartTime>)_",
R"_(          </T>)_",
R"_(          <Z>)_",
R"_(            <StartPosition>-11.25</StartPosition>)_",
R"_(            <Positions>)_",
R"_(              <Interval>)_",
R"_(                <Start>0</Start>)_",
R"_(                <Increment>9.525</Increment>)_",
R"_(              </Interval>)_",
R"_(            </Positions>)_",
R"_(          </Z>)_",
R"_(        </Dimensions>)_",
R"_(      </Image>)_",
R"_(      <Instrument />)_",
R"_(      <Processing>)_",
R"_(        <CustomAttributes />)_",
R"_(        <PSF>)_",
R"_(          <ZStackDirection>Descending</ZStackDirection>)_",
R"_(          <Dimension>ThreeDimensional</Dimension>)_",
R"_(          <WorkingDistance>0</WorkingDistance>)_",
R"_(          <UsedImmersionIndex>1.4</UsedImmersionIndex>)_",
R"_(          <UsedCoverglassIndex>1.515</UsedCoverglassIndex>)_",
R"_(          <UsedCoverglassThickness>170</UsedCoverglassThickness>)_",
R"_(          <EmbeddingIndex>1</EmbeddingIndex>)_",
R"_(          <EmbeddingThickness>5</EmbeddingThickness>)_",
R"_(          <NAObjective>1</NAObjective>)_",
R"_(          <Channels>)_",
R"_(            <c0 Id="Channel:0">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>500</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>550</DetectionWavelength>)_",
R"_(            </c0>)_",
R"_(            <c1 Id="Channel:1">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>550</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>580</DetectionWavelength>)_",
R"_(            </c1>)_",
R"_(            <c2 Id="Channel:2">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>600</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>630</DetectionWavelength>)_",
R"_(            </c2>)_",
R"_(          </Channels>)_",
R"_(        </PSF>)_",
R"_(      </Processing>)_",
R"_(    </Information>)_",
R"_(    <Scaling>)_",
R"_(      <Items>)_",
R"_(        <Distance Id="X">)_",
R"_(          <Value>1.06822E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(        <Distance Id="Y">)_",
R"_(          <Value>1.06822E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(        <Distance Id="Z">)_",
R"_(          <Value>5E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(      </Items>)_",
R"_(    </Scaling>)_",
R"_(    <DisplaySetting>)_",
R"_(      <Channels>)_",
R"_(        <Channel Id="Channel:0" Name="FITC">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>FITC</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>525</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>490</DyeMaxExcitation>)_",
R"_(          <Color>#FF41FF00</Color>)_",
R"_(          <OriginalColor>#FF41FF00</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(        <Channel Id="Channel:1" Name="Rhodamin">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>Rhodamin</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>580</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>550</DyeMaxExcitation>)_",
R"_(          <Color>#FFFFD200</Color>)_",
R"_(          <OriginalColor>#FFFFD200</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(        <Channel Id="Channel:2" Name="DAPI">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>DAPI</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>461</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>359</DyeMaxExcitation>)_",
R"_(          <Color>#FF4178FF</Color>)_",
R"_(          <OriginalColor>#FF4178FF</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(      </Channels>)_",
R"_(    </DisplaySetting>)_",
R"_(    <Layers />)_",
R"_(    <MetadataNodes>)_",
R"_(      <MetadataNode StartC="0">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>997</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(      <MetadataNode StartC="1">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>476</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(      <MetadataNode StartC="2">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>72</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(    </MetadataNodes>)_",
R"_(  </Metadata>)_",
R"_(</ImageDocument>)_"
};
// editorconfig-checker-enable

// editorconfig-checker-disable
static const char* test4Metadata[] =
{
    R"_(<ImageDocument>)_",
R"_(  <Metadata>)_",
R"_(    <CustomAttributes>)_",
R"_(      <GrabberName>Carl Zeiss AxioCam</GrabberName>)_",
R"_(      <TransmittedLightFieldstop>0</TransmittedLightFieldstop>)_",
R"_(      <CameraImageAcquisitionTo8Bit>0</CameraImageAcquisitionTo8Bit>)_",
R"_(      <MonoReferenceLow>0</MonoReferenceLow>)_",
R"_(      <MonoReferenceHigh>0</MonoReferenceHigh>)_",
R"_(      <RedReferenceLow>0</RedReferenceLow>)_",
R"_(      <RedReferenceHigh>0</RedReferenceHigh>)_",
R"_(      <GreenReferenceLow>0</GreenReferenceLow>)_",
R"_(      <GreenReferenceHigh>0</GreenReferenceHigh>)_",
R"_(      <BlueReferenceLow>0</BlueReferenceLow>)_",
R"_(      <BlueReferenceHigh>0</BlueReferenceHigh>)_",
R"_(      <CameraUniqueId>0</CameraUniqueId>)_",
R"_(      <CameraTriggerSignalType>0</CameraTriggerSignalType>)_",
R"_(      <CameraTriggerEnable>0</CameraTriggerEnable>)_",
R"_(      <GrabberTimeout>0</GrabberTimeout>)_",
R"_(      <ScaleOriginWidth>0</ScaleOriginWidth>)_",
R"_(      <ScaleOriginHeight>0</ScaleOriginHeight>)_",
R"_(      <ReflectedLightAperture>0</ReflectedLightAperture>)_",
R"_(      <FluoLampIntensity>0</FluoLampIntensity>)_",
R"_(    </CustomAttributes>)_",
R"_(    <Information>)_",
R"_(      <HardwareSetting>)_",
R"_(        <Camera Name="Axio Camera (Type = 1, SNO = 1234567)">)_",
R"_(          <ExposureTime>0</ExposureTime>)_",
R"_(        </Camera>)_",
R"_(        <Microscope>)_",
R"_(          <Model>Axioskop 2</Model>)_",
R"_(        </Microscope>)_",
R"_(        <OptovarChanger>)_",
R"_(          <ChangerElements>)_",
R"_(            <OptovarLens Type="Optovar">)_",
R"_(              <Magnification>0</Magnification>)_",
R"_(            </OptovarLens>)_",
R"_(          </ChangerElements>)_",
R"_(        </OptovarChanger>)_",
R"_(        <TransmittedLightApertureStop>)_",
R"_(          <PositionUnit>NA</PositionUnit>)_",
R"_(          <Position>0</Position>)_",
R"_(        </TransmittedLightApertureStop>)_",
R"_(      </HardwareSetting>)_",
R"_(      <User Id="0" />)_",
R"_(      <Institution />)_",
R"_(      <Document>)_",
R"_(        <Title>DCV</Title>)_",
R"_(        <Author>Carl Zeiss</Author>)_",
R"_(        <Keywords>DCV</Keywords>)_",
R"_(        <Comment>DCV</Comment>)_",
R"_(        <Description>DCV</Description>)_",
R"_(        <UserName>zhwba</UserName>)_",
R"_(        <Name>DCV.zvi</Name>)_",
R"_(        <CreationDate>2012-07-23T10:55:51.8124499+02:00</CreationDate>)_",
R"_(      </Document>)_",
R"_(      <Image>)_",
R"_(        <SizeX>512</SizeX>)_",
R"_(        <SizeY>512</SizeY>)_",
R"_(        <SizeZ>20</SizeZ>)_",
R"_(        <SizeC>3</SizeC>)_",
R"_(        <PixelType>Gray16</PixelType>)_",
R"_(        <ComponentBitCount>14</ComponentBitCount>)_",
R"_(        <ComponentHighValue>16383</ComponentHighValue>)_",
R"_(        <SizeT>1</SizeT>)_",
R"_(        <Dimensions>)_",
R"_(          <Channels>)_",
R"_(            <Channel Id="Channel:0">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(            <Channel Id="Channel:1">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(            <Channel Id="Channel:2">)_",
R"_(              <PixelType>Gray16</PixelType>)_",
R"_(              <ComponentBitCount>14</ComponentBitCount>)_",
R"_(              <IlluminationWavelength />)_",
R"_(              <DetectionWavelength />)_",
R"_(            </Channel>)_",
R"_(          </Channels>)_",
R"_(          <T>)_",
R"_(            <StartTime>0001-01-01T00:00:00</StartTime>)_",
R"_(          </T>)_",
R"_(          <Z>)_",
R"_(            <StartPosition>-11.25</StartPosition>)_",
R"_(            <Positions>)_",
R"_(              <List>)_",
R"_(                <Offsets>  0   0.5 1 1.5 2 2.5 3 3.8 4.2</Offsets>)_",
R"_(              </List>)_",
R"_(            </Positions>)_",
R"_(          </Z>)_",
R"_(        </Dimensions>)_",
R"_(      </Image>)_",
R"_(      <Instrument />)_",
R"_(      <Processing>)_",
R"_(        <CustomAttributes />)_",
R"_(        <PSF>)_",
R"_(          <ZStackDirection>Descending</ZStackDirection>)_",
R"_(          <Dimension>ThreeDimensional</Dimension>)_",
R"_(          <WorkingDistance>0</WorkingDistance>)_",
R"_(          <UsedImmersionIndex>1.4</UsedImmersionIndex>)_",
R"_(          <UsedCoverglassIndex>1.515</UsedCoverglassIndex>)_",
R"_(          <UsedCoverglassThickness>170</UsedCoverglassThickness>)_",
R"_(          <EmbeddingIndex>1</EmbeddingIndex>)_",
R"_(          <EmbeddingThickness>5</EmbeddingThickness>)_",
R"_(          <NAObjective>1</NAObjective>)_",
R"_(          <Channels>)_",
R"_(            <c0 Id="Channel:0">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>500</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>550</DetectionWavelength>)_",
R"_(            </c0>)_",
R"_(            <c1 Id="Channel:1">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>550</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>580</DetectionWavelength>)_",
R"_(            </c1>)_",
R"_(            <c2 Id="Channel:2">)_",
R"_(              <AttachedPsfName>None</AttachedPsfName>)_",
R"_(              <IlluminationWavelength>600</IlluminationWavelength>)_",
R"_(              <Source>ScalarTheory</Source>)_",
R"_(              <DetectionWavelength>630</DetectionWavelength>)_",
R"_(            </c2>)_",
R"_(          </Channels>)_",
R"_(        </PSF>)_",
R"_(      </Processing>)_",
R"_(    </Information>)_",
R"_(    <Scaling>)_",
R"_(      <Items>)_",
R"_(        <Distance Id="X">)_",
R"_(          <Value>1.06822E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(        <Distance Id="Y">)_",
R"_(          <Value>1.06822E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(        <Distance Id="Z">)_",
R"_(          <Value>5E-07</Value>)_",
R"_(          <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(        </Distance>)_",
R"_(      </Items>)_",
R"_(    </Scaling>)_",
R"_(    <DisplaySetting>)_",
R"_(      <Channels>)_",
R"_(        <Channel Id="Channel:0" Name="FITC">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>FITC</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>525</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>490</DyeMaxExcitation>)_",
R"_(          <Color>#FF41FF00</Color>)_",
R"_(          <OriginalColor>#FF41FF00</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(        <Channel Id="Channel:1" Name="Rhodamin">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>Rhodamin</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>580</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>550</DyeMaxExcitation>)_",
R"_(          <Color>#FFFFD200</Color>)_",
R"_(          <OriginalColor>#FFFFD200</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(        <Channel Id="Channel:2" Name="DAPI">)_",
R"_(          <High>0.25</High>)_",
R"_(          <BitCountRange>14</BitCountRange>)_",
R"_(          <DyeName>DAPI</DyeName>)_",
R"_(          <IlluminationType>Fluorescence</IlluminationType>)_",
R"_(          <DyeMaxEmission>461</DyeMaxEmission>)_",
R"_(          <DyeMaxExcitation>359</DyeMaxExcitation>)_",
R"_(          <Color>#FF4178FF</Color>)_",
R"_(          <OriginalColor>#FF4178FF</OriginalColor>)_",
R"_(        </Channel>)_",
R"_(      </Channels>)_",
R"_(    </DisplaySetting>)_",
R"_(    <Layers />)_",
R"_(    <MetadataNodes>)_",
R"_(      <MetadataNode StartC="0">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>997</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(      <MetadataNode StartC="1">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>476</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(      <MetadataNode StartC="2">)_",
R"_(        <Information>)_",
R"_(          <HardwareSetting>)_",
R"_(            <Camera Name="Axio Camera (Type = 0, SNO = 101)">)_",
R"_(              <ExposureTime>72</ExposureTime>)_",
R"_(            </Camera>)_",
R"_(            <TransmittedLightApertureStop>)_",
R"_(              <PositionUnit>NA</PositionUnit>)_",
R"_(              <Position>0.07</Position>)_",
R"_(            </TransmittedLightApertureStop>)_",
R"_(          </HardwareSetting>)_",
R"_(        </Information>)_",
R"_(      </MetadataNode>)_",
R"_(    </MetadataNodes>)_",
R"_(  </Metadata>)_",
R"_(</ImageDocument>)_"
};
// editorconfig-checker-enable

// editorconfig-checker-disable
static const char* test5Metadata[] =
{
R"_(<?xml version="1.0"?>)_",
R"_(<ImageDocument>)_",
R"_()",
R"_(<Metadata>)_",
R"_(     <!-- ================================== Scaling ====================================-->)_",
R"_(   <Scaling>)_",
R"_(     <AutoScaling>)_",
R"_(       <CameraFramePixelDistance>1</CameraFramePixelDistance>)_",
R"_(       <CameraFrameBinning>1</CameraFrameBinning>)_",
R"_(       <CameraAdapterMagnification>1</CameraAdapterMagnification>)_",
R"_(       <Optovar>1</Optovar>)_",
R"_(       <ReflectorMagnification>1</ReflectorMagnification>)_",
R"_(       <Objective>Plan-Neofluar 1.25x/0.04</Objective>)_",
R"_(     </AutoScaling>)_",
R"_(     <Items>)_",
R"_(       <Distance Id="X">)_",
R"_(         <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(         <Direction>-1</Direction>)_",
R"_(         <Origin>1</Origin>)_",
R"_(         <Value>1.21e-6</Value>)_",
R"_(       </Distance>)_",
R"_(       <Distance Id="Y">)_",
R"_(         <DefaultUnitFormat>um</DefaultUnitFormat>)_",
R"_(         <Direction>-1</Direction>)_",
R"_(         <Origin>1</Origin>)_",
R"_(         <Value>1.22e-6</Value>)_",
R"_(       </Distance>)_",
R"_(       <TimeSpan Id="Z">)_",
R"_(         <DefaultUnitFormat>ms</DefaultUnitFormat>)_",
R"_(         <Value>1.22e-3</Value>)_",
R"_(       </TimeSpan>)_",
R"_(       <Pixels Id="I">)_",
R"_(         <Value>145</Value>)_",
R"_(       </Pixels>)_",
R"_(     </Items>)_",
R"_(   </Scaling>)_",
R"_(   )_",
R"_(   <!-- ================================== DisplaySetting ====================================-->)_",
R"_(   <DisplaySetting>)_",
R"_(     <Channels>)_",
R"_(       <Channel StartC ="0" Id="DAPI">)_",
R"_(         <Low>0.1</Low>)_",
R"_(         <High>0.5</High>)_",
R"_(         <DyeName>DAPI</DyeName>)_",
R"_(         <Color>Red</Color>)_",
R"_(         <ColorMode>Color</ColorMode>)_",
R"_(       </Channel>)_",
R"_()_",
R"_(        <Channel StartC="1" Id="FITC">)_",
R"_(          <DyeName>FITC</DyeName>)_",
R"_(          <Color>Blue</Color>)_",
R"_(          <ColorMode>Color</ColorMode>)_",
R"_(        </Channel>)_",
R"_()_",
R"_(        <Channel StartC="2" Id="Rhodamin">)_",
R"_(          <DyeName>Rhodamin</DyeName>)_",
R"_(          <Color>Green</Color>)_",
R"_(          <ColorMode>Color</ColorMode>)_",
R"_(        </Channel>)_",
R"_(      </Channels>)_",
R"_(    </DisplaySetting>)_",
R"_()_",
R"_(    <!-- Profile: proposal 1 -->)_",
R"_(    <AttachmentInfos>)_",
R"_(      <AttachmentInfo Id="Profile">)_",
R"_(        <Profile>)_",
R"_(          <ProfileName>WidefieldProfile</ProfileName>)_",
R"_(        </Profile>)_",
R"_(      </AttachmentInfo>)_",
R"_(    </AttachmentInfos>)_",
R"_()_",
R"_(    <!-- ================================== Information ====================================-->)_",
R"_(    <Information>)_",
R"_(      <Image>)_",
R"_(        <SizeX>1024</SizeX>)_",
R"_(        <SizeY>1024</SizeY>)_",
R"_(        <SizeC>4</SizeC>)_",
R"_(        <SizeZ>20</SizeZ>)_",
R"_(        <SizeT>100</SizeT>)_",
R"_()_",
R"_(        <!-- Profile: proposal 2 -->)_",
R"_(        <!--<Profile>)_",
R"_(          <ProfileName>BFProfile</ProfileName>)_",
R"_(          <AttachmentName>Profile</AttachmentName>)_",
R"_(        </Profile>-->)_",
R"_(        )_",
R"_(        <AcquisitionDateAndTime>2001-01-28T00:00:00</AcquisitionDateAndTime>)_",
R"_(        <PixelType>Gray8</PixelType>)_",
R"_(        <ComponentBitCount>8</ComponentBitCount>)_",
R"_(        <OriginalScanData>true</OriginalScanData>)_",
R"_()_",
R"_(        <ObjectiveSettings>)_",
R"_(          <ObjectiveRef Id="Objective:1"/>)_",
R"_(          <Medium>Glycerol</Medium>)_",
R"_(          <CorrectionCollar>2</CorrectionCollar>)_",
R"_(          <RefractiveIndex>1.7</RefractiveIndex>)_",
R"_(        </ObjectiveSettings>)_",
R"_()_",
R"_(        <Dimensions>)_",
R"_(          <Channels>)_",
R"_(            <Channel Id="Ch1" Name="Ch1">)_",
R"_(              <AcquisitionMode>LaserScanningConfocalMicroscopy</AcquisitionMode>)_",
R"_(              <IlluminationType>Epifluorescence</IlluminationType>)_",
R"_(              <ContrastMethod>Fluorescence</ContrastMethod>)_",
R"_()_",
R"_(              <IlluminationWavelength>)_",
R"_(                <Ranges>400-420,460-500,600</Ranges>)_",
R"_(              </IlluminationWavelength>)_",
R"_()_",
R"_(              <DetectionWavelength>)_",
R"_(                <SinglePeak>420</SinglePeak>)_",
R"_(              </DetectionWavelength>)_",
R"_()_",
R"_(              <ExcitationWavelength>500</ExcitationWavelength>)_",
R"_(              <EmissionWavelength>490</EmissionWavelength>)_",
R"_()_",
R"_(              <Fluor>DAPI</Fluor>)_",
R"_(              <PinholeSize>8.5</PinholeSize>)_",
R"_(              <PinholeSizeAiry>3.95</PinholeSizeAiry>)_",
R"_(              <NACondenser>1.3</NACondenser>)_",
R"_(              <ExposureTime>100 - 2993</ExposureTime>)_",
R"_(              )_",
R"_(              <DyeId>5</DyeId>)_",
R"_(              <DyeDatabaseId>42</DyeDatabaseId>)_",
R"_()_",
R"_(              <DetectorSettings>)_",
R"_(                <Detector Id="Detector1"/>)_",
R"_(                <Binning>2x2</Binning>)_",
R"_(                <EMGain>4.2</EMGain>)_",
R"_(              </DetectorSettings>)_",
R"_()_",
R"_(              <LightSourcesSettings>)_",
R"_(                <LightSourceSettings>)_",
R"_(                  <LightSource Id="LightSource1"/>)_",
R"_(                  <Wavelength>442</Wavelength>)_",
R"_(                </LightSourceSettings>)_",
R"_(              </LightSourcesSettings>)_",
R"_(            </Channel>)_",
R"_(            )_",
R"_(            <Channel Id="FITC" Name="ch2">)_",
R"_(              <AcquisitionMode>WideField</AcquisitionMode>)_",
R"_(              <IlluminationType>Transmitted</IlluminationType>)_",
R"_(              <ContrastMethod>Brightfield</ContrastMethod>)_",
R"_()_",
R"_(              <ExcitationWavelength>480</ExcitationWavelength>)_",
R"_(              <EmissionWavelength>344</EmissionWavelength>)_",
R"_()_",
R"_(              <DyeId>6</DyeId>)_",
R"_(              <DyeDatabaseId>42</DyeDatabaseId>)_",
R"_(              <ExposureTime>10099</ExposureTime>)_",
R"_()_",
R"_(              <DetectorSettings>)_",
R"_(                <Detector Id="Detector2"/>)_",
R"_(                <Binning>2x2</Binning>)_",
R"_(                <EMGain>4.2</EMGain>)_",
R"_(              </DetectorSettings>)_",
R"_()_",
R"_(              <LightSourcesSettings>)_",
R"_(                <LightSourceSettings>)_",
R"_(                  <LightSource Id="LightSource2"/>)_",
R"_(                  <Wavelength>449</Wavelength>)_",
R"_(                  <Attenuation>1.5</Attenuation>)_",
R"_(                </LightSourceSettings>)_",
R"_(              </LightSourcesSettings>)_",
R"_()_",
R"_(            </Channel>)_",
R"_(            )_",
R"_(            <Channel Id="Rhodamin" Name="ch3">)_",
R"_(              <AcquisitionMode>LaserScanningConfocalMicroscopy</AcquisitionMode>)_",
R"_(              <IlluminationType>Epifluorescence</IlluminationType>)_",
R"_(              <ContrastMethod>Brightfield</ContrastMethod>)_",
R"_()_",
R"_(              <IlluminationWavelength>)_",
R"_(                <Ranges>-100,400-480,500-520,600,800-</Ranges>)_",
R"_(              </IlluminationWavelength>)_",
R"_()_",
R"_(              <DetectionWavelength>)_",
R"_(                <SinglePeak>620</SinglePeak>)_",
R"_(              </DetectionWavelength>)_",
R"_()_",
R"_(              <ExcitationWavelength>444</ExcitationWavelength>)_",
R"_(              <EmissionWavelength>324</EmissionWavelength>)_",
R"_()_",
R"_(              <DetectorSettings>)_",
R"_(                <Detector Id="Detector2"/>)_",
R"_(                <Binning>2x2</Binning>)_",
R"_(                <EMGain>4.2</EMGain>)_",
R"_(              </DetectorSettings>)_",
R"_()_",
R"_(              <LightSourcesSettings>)_",
R"_(                <LightSourceSettings>)_",
R"_(                  <LightSource Id="LightSource1"/>)_",
R"_(                  <Wavelength>442</Wavelength>)_",
R"_(                </LightSourceSettings>)_",
R"_(                <LightSourceSettings>)_",
R"_(                  <LightSource Id="LightSource2"/>)_",
R"_(                  <Wavelength>449</Wavelength>)_",
R"_(                  <Attenuation>1.5</Attenuation>)_",
R"_(                </LightSourceSettings>)_",
R"_(              </LightSourcesSettings>)_",
R"_()_",
R"_(              <LaserScanInfo>)_",
R"_(                <PixelTime>4</PixelTime>)_",
R"_(                <LineTime>44</LineTime>)_",
R"_(                <ScanningMode>FastFrame</ScanningMode>)_",
R"_(                <SampleRotation>30</SampleRotation>)_",
R"_(                <ZoomX>4</ZoomX>)_",
R"_(              </LaserScanInfo>)_",
R"_(            </Channel>)_",
R"_(          </Channels>)_",
R"_()_",
R"_(          <Tracks>)_",
R"_(            <Track Id="t1">)_",
R"_(              <ChannelRefs>)_",
R"_(                <ChannelRef Id="DAPI"/>)_",
R"_(                <ChannelRef Id="FITC"/>)_",
R"_(              </ChannelRefs>)_",
R"_(            </Track>)_",
R"_()_",
R"_(            <Track Id="t2">)_",
R"_(              <ChannelRefs>)_",
R"_(                <ChannelRef Id="Rhodamin"/>)_",
R"_(              </ChannelRefs>)_",
R"_(            </Track>)_",
R"_(          </Tracks>)_",
R"_()_",
R"_(          <T>)_",
R"_(            <StartTime>2011-01-28T00:00:00</StartTime>)_",
R"_(            <Positions>)_",
R"_(              <Interval>)_",
R"_(                <Start>0</Start>)_",
R"_(                <Increment>3e-2</Increment>)_",
R"_(              </Interval>)_",
R"_(            </Positions>)_",
R"_(          </T>)_",
R"_()_",
R"_(          <Z>)_",
R"_(            <StartPosition>12.78</StartPosition>)_",
R"_(            <Positions>)_",
R"_(              <List>)_",
R"_(                <Offsets>10 20 30 40 50</Offsets>)_",
R"_(              </List>)_",
R"_(            </Positions>)_",
R"_(          </Z>)_",
R"_()_",
R"_(          <I>)_",
R"_(            <StartPosition>1.0</StartPosition>)_",
R"_(            <Positions>)_",
R"_(              <List>)_",
R"_(                <Offsets>10 20 30 40 50</Offsets>)_",
R"_(              </List>)_",
R"_(            </Positions>)_",
R"_(          </I>)_",
R"_()_",
R"_(          <R>)_",
R"_(            <StartPosition>10.0</StartPosition>)_",
R"_(            <Positions>)_",
R"_(              <List>)_",
R"_(                <Offsets>20 30 40</Offsets>)_",
R"_(              </List>)_",
R"_(            </Positions>)_",
R"_(          </R>)_",
R"_()_",
R"_(        </Dimensions>)_",
R"_()_",
R"_(        <MicroscopeRef Id="Microscope1"/>)_",
R"_(        )_",
R"_(        <TubeLenses>)_",
R"_(          <TubeLensRef Id="TubeLense3"/>)_",
R"_(        </TubeLenses>)_",
R"_()_",
R"_(      </Image>)_",
R"_()_",
R"_(      <User Id="User1">)_",
R"_(        <FirstName>Hugo</FirstName>)_",
R"_(        <LastName>Habicht</LastName>)_",
R"_(        <Email>hugo@habicht.de</Email>)_",
R"_(        <DisplayName>The users display name</DisplayName>)_",
R"_(        <Institution>Carl Zeiss Munich</Institution>)_",
R"_(        <MiddleName>Mc.</MiddleName>)_",
R"_(        <Phone>+4989-234578</Phone>)_",
R"_(      </User>)_",
R"_()_",
R"_(      <Document>)_",
R"_(        <CreationDate>2001-01-28T00:00:00</CreationDate>)_",
R"_(        <Description>This is a sample image by WBa</Description>)_",
R"_(        <Thumbnail>sample.jpg</Thumbnail>)_",
R"_(        <Comment>This is a comment</Comment>)_",
R"_(        <Keywords>one two three</Keywords>)_",
R"_(        <Author>Author1</Author>)_",
R"_(        <Name>Name</Name>)_",
R"_(        <Rating>3</Rating>)_",
R"_(        <SubType>None</SubType>)_",
R"_(        <Title>The title</Title>)_",
R"_(        <UserName>UserName1</UserName>)_",
R"_(      </Document>)_",
R"_()_",
R"_(      <Environment>)_",
R"_(        <Temperature>20.1</Temperature>)_",
R"_(        <AirPressure>700.3</AirPressure>)_",
R"_(        <Humidity>0.5</Humidity>)_",
R"_(        <CO2Percent>0.7</CO2Percent>)_",
R"_(      </Environment>)_",
R"_()_",
R"_(      <Application>)_",
R"_(        <Name>The Name</Name>)_",
R"_(        <CustomAttributes>)_",
R"_(          <dudel/>)_",
R"_(        </CustomAttributes>)_",
R"_(      </Application>)_",
R"_()_",
R"_(      <Institution>)_",
R"_(        <Address>Address</Address>)_",
R"_(        <City>City</City>)_",
R"_(        <Country>Country</Country>)_",
R"_(        <Email>Email</Email>)_",
R"_(        <Fax>Fax</Fax>)_",
R"_(        <Logo>Logo</Logo>)_",
R"_(        <Name>Name</Name>)_",
R"_(        <Phone>Phone</Phone>)_",
R"_(        <ZIP>ZIP</ZIP>)_",
R"_(      </Institution>)_",
R"_()_",
R"_(      <!-- ================================== Instrument ====================================-->)_",
R"_(      <Instrument Id="Instrument:1">)_",
R"_(        <Detectors>)_",
R"_(          <Detector Id="Detector1">)_",
R"_(            <AmplificationGain>10</AmplificationGain>)_",
R"_(            <Gain>2.5</Gain>)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>3</LotNumber>)_",
R"_(              <Model>model1</Model>)_",
R"_(              <SerialNumber>2345678</SerialNumber>)_",
R"_(              <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(            </Manufacturer>)_",
R"_(            <Offset>10</Offset>)_",
R"_(            <Voltage>2.3</Voltage>)_",
R"_(            <Type>Photodiode</Type>)_",
R"_(            <Zoom>3.4</Zoom>)_",
R"_(            <Adapter>)_",
R"_(              <Manufacturer>)_",
R"_(                <Model>1x Kamera Adapter</Model>)_",
R"_(                <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(              </Manufacturer>)_",
R"_(              <Magnification>1</Magnification>)_",
R"_(            </Adapter>)_",
R"_(          </Detector>)_",
R"_(          <Detector Id="Detector2">)_",
R"_(            <AmplificationGain>20</AmplificationGain>)_",
R"_(            <Gain>3.5</Gain>)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>3</LotNumber>)_",
R"_(              <Model>model2</Model>)_",
R"_(              <SerialNumber>234567890</SerialNumber>)_",
R"_(              <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(            </Manufacturer>)_",
R"_(            <Offset>20</Offset>)_",
R"_(            <Voltage>2.3</Voltage>)_",
R"_(            <Type>Photodiode</Type>)_",
R"_(            <Zoom>3.4</Zoom>)_",
R"_(            <Adapter>)_",
R"_(              <Manufacturer>)_",
R"_(                <Model>0.63x Kamera Adapter</Model>)_",
R"_(                <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(              </Manufacturer>)_",
R"_(              <Magnification>0.63</Magnification>)_",
R"_(            </Adapter>)_",
R"_(          </Detector>)_",
R"_(        </Detectors>)_",
R"_()_",
R"_(        <Dichroics>)_",
R"_(          <Dichroic Id="1">)_",
R"_(            <Manufacturer>)_",
R"_(              <Model>Dic1</Model>)_",
R"_(            </Manufacturer>)_",
R"_(            <Wavelengths>)_",
R"_(              <Wavelength>460</Wavelength>)_",
R"_(              <Wavelength>520</Wavelength>)_",
R"_(            </Wavelengths>)_",
R"_(          </Dichroic>)_",
R"_()_",
R"_(          <Dichroic Id="2">)_",
R"_(          <Wavelengths>)_",
R"_(            <Wavelength>585</Wavelength>)_",
R"_(          </Wavelengths>)_",
R"_(          </Dichroic>)_",
R"_(            )_",
R"_(        </Dichroics>)_",
R"_()_",
R"_(        <Filters>)_",
R"_(          <Filter Id="1">)_",
R"_(            <FilterWheel>3</FilterWheel>)_",
R"_(            <TransmittanceRange Id="1">)_",
R"_(              <CutIn>450</CutIn>)_",
R"_(              <CutInTolerance>3</CutInTolerance>)_",
R"_(              <CutOut>480</CutOut>)_",
R"_(              <CutOutTolerance>10</CutOutTolerance>)_",
R"_(              <Transmittance>0.9</Transmittance>)_",
R"_(            </TransmittanceRange>)_",
R"_(          </Filter>)_",
R"_()_",
R"_(          <Filter Id="Filter:2">)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>1</LotNumber>)_",
R"_(            </Manufacturer>)_",
R"_(            <TransmittanceRange Id="2">)_",
R"_(              <CutIn>550</CutIn>)_",
R"_(              <CutInTolerance>3</CutInTolerance>)_",
R"_(              <CutOut>580</CutOut>)_",
R"_(              <CutOutTolerance>10</CutOutTolerance>)_",
R"_(              <Transmittance>0.9</Transmittance>)_",
R"_(            </TransmittanceRange>)_",
R"_(          </Filter>)_",
R"_()_",
R"_(          <Filter Id="Filter:3">)_",
R"_(            <FilterWheel>3</FilterWheel>)_",
R"_(            <TransmittanceRange Id="3">)_",
R"_(              <CutIn>590</CutIn>)_",
R"_(              <CutOut>650</CutOut>)_",
R"_(            </TransmittanceRange>)_",
R"_(          </Filter>)_",
R"_()_",
R"_(        </Filters>)_",
R"_()_",
R"_(        <FilterSets>)_",
R"_(          <FilterSet Id="FilterSet:AlexaFluor568" Name="Alexa Fluor 568">)_",
R"_(            <EmissionFilters>)_",
R"_(              <EmissionFilterRef Id="Filter:2"/>)_",
R"_(            </EmissionFilters>)_",
R"_(            <ExcitationFilters>)_",
R"_(              <ExcitationFilterRef Id="Filter:3"/>)_",
R"_(            </ExcitationFilters>)_",
R"_(            <DichroicRef Id="2"/>)_",
R"_(          </FilterSet>)_",
R"_(        </FilterSets>)_",
R"_()_",
R"_(        <LightSources>)_",
R"_(          <LightSource Id="LightSource1">)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>4</LotNumber>)_",
R"_(            </Manufacturer>)_",
R"_(            <Power>3.56</Power>)_",
R"_(            <LightSourceType>)_",
R"_(              <Arc>)_",
R"_(                <Type>Hg</Type>)_",
R"_(              </Arc>)_",
R"_(            </LightSourceType>)_",
R"_(          </LightSource>)_",
R"_()_",
R"_(          <LightSource Id="LightSource2">)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>4</LotNumber>)_",
R"_(            </Manufacturer>)_",
R"_(            <Power>3.56</Power>)_",
R"_(            <LightSourceType>)_",
R"_(              <Filament>)_",
R"_(                <Type>Halogen</Type>)_",
R"_(              </Filament>)_",
R"_(            </LightSourceType>)_",
R"_(          </LightSource>)_",
R"_()_",
R"_(          <LightSource Id="LightSource3">)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>4</LotNumber>)_",
R"_(            </Manufacturer>)_",
R"_(            <Power>3.56</Power>)_",
R"_(            <LightSourceType>)_",
R"_(              <Laser>)_",
R"_(                <Type>Gas</Type>)_",
R"_(                <LaserMedium>H2O</LaserMedium>)_",
R"_(                <Wavelength>488</Wavelength>)_",
R"_(                <PockelCell>1</PockelCell>)_",
R"_(              </Laser>)_",
R"_(            </LightSourceType>)_",
R"_(          </LightSource>)_",
R"_()_",
R"_(          <LightSource Id="LightSource4">)_",
R"_(            <Power>3.56</Power>)_",
R"_(            <LightSourceType>)_",
R"_(              <LightEmittingDiode/>)_",
R"_(            </LightSourceType>)_",
R"_(            <Manufacturer>)_",
R"_(              <SerialNumber>223456</SerialNumber>)_",
R"_(              <LotNumber>10</LotNumber>)_",
R"_(              <Manufacturer>Carl Zeiss AG</Manufacturer>)_",
R"_(              <Model>5.5</Model>)_",
R"_(            </Manufacturer>)_",
R"_(          </LightSource>)_",
R"_()_",
R"_(        </LightSources>)_",
R"_()_",
R"_(        <Microscopes>)_",
R"_(          <Microscope Id="Microscope1">)_",
R"_(            <Type>Upright</Type>)_",
R"_(            <Manufacturer>)_",
R"_(              <Manufacturer>Carl Zeiss AG</Manufacturer>)_",
R"_(              <Model>AxioPlan</Model>)_",
R"_(              <SerialNumber>345678</SerialNumber>)_",
R"_(            </Manufacturer>)_",
R"_(          </Microscope>)_",
R"_(        </Microscopes>)_",
R"_(        )_",
R"_(        <Objectives>)_",
R"_(          <Objective Id="Objective:1">)_",
R"_(            <Manufacturer>)_",
R"_(              <Manufacturer>ACME</Manufacturer>)_",
R"_(              <Model>GreatObjective</Model>)_",
R"_(              <SerialNumber>123456</SerialNumber>)_",
R"_(              <LotNumber>3</LotNumber>)_",
R"_(            </Manufacturer>)_",
R"_(            <Immersion>Oil</Immersion>)_",
R"_(            <ImmersionRefractiveIndex>1.517</ImmersionRefractiveIndex>)_",
R"_(            <LensNA>1.95</LensNA>)_",
R"_(            <NominalMagnification>40</NominalMagnification>)_",
R"_(            <WorkingDistance>105.534</WorkingDistance>)_",
R"_(            <PupilGeometry>Other</PupilGeometry>)_",
R"_(            <CalibratedMagnification>39.887</CalibratedMagnification>)_",
R"_(            <Correction>PlanApo</Correction>)_",
R"_(            <Iris>1</Iris>)_",
R"_(          </Objective>)_",
R"_()_",
R"_(          <Objective Id="Objective:2">)_",
R"_(            <NominalMagnification>100</NominalMagnification>)_",
R"_(            <CalibratedMagnification>99.887</CalibratedMagnification>)_",
R"_(            <Correction>PlanApo</Correction>)_",
R"_(            <Immersion>Oil</Immersion>)_",
R"_(            <Iris>1</Iris>)_",
R"_(            <WorkingDistance>30.2</WorkingDistance>)_",
R"_(            <LensNA>0.3</LensNA>)_",
R"_(            <Manufacturer>)_",
R"_(              <LotNumber>3</LotNumber>)_",
R"_(              <Model>Apoplan1</Model>)_",
R"_(              <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(              <SerialNumber>345678</SerialNumber>)_",
R"_(            </Manufacturer>)_",
R"_(          </Objective>)_",
R"_()_",
R"_(        <Objective Id="Objective:3">)_",
R"_(          <NominalMagnification>10</NominalMagnification>)_",
R"_(          <CalibratedMagnification>20.123</CalibratedMagnification>)_",
R"_(          <Correction>PlanApo</Correction>)_",
R"_(          <Immersion>Oil</Immersion>)_",
R"_(          <WorkingDistance>30.2</WorkingDistance>)_",
R"_(          <LensNA>0.3</LensNA>)_",
R"_(          <Manufacturer>)_",
R"_(            <LotNumber>3</LotNumber>)_",
R"_(            <Model>Apoplan1</Model>)_",
R"_(            <Manufacturer>Carl Zeiss</Manufacturer>)_",
R"_(            <SerialNumber>345678</SerialNumber>)_",
R"_(          </Manufacturer>)_",
R"_(        </Objective>)_",
R"_(        </Objectives>)_",
R"_()_",
R"_(        <!--<LightsheetCondensers>)_",
R"_(          <LightsheetCondenser Id="LightsheetCondenser:1">)_",
R"_(            <LightsheetNA>1.3</LightsheetNA>)_",
R"_(            <PupilGeometry>1.2</PupilGeometry>)_",
R"_(          </LightsheetCondenser>)_",
R"_(        </LightsheetCondensers>-->)_",
R"_()_",
R"_(        <TubeLenses>)_",
R"_(          <TubeLens Id="TubeLens:1">)_",
R"_(            <Magnification>1</Magnification>)_",
R"_(            <Type>TubeLens</Type>)_",
R"_(          </TubeLens>)_",
R"_(          <TubeLens Id="TubeLens:2">)_",
R"_(            <Magnification>1.25</Magnification>)_",
R"_(            <Type>TubeLens</Type>)_",
R"_(          </TubeLens>)_",
R"_(          <TubeLens Id="TubeLens:3">)_",
R"_(            <Magnification>1.6</Magnification>)_",
R"_(            <Type>TubeLens</Type>)_",
R"_(          </TubeLens>)_",
R"_(          <TubeLens Id="TubeLens:4">)_",
R"_(            <Magnification>2.5</Magnification>)_",
R"_(            <Type>BertrandLens</Type>)_",
R"_(          </TubeLens>)_",
R"_(        </TubeLenses>)_",
R"_(        )_",
R"_(      </Instrument>)_",
R"_()_",
R"_(      <Processing>)_",
R"_(        <DFT>)_",
R"_(          <BlockShift>true</BlockShift>)_",
R"_(          <ImageType>Basic</ImageType>)_",
R"_(          <ProcessingDimension>2</ProcessingDimension>)_",
R"_(          <RefPixelType>3</RefPixelType>)_",
R"_(          <RefWidth>10</RefWidth>)_",
R"_(          <ThirdDimension>4</ThirdDimension>)_",
R"_(          <WindowMode>4</WindowMode>)_",
R"_(        </DFT>)_",
R"_()_",
R"_(        <PSF>)_",
R"_(          <AnticipatedPolarHeight>10</AnticipatedPolarHeight>)_",
R"_(          <AxialResolution>3.5</AxialResolution>)_",
R"_(          <Defocus>10</Defocus>)_",
R"_(          <DesignCoverglassThickness>1e-3</DesignCoverglassThickness>)_",
R"_(          <AnticipatedPolarWidth>3</AnticipatedPolarWidth>)_",
R"_(          <Dimension>3</Dimension>)_",
R"_(          <EmbeddingIndex>4</EmbeddingIndex>)_",
R"_(          <Illumination>3</Illumination>)_",
R"_(          <IlluminationWavelength>300</IlluminationWavelength>)_",
R"_(          <PinholeGeometry>4</PinholeGeometry>)_",
R"_(          <DesignCoverglassIndex>3</DesignCoverglassIndex>)_",
R"_(          <EmbeddingThickness>4</EmbeddingThickness>)_",
R"_(          <DetectionWavelength>400</DetectionWavelength>)_",
R"_(          <DesignImmersionIndex>4</DesignImmersionIndex>)_",
R"_(          <ImageFormation>4</ImageFormation>)_",
R"_(          <LateralMagnification>3</LateralMagnification>)_",
R"_(          <NACondenser>0.4</NACondenser>)_",
R"_(          <NAObjective>2.5</NAObjective>)_",
R"_(          <DisplayMode>3</DisplayMode>)_",
R"_(          <Optovar>1</Optovar>)_",
R"_(          <PinholeSize>3</PinholeSize>)_",
R"_(          <PupilGeometry>3</PupilGeometry>)_",
R"_(          <TestObjectGeometry>3</TestObjectGeometry>)_",
R"_(          <TestObjectSize>3</TestObjectSize>)_",
R"_(          <TransverseResolution>4</TransverseResolution>)_",
R"_(          <UsedCoverglassIndex>3</UsedCoverglassIndex>)_",
R"_(          <UsedCoverglassThickness>3</UsedCoverglassThickness>)_",
R"_(          <Instrument>3</Instrument>)_",
R"_(          <UsedImmersionIndex>3</UsedImmersionIndex>)_",
R"_(          <Type>3</Type>)_",
R"_(          <Source>3</Source>)_",
R"_(          <WorkingDistance>3.4</WorkingDistance>)_",
R"_(          <ZStackDirection>3</ZStackDirection>)_",
R"_(        </PSF>)_",
R"_()_",
R"_(        <SIM>)_",
R"_(          <Epsilon>0.3</Epsilon>)_",
R"_(          <FilterHarmonics>3</FilterHarmonics>)_",
R"_(          <FilterName>filter1</FilterName>)_",
R"_(          <FilterSize>10</FilterSize>)_",
R"_(          <HardwareVersion>3</HardwareVersion>)_",
R"_(          <GratingPeriod>10</GratingPeriod>)_",
R"_(          <Normalize>true</Normalize>)_",
R"_(          <PhaseAngles>3</PhaseAngles>)_",
R"_(          <ProcessingMode>4</ProcessingMode>)_",
R"_(          <SimCorrectionMode>3</SimCorrectionMode>)_",
R"_(        </SIM>)_",
R"_()_",
R"_(        <Unmixing>)_",
R"_(          <BackgroundSpectrum>3</BackgroundSpectrum>)_",
R"_(          <Autoscale>true</Autoscale>)_",
R"_(          <CalculateResiduals>false</CalculateResiduals>)_",
R"_(          <ConfidenceIntervals>true</ConfidenceIntervals>)_",
R"_(          <IgnoreNegativeResults>true</IgnoreNegativeResults>)_",
R"_(          <Weighted>true</Weighted>)_",
R"_(          <Spectra>)_",
R"_(            <Spectrum>)_",
R"_(              <ComponentName>30</ComponentName>)_",
R"_(              <MainBeamSplitter>3</MainBeamSplitter>)_",
R"_(              <Objective>456</Objective>)_",
R"_(              <Excitations>)_",
R"_(                <ExcitationWavelength>278</ExcitationWavelength>)_",
R"_(                <ExcitationWavelength>311</ExcitationWavelength>)_",
R"_(              </Excitations>)_",
R"_(              <SpectralData>)_",
R"_(                <SpectralChannel>)_",
R"_(                  <Intensity>67</Intensity>)_",
R"_(                  <StandardDeviation>4</StandardDeviation>)_",
R"_(                  <Wavelength>100</Wavelength>)_",
R"_(                </SpectralChannel>)_",
R"_(              </SpectralData>)_",
R"_(            </Spectrum>)_",
R"_(            <Spectrum>)_",
R"_(              <ComponentName>30</ComponentName>)_",
R"_(              <MainBeamSplitter>3</MainBeamSplitter>)_",
R"_(              <Objective>456</Objective>)_",
R"_(              <Excitations>)_",
R"_(                <ExcitationWavelength>278</ExcitationWavelength>)_",
R"_(                <ExcitationWavelength>311</ExcitationWavelength>)_",
R"_(              </Excitations>)_",
R"_(              <SpectralData>)_",
R"_(                <SpectralChannel>)_",
R"_(                  <Intensity>67</Intensity>)_",
R"_(                  <StandardDeviation>4</StandardDeviation>)_",
R"_(                  <Wavelength>100</Wavelength>)_",
R"_(                </SpectralChannel>)_",
R"_(              </SpectralData>)_",
R"_(            </Spectrum>)_",
R"_(          </Spectra>)_",
R"_(        </Unmixing>)_",
R"_()_",
R"_(        <StructuredIllumination>)_",
R"_(          <Channels>)_",
R"_(            <Channel>)_",
R"_(              <ExcitationWavelength>200</ExcitationWavelength>)_",
R"_(              <GratingPeriod>3</GratingPeriod>)_",
R"_(              <GratingPixels>3</GratingPixels>)_",
R"_(              <DiffractionEfficiency1>1.0</DiffractionEfficiency1>)_",
R"_(              <DiffractionEfficiency2>1.0</DiffractionEfficiency2>)_",
R"_(              <DiffractionEfficiency3>1.0</DiffractionEfficiency3>)_",
R"_(            </Channel>)_",
R"_(          </Channels>)_",
R"_(          <DimensionC>3</DimensionC>)_",
R"_(          <DimensionH>3</DimensionH>)_",
R"_(          <DimensionR>3</DimensionR>)_",
R"_(          <Phases>3</Phases>)_",
R"_(          <Rotations>4</Rotations>)_",
R"_(        </StructuredIllumination>)_",
R"_(        )_",
R"_(      </Processing>)_",
R"_()_",
R"_(    </Information>)_",
R"_()_",
R"_(  </Metadata>)_",
R"_(  )_",
R"_(</ImageDocument>)_"
};
// editorconfig-checker-enable

// editorconfig-checker-disable
static const char* test6Metadata[] =
{
R"(<ImageDocument>)",
R"(  <Metadata>)",
R"(    <Information>)",
R"(      <Image>)",
R"(        <SizeX>512</SizeX>)",
R"(        <SizeY>512</SizeY>)",
R"(        <SizeC>1</SizeC>)",
R"(        <SizeH>32</SizeH>)",
R"()",
R"(        <PixelType>Gray8</PixelType>)",
R"()",
R"(        <Dimensions>)",
R"()",
R"(          <Channels>)",
R"(            <Channel Id="Channel:1">)",
R"(              <ChannelType>AiryScanRaw</ChannelType>)",
R"(              <AcquisitionMode>LaserScanningConfocalMicroscopy</AcquisitionMode>)",
R"()",
R"(              <AiryscanSettings>)",
R"(                <Mode>FastAiryScan</Mode>)",
R"(                <FastAiryScanSettings>)",
R"(                  <IlluminationLinePixels>0 4</IlluminationLinePixels>)",
R"(                  <IlluminationLine>0 3.975</IlluminationLine>)",
R"(                  <FiberMapping>1 2 3 4 5 6 7 8 10 22  </FiberMapping>)",
R"(                  <EffectiveNA_in_x>0.43</EffectiveNA_in_x>)",
R"(                  <EffectiveNA_in_y>0.38</EffectiveNA_in_y>)",
R"(                </FastAiryScanSettings>)",
R"(              </AiryscanSettings>)",
R"()",
R"(            </Channel>)",
R"()",
R"(          </Channels>)",
R"(        </Dimensions>)",
R"()",
R"(      </Image>)",
R"(   )",
R"(    </Information>)",
R"(  </Metadata>)",
R"(</ImageDocument>)"
};
// editorconfig-checker-enable

static string GetTestData()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(testMetadata) / sizeof(testMetadata[0]); ++i)
    {
        str += testMetadata[i];
    }

    return str;
}

static string GetTest2Data()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(test2Metadata) / sizeof(test2Metadata[0]); ++i)
    {
        str += test2Metadata[i];
    }

    return str;
}

static string GetTest3Data()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(test3Metadata) / sizeof(test3Metadata[0]); ++i)
    {
        str += test3Metadata[i];
    }

    return str;
}

static string GetTest4Data()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(test4Metadata) / sizeof(test4Metadata[0]); ++i)
    {
        str += test4Metadata[i];
    }

    return str;
}

static string GetTest5Data()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(test5Metadata) / sizeof(test5Metadata[0]); ++i)
    {
        str += test5Metadata[i];
    }

    return str;
}

static string GetTest6Data()
{
    // the size of static strings seems to be limited (with various compilers), so the easiest workaround was to
    // have a lot of short strings and put them together here...
    string str;
    for (int i = 0; i < sizeof(test6Metadata) / sizeof(test6Metadata[0]); ++i)
    {
        str += test6Metadata[i];
    }

    return str;
}

static string GetTestDataInvalid()
{
    // here we leave out the last line, resulting in invalid XML
    string str;
    for (int i = 0; i < sizeof(testMetadata) / sizeof(testMetadata[0]) - 1; ++i)
    {
        str += testMetadata[i];
    }

    return str;
}

MockMetadataSegment::MockMetadataSegment(Type type/*= Type::Data1*/)
{
    switch (type)
    {
    case Type::Data1:
        this->xmlData = GetTestData();
        break;
    case Type::Data2:
        this->xmlData = GetTest2Data();
        break;
    case Type::Data3:
        this->xmlData = GetTest3Data();
        break;
    case Type::Data4:
        this->xmlData = GetTest4Data();
        break;
    case Type::Data5:
        this->xmlData = GetTest5Data();
        break;
    case Type::Data6:
        this->xmlData = GetTest6Data();
        break;
    case Type::InvalidData:
        this->xmlData = GetTestDataInvalid();
        break;
    }
}

/*virtual*/ std::shared_ptr<const void> MockMetadataSegment::GetRawData(MemBlkType type, size_t* ptrSize)
{
    throw  std::logic_error("Function not implemented.");
}

/*virtual*/void MockMetadataSegment::DangerousGetRawData(MemBlkType type, const void*& ptr, size_t& size) const
{
    switch (type)
    {
    case XmlMetadata:
        ptr = this->xmlData.c_str();
        size = this->xmlData.length();
        break;
    case Attachment:
        ptr = nullptr;
        size = 0;
        break;
    default:
        throw std::logic_error("illegal value for type");
    }
}
